(* ******************************************************************** *)
(*
 * Project         : HOL-CSP - A Shallow Embedding of CSP in  Isabelle/HOL
 * Version         : 2.0
 *
 * Author          : Burkhart Wolff
 *                   (Based on HOL-CSP 1.0 by Haykal Tej and Burkhart Wolff)
 *
 * This file       : A Reference Example
 *
 * Copyright (c) 2009 Universit Paris-Sud, France
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 *
 *     * Neither the name of the copyright holders nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************)
(* $Id: ROOT.ML 8591 2009-09-26 09:58:14Z brucker $ *)


header{* Refinement Example with Buffer over infinite Alphabet*}

theory     CopyBuffer
imports    CSP
begin 

section{* Defining the Copy-Buffer Example *}

datatype 'a channel = left 'a | right 'a | mid 'a | ack

constdefs SYN  :: "('a channel) set"
where    "SYN  \<equiv> (range mid) \<union> {ack}"

constdefs COPY :: "('a channel) process"
where    "COPY \<equiv> (\<mu> COPY. left`?`x \<rightarrow> right`!`x \<rightarrow> COPY)"

constdefs SEND :: "('a channel) process"
where    "SEND \<equiv> (\<mu> SEND. left`?`x \<rightarrow> mid`!`x \<rightarrow> ack \<rightarrow> SEND)"

constdefs REC  :: "('a channel) process"
where    "REC  \<equiv> (\<mu> REC. mid`?`x \<rightarrow> right`!`x \<rightarrow> ack \<rightarrow> REC)"


constdefs
  SYSTEM :: "('a channel) process"
 "SYSTEM \<equiv> ((SEND \<lbrakk> SYN \<rbrakk> REC) \\ SYN)"



(* Was originally in old theory presented as follows:

consts
  SYSTEM :: "('a channel) process"
  COPY   :: "('a channel) process"
  SYN    :: "('a channel) set"

defs

  COPY_def   " COPY   ==(letrec COPY = left`?`x `->` right`!`x `->` COPY
                         in     COPY)"

  SYN_def    " SYN    == (range mid) Un {ack}"

  SYSTEM_def " SYSTEM ==(letrec
                                SEND = left`?`x`->`mid`!`x`->`ack`->`SEND;
                                REC  = mid`?`x`->`right`!`x`->` ack`->`REC
                         in     ((SEND `[|` SYN `|]` REC) `\\` SYN))"
*)


section{* The Standard Proof *}

end

