(* ******************************************************************** *)
(*
        Project         : HOL-CSP - A Shallow Embedding of CSP 
                          in Isabelle/HOL

        $Source: /home/uniform/rep/isa_lang/CSP/univ_csp/Par_Int.thy,v $
        Author          : Haykal Tej, Burkhart Wolff
                          (Last modification by $Author: bu $)

        Affiliation     : (1)University of Bremen,
                             Bremen Institute for Safe Systems (BISS)
                          (2)Albert-Ludwigs Universitt Freiburg

        Level           : 2 (operator definitions & proof of invariance)
        This file       : parallel interleave operator

        $Date: 2000/09/19 18:17:22 $
        $Revision: 1.11 $
*)
(* ******************************************************************** *)


theory  Sync
imports Process (* Legacy *)
begin


consts setinterleaving ::"'a trace \<times> ('a event) set \<times> 'a trace \<Rightarrow> ('a trace)set 
       " (*(3_ `[|`_`|]`/ _)"*)

recdef setinterleaving "measure(\<lambda>(l1, s, l2). size l1 + size l2)"

si_empty1: "setinterleaving([], X, []) = {[]}"
si_empty2: "setinterleaving([], X, (y # t)) = 
               (if (y \<in> X) 
                then {} 
                else {z.\<exists> u. z = (y # u) \<and> u \<in> setinterleaving ([], X, t)})"
si_empty3: "setinterleaving((x # s), X, []) = 
               (if (x \<in> X) 
                then {} 
                else {z.\<exists> u. z = (x # u) \<and> u \<in> setinterleaving (s, X, [])})"
si_neq   : "setinterleaving((x # s), X, (y # t)) = 
               (if (x \<in> X) 
                then if (y \<in> X) 
                     then if (x = y) 
                          then {z.\<exists> u. z = (x#u) \<and> u \<in> setinterleaving(s, X, t)} 
                          else {}
                     else {z.\<exists> u. z = (y#u) \<and> u \<in> setinterleaving ((x#s), X, t)}
                else if (y \<notin> X) 
                     then {z.\<exists> u. z = (x # u) \<and> u \<in> setinterleaving (s, X, (y # t))} 
                          \<union> {z.\<exists> u. z = (y # u) \<and> u \<in> setinterleaving((x # s), X, t)} 
                     else {z.\<exists> u. z = (x # u) \<and> u \<in> setinterleaving (s, X, (y # t))})"


lemma sym1 [simp]: "setinterleaving([], X, t) = setinterleaving(t, X ,[])"
  by (induct t, simp_all)


lemma sym2 [simp]: "
      \<forall> s. setinterleaving (s, X, t) = setinterleaving (t, X, s)
      \<longrightarrow> setinterleaving (a # s, X, t) = setinterleaving (t, X, a # s)"
  apply (induct t)
  apply (simp_all)
  apply auto
  apply (case_tac "t",simp)
sorry


lemma sym [simp] : "setinterleaving(s, X, t)= setinterleaving(t, X, s)"
  by (induct s, simp_all)


consts setinterleaves :: "['a trace, ('a trace\<times>'a trace)\<times>('a event) set] \<Rightarrow> bool"    
                         (infixl "setinterleaves" 70)
translations
 "u setinterleaves ((s, t), X)" == "(u \<in> setinterleaving(s, X, t))"


definition sync :: "['a process,'a set,'a process] => 'a process" 
           ("(3_ \<lbrakk>_ \<rbrakk>/ _)" [14,0,15] 14)  
where
 "P \<lbrakk> A \<rbrakk> Q == 
       Abs_Process({(s,R).\<exists> t u X Y. (t,X) \<in> F P \<and> (u,Y) \<in> F Q \<and> 
                                     s setinterleaves ((t,u),(ev`A) \<union> {tick}) \<and> 
                                     R = (X \<union> Y) \<inter> ((ev`A) \<union> {tick}) \<union> X \<inter> Y} \<union> 
                   {(s,R).\<exists> t u r v. front_tickFree v \<and> (tickFree r \<or> v=[]) \<and> 
                                     s = r@v \<and> 
                                     r setinterleaves ((t,u),(ev`A) \<union> {tick}) \<and> 
                                     (t \<in> D P \<and> u \<in> T Q \<or> t \<in> D Q \<and> u \<in> T P)},
                   {s.    \<exists> t u r v. front_tickFree v \<and> (tickFree r \<or> v=[]) \<and> 
                                     s = r@v \<and> 
                                     r setinterleaves ((t,u),(ev`A) \<union> {tick}) \<and> 
                                     (t \<in> D P \<and> u \<in> T Q \<or> t \<in> D Q \<and> u \<in> T P)})"


axioms
  F_sync    : "F(P \<lbrakk> A \<rbrakk> Q) = 
                   {(s,R).\<exists> t u X Y. (t,X) \<in> F P \<and> 
                                       (u,Y) \<in> F Q \<and> 
                                       s setinterleaves ((t,u),(ev`A) \<union> {tick}) \<and> 
                                       R = (X \<union> Y) \<inter> ((ev`A) \<union> {tick}) \<union> X \<inter> Y} \<union> 
                   {(s,R).\<exists> t u r v. front_tickFree v \<and> 
                                       (tickFree r \<or> v=[]) \<and> 
                                       s = r@v \<and> 
                                       r setinterleaves ((t,u),(ev`A)\<union>{tick}) \<and> 
                                       (t \<in> D P \<and> u \<in> T Q \<or> t \<in> D Q \<and> u \<in> T P)}"

  D_sync    : "D(P \<lbrakk> A \<rbrakk> Q) = 
                   {s.    \<exists> t u r v. front_tickFree v \<and> (tickFree r \<or> v=[]) \<and> 
                            s = r@v \<and> r setinterleaves ((t,u),(ev`A) \<union> {tick}) \<and> 
                            (t \<in> D P \<and> u \<in> T Q \<or> t \<in> D Q \<and> u \<in> T P)}"

  T_sync    : "T(P \<lbrakk> A \<rbrakk> Q) = 
                   {s.    \<forall> t u. t \<in> T P \<and> u \<in> T Q \<and> 
                                 s setinterleaves ((t,u),(ev`A) \<union> {tick})}"


end
