(*****************************************************************************
 * HOL-TestGen --- theorem-prover based test case generation
 *                 http://www.brucker.ch/projects/hol-testgen/
 *                                                                            
 * FWCompilationExample.thy --- 
 * This file is part of HOL-TestGen.
 *
 * Copyright (c) 2005-2010 ETH Zurich, Switzerland
 *               2008-2010 Achim D. Brucker, Germany
 *               2009-2010 Univerisite Paris-Sud, France 
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 *
 *     * Neither the name of the copyright holders nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************)
(* $Id: FWCompilationExample.thy 8835 2010-03-15 22:48:23Z brucker $ *)

theory FWCompilationExample
imports FWTesting
begin



text{*
To test our definitions, we define an example network and policy
*}


constdefs 
 one :: "IntegerPort net"
 "one \<equiv> {{(a,b). a = 1}}"
 two :: "IntegerPort net"
 "two \<equiv> {{(a,b). a = 2}}"
 three :: "IntegerPort net"
 "three \<equiv> {{(a,b). a = 3}}"
 four :: "IntegerPort net"
 "four \<equiv> {{(a,b). a = 4}}"
 five :: "IntegerPort net"
 "five \<equiv> {{(a,b). a = 5}}"
 six :: "IntegerPort net"
 "six \<equiv> {{(a,b). a = 6}}"


constdefs ex :: "(IntegerPort net, port) Combinators"
"ex \<equiv> AllowPortFromTo  three one 144\<oplus>
DenyAll \<oplus> AllowPortFromTo one two 62  \<oplus>
AllowPortFromTo  three four 564\<oplus>
AllowPortFromTo one three 82 \<oplus>
AllowPortFromTo two four 33  \<oplus>
AllowPortFromTo three two 43  \<oplus>
DenyAllFromTo two four \<oplus> 
AllowPortFromTo  three four 5\<oplus>
AllowPortFromTo  two four 37\<oplus>
AllowPortFromTo  four one 34\<oplus>
AllowPortFromTo  one four 3\<oplus>
AllowPortFromTo  four three 34\<oplus>
AllowPortFromTo  two three 234\<oplus>
AllowPortFromTo  three two 35\<oplus>
DenyAllFromTo four one \<oplus>
AllowPortFromTo  one two 343"


text{*
Some auxiliary lemmas
*}

lemma sets_distinct1: "(n::int) \<noteq> m \<Longrightarrow> {(a,b). a = n} \<noteq> {(a,b). a = m}"
apply auto
done

lemma sets_distinct2: "(m::int) \<noteq> n \<Longrightarrow> {(a,b). a = n} \<noteq> {(a,b). a = m}"
apply auto
done


lemma nets_different: "one \<noteq> two \<and> one \<noteq> three \<and> one \<noteq> four  \<and> two \<noteq> three \<and> two \<noteq> four \<and> three \<noteq> four \<and> 
                       two \<noteq> one \<and> three \<noteq> one \<and> four \<noteq> one  \<and> three \<noteq> two \<and> four \<noteq> two \<and> four \<noteq> three"
apply (simp add: one_def two_def three_def four_def)
apply (simp add: sets_distinct1 sets_distinct2)
done

lemma aux: "\<lbrakk>x \<noteq> a; y\<noteq>b; (x \<noteq> y \<and> x \<noteq> b) \<or>  (a \<noteq> b \<and> a \<noteq> y)\<rbrakk> \<Longrightarrow> {x,a} \<noteq> {y,b}"
  apply auto
done


lemma aux2: "{a,b} = {b,a}"
  apply auto
done


lemma noMT: "\<forall> x \<in> set (policy2list ex). dom (C x) \<noteq> {}"
apply (simp add: ex_def)
apply (simp add: PLemmas one_def two_def three_def four_def IntegerPort.src_port)
done



lemma ANDex: "allNetsDistinct (policy2list ex)"
apply (simp add: ex_def allNetsDistinct_def nets_different)
apply (auto simp: PLemmas one_def two_def three_def four_def netsDistinct_def IntegerPort.src_port)
done


lemma count_the_rules: "(int (length(policy2list (list2policy(normalize ex)))) = post) \<and> (int(length (policy2list ex)) = pre) \<and>
 (int (length((normalize ex))) = Partitions)"
apply (insert nets_different noMT)
apply (simp add:  normalize_def ex_def bothNets_def aux aux2 Nets_List_def, thin_tac "?X",thin_tac "?S")
oops



lemma all_in_list: "all_in_list (policy2list ex) (Nets_List ex)"
apply (simp add: ex_def)
apply (unfold Nets_List_def)
apply (unfold bothNets_def)
apply (rule conjI)
apply (insert nets_different)
apply simp
apply (rule disjI1)
apply (rule aux2)
apply (simp add: aux2)
done



lemma C_ex_compile: " 
C (list2policy (normalize ex)) =  C ex"
apply (unfold normalize_def)
apply (rule C_eq_compile)
apply (simp add: ex_def)
apply (rule all_in_list)
apply (rule ANDex)
done




(*
test_spec "FUT x = C (ex) x"
apply (prepareFWthm)
apply (simp add: ex_def)
apply (simp add: C.simps)
apply (insert nets_different)
apply (gen_test_cases "FUT")
apply simp_all
apply (tactic{* TRYALL (dresolve_tac  [sym] )*})
apply (simp_all)
store_test_thm "foo0"
gen_test_data "foo0"
*)


test_spec "makeFUT FUT ex x 0"
apply (simp add: makeFUT_def)
apply (rule_tac x=x in spec, rule allI, simp only: split_tupled_all, rule impI)
apply (insert nets_different noMT)
apply (simp add:  normalize_def ex_def bothNets_def aux aux2 Nets_List_def)
apply (rotate_tac -1, thin_tac "?x")
apply (rotate_tac -1, thin_tac "?x")
apply (simp add: packet_Nets_def)
apply (simp add: PLemmas one_def two_def three_def four_def five_def six_def IntegerPort.src_port)
apply (gen_test_cases "FUT")
apply simp_all
store_test_thm "foo0"
gen_test_data "foo0"

thm foo0.test_data



test_spec "makeFUT FUT ex x 1"
apply (simp add: makeFUT_def)
apply (rule_tac x=x in spec, rule allI, simp only: split_tupled_all, rule impI)

apply (insert nets_different noMT)
apply (simp add:  normalize_def ex_def bothNets_def aux aux2 Nets_List_def)
apply (rotate_tac -1, thin_tac "?x")
apply (rotate_tac -1, thin_tac "?x")
apply (simp add: packet_Nets_def)
apply (simp add: PLemmas one_def two_def three_def four_def five_def six_def IntegerPort.src_port)
apply (gen_test_cases "FUT")
apply simp_all
store_test_thm "foo1"
gen_test_data "foo1"

thm foo1.test_data


test_spec "makeFUT FUT ex x 2"
apply (simp add: makeFUT_def)
apply (rule_tac x=x in spec, rule allI, simp only: split_tupled_all, rule impI)

apply (insert nets_different noMT)
apply (simp add:  normalize_def ex_def bothNets_def aux aux2 Nets_List_def)
apply (rotate_tac -1, thin_tac "?x")
apply (rotate_tac -1, thin_tac "?x")
apply (simp add: packet_Nets_def)
apply (simp add: PLemmas one_def two_def three_def four_def five_def six_def IntegerPort.src_port)
apply (gen_test_cases "FUT")
apply simp_all
store_test_thm "foo2"

gen_test_data "foo2"

thm foo2.test_data




test_spec "makeFUT FUT ex x 3"
apply (simp add: makeFUT_def)
apply (rule_tac x=x in spec, rule allI, simp only: split_tupled_all, rule impI)

apply (insert nets_different noMT)
apply (simp add:  normalize_def ex_def bothNets_def aux aux2 Nets_List_def)
apply (rotate_tac -1, thin_tac "?x")
apply (rotate_tac -1, thin_tac "?x")
apply (simp add: packet_Nets_def)
apply (simp add: PLemmas one_def two_def three_def four_def five_def six_def IntegerPort.src_port)
apply (gen_test_cases "FUT")
apply simp_all
store_test_thm "foo3"

gen_test_data "foo3"

thm foo3.test_data


test_spec "makeFUT FUT ex x 4"
apply (simp add: makeFUT_def)
apply (rule_tac x=x in spec, rule allI, simp only: split_tupled_all, rule impI)

apply (insert nets_different noMT)
apply (simp add:  normalize_def ex_def bothNets_def aux aux2 Nets_List_def)
apply (rotate_tac -1, thin_tac "?x")
apply (rotate_tac -1, thin_tac "?x")
apply (simp add: packet_Nets_def)
apply (simp add: PLemmas one_def two_def three_def four_def five_def six_def IntegerPort.src_port)
apply (gen_test_cases "FUT")
apply simp_all
store_test_thm "foo4"

gen_test_data "foo4"

thm foo4.test_data


test_spec "makeFUT FUT ex x 5"
apply (simp add: makeFUT_def)
apply (rule_tac x=x in spec, rule allI, simp only: split_tupled_all, rule impI)

apply (insert nets_different noMT)
apply (simp add:  normalize_def ex_def bothNets_def aux aux2 Nets_List_def)
apply (rotate_tac -1, thin_tac "?x")
apply (rotate_tac -1, thin_tac "?x")
apply (simp add: packet_Nets_def)
apply (simp add: PLemmas one_def two_def three_def four_def five_def six_def IntegerPort.src_port)
apply (gen_test_cases "FUT")
apply simp_all
store_test_thm "foo5"

gen_test_data "foo5"

thm foo0.test_data
    foo4.test_data
    foo3.test_data
    foo2.test_data
    foo1.test_data
    foo5.test_data



text{*

For information: this is the log data for this theory

In short, we get 51 test data for the policy consisting of 17 rules in
a couple of seconds.



# This file was generated automatically 
# by HOL-TestGen 1.4.0 (build: 8240)
# on Wed Jan 14 19:52:49 2009 (UTC)
# Host: hostname not set
# 
# theory,  test case name, type, num. of tests cases/data, time in seconds
compile_example, foo0, test case, 6, 0.237
compile_example, foo0, test data, 6, 0.478
compile_example, foo1, test case, 6, 0.322
compile_example, foo1, test data, 6, 0.561
compile_example, foo2, test case, 12, 0.972
compile_example, foo2, test data, 12, 0.538
compile_example, foo3, test case, 12, 1.012
compile_example, foo3, test data, 12, 0.558
compile_example, foo4, test case, 6, 0.248
compile_example, foo4, test data, 6, 0.432
compile_example, foo5, test case, 9, 0.467
compile_example, foo5, test data, 9, 0.498

*}


end
