(*****************************************************************************
 * HOL-TestGen --- theorem-prover based test case generation
 *                 http://www.brucker.ch/projects/hol-testgen/
 *                                                                            
 * FWTesting.thy --- 
 * This file is part of HOL-TestGen.
 *
 * Copyright (c) 2005-2010 ETH Zurich, Switzerland
 *               2008-2010 Achim D. Brucker, Germany
 *               2009-2010 Univerisite Paris-Sud, France
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 *
 *     * Neither the name of the copyright holders nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************.***************************************************************)
(* $Id: FWTesting.thy 8852 2010-03-24 11:31:11Z lukasbru $ *)
theory 
  FWTesting
imports 
  "PacketFilter/PacketFilter" 
  "FWCompilation/FWCompilationProof"
  "StatefulFW/StatefulFW"
  Testing
begin


text{* This is the formalisation in Isabelle/HOL of firewall policies
and corresponding networks and packets. It first contains the
formalisation of stateless packet filters as described
in~\cite{brucker.ea:model-based:2008}, followed by a verified policy
normalisation technique (described in~\cite{brucker.ea:icst:2010}),
and a formalisation of stateful protocols described
in~\cite{brucker.ea:test-sequence:2007}.  *}

text{* The following statement adjusts the pre-normalization step of
the default test case generation algorithm. This turns out to be more
efficient for the specific case of firewall policies.
*}

setup{* map_testgen_params(TestGen.pre_normalizeTNF_tac_update (
                fn ctxt => 
                      fn clasimp => 
                        (TestGen.ALLCASES (asm_full_simp_tac (simpset_of (ThyInfo.get_theory "Int"))))))
 *} 


text{* Next, the Isar command @{text prepare_fw_spec} is specified. It
 can be used to turn test specifications of the form: 
"$C\ x \Longrightarrow FUT\ x\ =\ policy\ x$" 
into the desired form for test case generation.  *}

ML {*
fun prepare_fw_spec_tac ctxt   =
                 (TRY((res_inst_tac ctxt [(("x",0),"x")] spec 1) THEN
                  (resolve_tac [allI] 1) THEN
                  (split_all_tac 1) THEN
                  (TRY (resolve_tac [impI] 1))));
*}


method_setup prepare_fw_spec = 
 {*
   Scan.succeed  (fn ctxt => SIMPLE_METHOD
       (prepare_fw_spec_tac ctxt))*} "Prepares the firewall test theorem"

end
