(*****************************************************************************
 * HOL-TestGen --- theorem-prover based test case generation
 *                 http://www.brucker.ch/projects/hol-testgen/
 *                                                                            
 * FWCompilationProof.thy --- 
 * This file is part of HOL-TestGen.
 *
 * Copyright (c) 2005-2010 ETH Zurich, Switzerland
 *               2008-2010 Achim D. Brucker, Germany
 *               2009-2010 Univerisite Paris-Sud, France 
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 *
 *     * Neither the name of the copyright holders nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************)
(* $Id: FWCompilationProof.thy 8876 2010-03-31 15:41:48Z lukasbru $ *)

theory FWCompilationProof
imports FWCompilation
begin

text {* This theory contains the complete proofs of the normalisation
procedure.  *}


lemma wellformed_policy1_charn[rule_format] : "wellformed_policy1 p \<longrightarrow>
DenyAll \<in> set p \<longrightarrow> (\<exists> p'. p = DenyAll # p' \<and> DenyAll \<notin> set p')"
by(induct p,simp_all)
 

lemma singleCombinatorsConc: "singleCombinators (x#xs) \<Longrightarrow> singleCombinators xs"
by (case_tac x,simp_all)

lemma aux0_0: "singleCombinators x \<Longrightarrow> \<not> (\<exists> a b. (a\<oplus>b) \<in> set x)"
apply (induct x, simp_all)
apply (rule allI)+
by (case_tac a,simp_all)


lemma aux0_4: "(a \<in> set x \<or> a \<in> set y) = (a \<in> set (x@y))"
by auto

lemma aux0_1: "\<lbrakk>singleCombinators xs; singleCombinators [x]\<rbrakk> \<Longrightarrow> singleCombinators (x#xs)"
by (case_tac x,simp_all) 


lemma aux0_6: "\<lbrakk>singleCombinators xs; \<not> (\<exists> a b. x = a \<oplus> b)\<rbrakk> \<Longrightarrow> singleCombinators(x#xs)"
apply (rule aux0_1,simp_all)
apply (case_tac x,simp_all)
apply auto
done

lemma aux0_5: " \<not> (\<exists> a b. (a\<oplus>b) \<in> set x) \<Longrightarrow> singleCombinators x"
apply (induct x)
apply simp_all
apply (rule aux0_6)
apply auto
done


lemma aux0_7: "\<lbrakk>singleCombinators x; singleCombinators y\<rbrakk> \<Longrightarrow> singleCombinators (x@y)"
apply (rule aux0_5)
apply auto
apply (insert aux0_0 [of x])
apply (insert aux0_0 [of y])
apply auto
done



lemma ConcAssoc: "C((A \<oplus> B) \<oplus> D) = C(A \<oplus> (B \<oplus> D))"
apply (simp add: C.simps)
done


lemma Caux: "x \<in> dom (C b) \<Longrightarrow> (C a ++ C b) x = C b x "
by (auto simp: C.simps dom_def)



lemma nCauxb: "x \<notin> dom (b) \<Longrightarrow> (a ++ b) x = a x "
by (simp_all add: C.simps dom_def map_add_def option.simps(4))


lemma Cauxb: "x \<notin> dom (C b) \<Longrightarrow> (C a ++ C b) x = C a x "
apply (rule nCauxb)
by simp




lemma aux0: "singleCombinators (policy2list p)"
apply (induct_tac p)
apply simp_all
apply (rule aux0_7)
apply simp_all
done


lemma ANDConc[rule_format]: "allNetsDistinct (a#p) \<longrightarrow> allNetsDistinct (p)"
apply (simp add: allNetsDistinct_def)
apply (case_tac "a")
by simp_all


lemma aux6: "twoNetsDistinct a1 a2 a b \<Longrightarrow> dom (deny_all_from_to a1 a2) \<inter> dom (deny_all_from_to a b) = {}"
by (auto simp: twoNetsDistinct_def netsDistinct_def src_def dest_def in_subnet_def PolicyCombinators.PolicyCombinators dom_def)

lemma aux5[rule_format]: "(DenyAllFromTo a b) \<in> set p \<longrightarrow> a \<in> set (net_list p)"
by (rule net_list_aux.induct,simp_all) 


lemma aux5a[rule_format]: "(DenyAllFromTo b a) \<in> set p \<longrightarrow> a \<in> set (net_list p)"
by (rule net_list_aux.induct,simp_all) 

lemma aux5c[rule_format]: "(AllowPortFromTo a b po) \<in> set p \<longrightarrow> a \<in> set (net_list p)"
by (rule net_list_aux.induct,simp_all) 

lemma aux5d[rule_format]: "(AllowPortFromTo b a po) \<in> set p \<longrightarrow> a \<in> set (net_list p)"
by (rule net_list_aux.induct,simp_all) 

lemma aux10[rule_format]: "a \<in> set (net_list p) \<longrightarrow> a \<in> set (net_list_aux p)"
by simp


lemma srcInNetListaux[simp]: "\<lbrakk>x \<in> set p; singleCombinators[x]; x \<noteq> DenyAll\<rbrakk> \<Longrightarrow> srcNet x \<in> set (net_list_aux p)"
apply (induct p)
apply simp_all
apply (case_tac "x = a", simp_all)
apply (case_tac a, simp_all)+
done 


lemma destInNetListaux[simp]: "\<lbrakk>x \<in> set p; singleCombinators[x]; x \<noteq> DenyAll\<rbrakk> \<Longrightarrow> destNet x \<in> set (net_list_aux p)"
apply (induct p)
apply simp_all
apply (case_tac "x = a", simp_all)
apply (case_tac a, simp_all)+
done 



lemma tND1: "\<lbrakk>allNetsDistinct p; x \<in> set p; y \<in> set p; a = srcNet x; b = destNet x; c = srcNet y; d = destNet y; a \<noteq> c;
             singleCombinators[x]; x \<noteq> DenyAll; singleCombinators[y]; y \<noteq> DenyAll\<rbrakk> \<Longrightarrow> twoNetsDistinct a b c d"
apply (simp add: allNetsDistinct_def twoNetsDistinct_def)
done


lemma tND2: "\<lbrakk>allNetsDistinct p; x \<in> set p; y \<in> set p; a = srcNet x; b = destNet x; c = srcNet y; d = destNet y; b \<noteq> d;
             singleCombinators[x]; x \<noteq> DenyAll; singleCombinators[y]; y \<noteq> DenyAll\<rbrakk> \<Longrightarrow> twoNetsDistinct a b c d"
apply (simp add: allNetsDistinct_def twoNetsDistinct_def)
done


lemma tND: "\<lbrakk>allNetsDistinct p; x \<in> set p; y \<in> set p; a = srcNet x; b = destNet x; c = srcNet y; d = destNet y; a \<noteq> c \<or> b \<noteq> d;
             singleCombinators[x]; x \<noteq> DenyAll; singleCombinators[y]; y \<noteq> DenyAll\<rbrakk> \<Longrightarrow> twoNetsDistinct a b c d"
apply (case_tac "a \<noteq> c", simp_all)
apply (erule_tac x = x and y =y in tND1, simp_all)
apply (erule_tac x = x and y =y in tND2, simp_all)
done


lemma aux7: "\<lbrakk>DenyAllFromTo a b \<in> set p; allNetsDistinct ((DenyAllFromTo c d)#p); a\<noteq> c\<or> b\<noteq> d\<rbrakk> \<Longrightarrow> twoNetsDistinct a b c d"
apply (erule_tac x = "DenyAllFromTo a b" and y = "DenyAllFromTo c d" in tND)
apply simp_all
done


lemma aux7a: "\<lbrakk>DenyAllFromTo a b  \<in> set p; allNetsDistinct ((AllowPortFromTo c d po)#p); a \<noteq> c\<or> b \<noteq> d\<rbrakk> \<Longrightarrow> twoNetsDistinct a b c d"
apply (erule_tac x = "DenyAllFromTo a b" and y = "AllowPortFromTo  c d po" in tND)
apply simp_all
done


lemma nDComm: assumes ab: "netsDistinct a b" shows ba: "netsDistinct b a"
apply (insert ab)
by (auto simp: netsDistinct_def  in_subnet_def)

lemma tNDComm: assumes abcd: "twoNetsDistinct a b c d" shows "twoNetsDistinct c d a b"
apply (insert abcd)
apply (metis twoNetsDistinct_def nDComm)
done

lemma aux[rule_format]: "a \<in> set (removeShadowRules2 p) \<longrightarrow> a \<in> set p"
apply (case_tac a)
by (rule removeShadowRules2.induct, simp_all)+

lemma aux12: "\<lbrakk>a \<in> x; b \<notin> x\<rbrakk> \<Longrightarrow> a \<noteq> b"
by auto

lemma aux26[simp]: "twoNetsDistinct a b c d \<Longrightarrow> dom (C (AllowPortFromTo a b p)) \<inter> dom (C (DenyAllFromTo c d)) = {}"
by (auto simp: PLemmas twoNetsDistinct_def netsDistinct_def) auto

lemma ND0aux1[rule_format]: "DenyAllFromTo x y \<in> set b \<Longrightarrow>  x \<in> set (net_list_aux b)"
by (metis aux5 net_list.simps set_remdups)

lemma ND0aux2[rule_format]: "DenyAllFromTo x y \<in> set b \<Longrightarrow>  y \<in> set (net_list_aux b)"
by (metis aux5a net_list.simps set_remdups)

lemma ND0aux3[rule_format]: "AllowPortFromTo x y p \<in> set b \<Longrightarrow>  x \<in> set (net_list_aux b)"
by (metis aux5c net_list.simps set_remdups)

lemma ND0aux4[rule_format]: "AllowPortFromTo x y p \<in> set b \<Longrightarrow>  y \<in> set (net_list_aux b)"
by (metis aux5d net_list.simps set_remdups)

lemma aNDSubsetaux[rule_format]: "singleCombinators a  \<longrightarrow> set a \<subseteq> set b \<longrightarrow> set (net_list_aux a) \<subseteq> set (net_list_aux b)"
apply (induct a)
apply simp_all
apply clarify
apply (drule mp, erule singleCombinatorsConc)
apply (case_tac "a1")
apply (simp_all add: contra_subsetD)
apply (metis contra_subsetD)
apply (metis ND0aux1 ND0aux2 contra_subsetD mem_def)
apply (metis ND0aux3 ND0aux4 contra_subsetD mem_def)
done

lemma aNDSetsEqaux[rule_format]: "singleCombinators a \<longrightarrow> singleCombinators b  \<longrightarrow> set a = set b \<longrightarrow> set (net_list_aux a) = set (net_list_aux b)"
apply (rule impI)+
apply (rule equalityI)
apply (rule aNDSubsetaux, simp_all)+
done

lemma aNDSubset: "\<lbrakk>singleCombinators a;set a \<subseteq> set b; allNetsDistinct b\<rbrakk> \<Longrightarrow> allNetsDistinct a"
apply (simp add: allNetsDistinct_def)
apply (rule allI)+
apply (rule impI)+
apply (drule_tac x = "aa" in spec, drule_tac x = "ba" in spec)
apply (metis subsetD aNDSubsetaux)
done

lemma aNDSetsEq: "\<lbrakk>singleCombinators a; singleCombinators b; set a = set b; allNetsDistinct b\<rbrakk> \<Longrightarrow> allNetsDistinct a"
apply (simp add: allNetsDistinct_def)
apply (rule allI)+
apply (rule impI)+
apply (drule_tac x = "aa" in spec, drule_tac x = "ba" in spec)
apply (metis aNDSetsEqaux mem_def)
done

lemma SCConca: "\<lbrakk>singleCombinators p; singleCombinators [a]\<rbrakk> \<Longrightarrow> singleCombinators (a#p)"
by (case_tac "a",simp_all)

lemma aux3[simp]: "\<lbrakk>singleCombinators p; singleCombinators [a];  allNetsDistinct (a#p)\<rbrakk> \<Longrightarrow> allNetsDistinct (a#a#p)"
apply (insert aNDSubset[of "(a#a#p)" "(a#p)"])
apply (simp add: SCConca)
done

lemma wp2_aux[rule_format]: "wellformed_policy2 (xs @ [x]) \<longrightarrow> wellformed_policy2 xs"
apply (induct xs, simp_all)
apply (case_tac "a", simp_all)
done

lemma wp1_aux1a[rule_format]: "xs \<noteq> [] \<longrightarrow> wellformed_policy1_strong (xs @ [x])  \<longrightarrow> wellformed_policy1_strong xs"
by (induct xs,simp_all)

lemma wp1alternative_RS1[rule_format]: "DenyAll \<in> set p \<longrightarrow> wellformed_policy1_strong (removeShadowRules1 p)"
by (induct p,simp_all)

lemma wellformed_eq: "DenyAll \<in> set p \<longrightarrow> ((wellformed_policy1 p) = (wellformed_policy1_strong p))"
by (induct p,simp_all)

lemma set_insort: "set(insort x xs l) = insert x (set xs)"
by (induct xs) auto

lemma set_sort[simp]: "set(sort xs l) = set xs"
by (induct xs) (simp_all add:set_insort)

lemma aux79[rule_format]: "y \<in> set (insort x a l) \<longrightarrow>  y \<noteq> x \<longrightarrow> y \<in> set a"
apply (induct a)
by auto

lemma aux80: "\<lbrakk>y \<notin> set p; y \<noteq> x\<rbrakk> \<Longrightarrow> y \<notin> set (insort x (sort p l) l)"
apply (metis aux79 set_sort)
done

lemma aux82: "(insort DenyAll p l) = DenyAll#p"
by (induct p,simp_all)

lemma WP1Conca: "DenyAll \<notin> set p \<Longrightarrow> wellformed_policy1 (a#p)"
by (case_tac a,simp_all)

lemma Cdom2: "x \<in> dom(C b) \<Longrightarrow> C (a \<oplus> b) x = (C b) x"
by (auto simp: C.simps)

lemma wp2Conc[rule_format]: "wellformed_policy2 (x # xs) \<Longrightarrow> wellformed_policy2 xs"
by (case_tac "x",simp_all)

lemma saux[simp]: "(insort DenyAll p l) = DenyAll#p"
by (induct_tac p,simp_all)
 
lemma saux3[rule_format]: "DenyAllFromTo a b \<in> set list \<longrightarrow> DenyAllFromTo c d \<notin> set list \<longrightarrow> (a \<noteq> c) \<or> (b \<noteq> d)"
by blast

lemma waux2[rule_format]: " (DenyAll \<notin> set xs) \<longrightarrow> wellformed_policy1 xs"
by (induct_tac xs,simp_all)

lemma waux3[rule_format]: "\<lbrakk>x \<noteq> a;  x \<notin> set p\<rbrakk> \<Longrightarrow> x \<notin> set (insort a p l)"
by (metis aux79)

lemma wellformed1_sorted_aux[rule_format]: "wellformed_policy1 (x#p) \<Longrightarrow> wellformed_policy1 (insort x p l)" 
apply (case_tac x,simp_all) 
by (rule waux2,rule waux3, simp_all)+

lemma SR1Subset: "set (removeShadowRules1 p) \<subseteq> set p"
apply (induct_tac p, simp_all)
apply (case_tac a, simp_all)
by auto

lemma SCSubset[rule_format]: " singleCombinators b \<longrightarrow> set a \<subseteq> set b \<longrightarrow>singleCombinators a"
apply (induct_tac a)
apply auto
apply (case_tac "a")
apply simp_all
apply (subgoal_tac "Combinators1 \<oplus> Combinators2 \<in> set b \<longrightarrow> \<not> singleCombinators b", simp)
apply (rule singleCombinators.induct, simp_all)
done

lemma setInsert[simp]: "set list \<subseteq> insert a (set list)"
by auto

lemma SC_RS1[rule_format,simp]: "singleCombinators p \<longrightarrow> allNetsDistinct p \<longrightarrow>
    singleCombinators (removeShadowRules1 p)"
apply (induct_tac p)
apply simp_all
apply (rule impI)+
apply (drule mp)
apply (erule SCSubset,simp)
by (simp add: ANDConc)

lemma RS2Set[rule_format]: "set (removeShadowRules2 p) \<subseteq> set p"
apply (induct p, simp_all)
apply (case_tac a, simp_all)
apply auto
done

lemma WP1: "a \<notin> set list \<Longrightarrow> a \<notin> set (removeShadowRules2 list)"
apply (insert RS2Set [of list])
apply blast
done

lemma denyAllDom[simp]: "x \<in> dom (deny_all)"
by (simp add: PLemmas)

lemma DAimpliesMR_E[rule_format]: "DenyAll \<in> set p \<longrightarrow> (\<exists> r. matching_rule x p = Some r)"
apply (simp add: matching_rule_def)
apply (rule_tac xs = p in rev_induct)
apply simp_all
by (metis C.simps(1) denyAllDom)

lemma DAimplieMR[rule_format]: "DenyAll \<in> set p \<Longrightarrow> matching_rule x p \<noteq> None"
by (auto intro: DAimpliesMR_E)

lemma MRList1[rule_format]: "x \<in> dom (C a) \<Longrightarrow> matching_rule x (b@[a]) = Some a"
by (simp add: matching_rule_def)

lemma MRList2: "x \<in> dom (C a) \<Longrightarrow> matching_rule x (c@b@[a]) = Some a"
by (simp add: matching_rule_def)

lemma MRList3: "x \<notin> dom (C xa) \<Longrightarrow> matching_rule x (a @ b # xs @ [xa]) = matching_rule x (a @ b # xs)"
by (simp add: matching_rule_def)

lemma CConcEnd[rule_format]: "C a x = Some y \<longrightarrow> C (list2policy (xs @ [a])) x = Some y"
 (is "?P xs")
apply (rule_tac P = ?P in list2policy.induct)
by (simp_all add:C.simps)

lemma CConcStartaux: " \<lbrakk>C a x = None\<rbrakk> \<Longrightarrow> (C aa ++ C a) x = C aa x"
by (simp add: PLemmas)

lemma CConcStart[rule_format]: "xs \<noteq> [] \<longrightarrow> C a x = None \<longrightarrow> C (list2policy (xs @ [a])) x = C (list2policy xs) x"
apply (rule list2policy.induct)
by (simp_all add: PLemmas)

lemma mrNnt[simp]: "matching_rule x p = Some a \<Longrightarrow> p \<noteq> []"
apply (simp add: matching_rule_def)
by auto

lemma mr_is_C[rule_format]: "matching_rule x p = Some a \<longrightarrow> C (list2policy (p)) x = C a x"
apply (simp add: matching_rule_def)
apply (rule rev_induct)
apply simp_all
apply safe
apply (metis CConcEnd rotate_simps)
apply (metis CConcEnd)
apply (metis CConcStart domD domIff foldl_Nil matching_rule_rev.simps(2) option.simps(1) rev_foldl_cons rotate_simps)
done

lemma CConcStart2: "\<lbrakk>p \<noteq> []; x \<notin> dom (C a)\<rbrakk> \<Longrightarrow> C (list2policy (p@[a])) x = C (list2policy p)x"
by (erule CConcStart,simp add: PLemmas)

lemma lCdom2: "(list2policy (a @ (b @ c))) = (list2policy ((a@b)@c))"
by auto

lemma CConcEnd1: "\<lbrakk>q@p \<noteq> []; x \<notin> dom (C a)\<rbrakk> \<Longrightarrow> C (list2policy (q@p@[a])) x = C (list2policy (q@p))x"
apply (subst lCdom2)
by (rule CConcStart2, simp_all)

lemma CConcEnd2[rule_format]: "x \<in> dom (C a) \<longrightarrow> C (list2policy (xs @ [a])) x = C a x"
 (is "?P xs")
apply (rule_tac P = ?P in list2policy.induct)
by (auto simp:C.simps)

lemma SCConcEnd: "singleCombinators (xs @ [xa]) \<Longrightarrow> singleCombinators xs"
by (induct "xs", simp_all, case_tac a, simp_all)

lemma bar3: "x \<in> dom (C (list2policy (xs @ [xa]))) \<Longrightarrow> x \<in> dom (C (list2policy xs)) \<or> x \<in> dom (C xa)"
by (metis CConcEnd1 domIff list2policy.simps(1) rotate_simps self_append_conv2)

lemma CeqEnd[rule_format,simp]: "a \<noteq> [] \<longrightarrow> x \<in> dom (C (list2policy a)) \<longrightarrow> 
                C (list2policy (b@a)) x = (C (list2policy a)) x"
apply (rule rev_induct,simp_all) 
apply (case_tac "xs \<noteq> []", simp_all)
apply (case_tac "x \<in> dom (C xa)")
apply (metis CConcEnd2 MRList2 mr_is_C rotate_simps)
apply (metis CConcEnd1 CConcStart2 Nil_is_append_conv bar3 rotate_simps)
apply (metis MRList2 eq_Nil_appendI mr_is_C rotate_simps)
done

lemma CConcStartA[rule_format,simp]: " x \<in> dom (C a) \<longrightarrow> x \<in> dom (C (list2policy (a # b)))"
(is "?P b")
apply (rule_tac P = ?P in list2policy.induct)  
apply (simp_all add: C.simps)
done

lemma list2policyconc[rule_format]: "a \<noteq> [] \<longrightarrow> (list2policy (xa # a)) = (xa) \<oplus> (list2policy a)"
by (induct a,simp_all)

lemma domConc: "\<lbrakk>x \<in> dom (C (list2policy b)); b \<noteq> []\<rbrakk>  \<Longrightarrow> x \<in> dom (C (list2policy (a@b)))"
by (auto simp: PLemmas)

lemma CeqStart[rule_format,simp]: "x \<notin> dom (C (list2policy a)) \<longrightarrow> a \<noteq> [] \<longrightarrow> b \<noteq> [] \<longrightarrow> 
                                   C (list2policy (b@a)) x = (C (list2policy b)) x"
apply (rule list2policy.induct,simp_all)
apply (auto simp: list2policyconc PLemmas)
done 

lemma C_eq_if_mr_eq2: "\<lbrakk>matching_rule x a = Some r; matching_rule x b = Some r; a \<noteq> []; b \<noteq> []\<rbrakk>  \<Longrightarrow>  
  (C (list2policy a)) x = (C (list2policy b)) x"
by (metis mr_is_C)

lemma nMRtoNone[rule_format]: "p \<noteq> [] \<longrightarrow> matching_rule x p = None \<longrightarrow> C (list2policy p) x = None"
apply (rule rev_induct, simp_all)
apply (case_tac "xs = []", simp_all)
by (simp_all add: matching_rule_def dom_def)

lemma C_eq_if_mr_eq: 
 "\<lbrakk>matching_rule x b = matching_rule x a; a \<noteq> []; b \<noteq> []\<rbrakk> \<Longrightarrow>  
  (C (list2policy a)) x = (C (list2policy b)) x"
apply (cases "matching_rule x a = None") 
apply simp_all
apply (subst nMRtoNone)
apply (simp_all)
apply (subst nMRtoNone)
apply simp_all
by (auto intro: C_eq_if_mr_eq2)

lemma wp1n_tl [rule_format]: "wellformed_policy1_strong p \<longrightarrow> p = (DenyAll#(tl p))"
by (induct p, simp_all)

lemma foo2: "\<lbrakk>a \<notin> set ps; a \<notin> set ss; set p = set s;  p = (a#(ps)); s = (a#ss)\<rbrakk> \<Longrightarrow> set (ps) = set (ss)"
by auto

lemma bar5: "matching_rule x (p@[a]) \<noteq> Some a \<Longrightarrow> x \<notin> dom (C a)"
by (simp add: matching_rule_def split: if_splits)

lemma foo3a[rule_format]: "matching_rule x (a@[b]@c) = Some b \<longrightarrow>  r \<in> set c \<longrightarrow> b \<notin> set c \<longrightarrow> x \<notin> dom (C r)"
apply (rule rev_induct) 
apply simp_all
apply (rule impI|rule conjI|simp)+
apply (rule_tac p = "a @ b # xs" in bar5,simp_all)
apply (rule impI,simp)+
apply (drule sym,drule mp, simp_all)
apply (rule MRList3[symmetric],drule sym)
apply (rule_tac p = "a @ b # xs" in bar5,simp_all)
done

lemma foo3D: "\<lbrakk>wellformed_policy1 p; p = (DenyAll#ps); matching_rule x p = Some DenyAll; r \<in> set ps\<rbrakk> \<Longrightarrow> x \<notin> dom (C r)"
by (rule_tac a = "[]" and b = "DenyAll" and c = "ps"  in foo3a, simp_all)

lemma foo4[rule_format]: "set p = set s \<and> (\<forall> r. r \<in> set p \<longrightarrow> x \<notin> dom (C r)) \<longrightarrow> (\<forall> r .r \<in> set s \<longrightarrow> x \<notin> dom (C r))"
by simp

lemma foo5b[rule_format]: "x \<in> dom (C b) \<longrightarrow> (\<forall> r. r \<in> set c \<longrightarrow> x \<notin> dom (C r))  \<longrightarrow> 
                           matching_rule x (b#c) = Some b"
apply (simp add: matching_rule_def)
apply (rule_tac xs = c in rev_induct, simp_all)  
done

lemma mr_first: "\<lbrakk>x \<in> dom (C b); (\<forall> r. r \<in> set c \<longrightarrow> x \<notin> dom (C r)); s = b#c\<rbrakk> \<Longrightarrow> 
                           matching_rule x s = Some b"
by (simp add: foo5b)

lemma mr_charn[rule_format]: "a \<in> set p \<longrightarrow> (x \<in> dom (C a)) \<longrightarrow> (\<forall> r. r \<in> set p \<and> x \<in> dom (C r) \<longrightarrow> r = a) \<longrightarrow>  matching_rule x p = Some a"
apply (rule_tac xs = p in rev_induct) 
by (simp_all add: matching_rule_def)

lemma foo8: "\<lbrakk>(\<forall> r. r \<in> set p \<and> x \<in> dom (C r) \<longrightarrow> r = a); set p = set s\<rbrakk> \<Longrightarrow> (\<forall> r. r \<in> set s \<and> x \<in> dom (C r) \<longrightarrow> r = a)"
by auto

lemma mrConcEnd[rule_format]: "matching_rule x (b # p) = Some a \<longrightarrow> a \<noteq> b \<longrightarrow> matching_rule x p = Some a"
apply (simp add: matching_rule_def)
apply (rule_tac xs = p in rev_induct,simp_all) 
by auto

lemma wp3tl[rule_format]: "wellformed_policy3 p \<longrightarrow> wellformed_policy3 (tl p)"
by (induct p, simp_all, case_tac a, simp_all)

lemma wp3Conc[rule_format]: "wellformed_policy3 (a#p) \<longrightarrow> wellformed_policy3 p"
by (induct p, simp_all, case_tac a, simp_all)

lemma SCnotConc[rule_format,simp]: "a \<oplus> b \<in> set p \<longrightarrow> singleCombinators p \<longrightarrow> False"
by (induct p, simp_all, case_tac aa, simp_all)

lemma foo98[rule_format]:"matching_rule x (aa # p) = Some a \<longrightarrow> x \<in> dom (C r) \<longrightarrow> 
            r \<in> set p
           \<longrightarrow> a \<in> set p"
apply (simp add: matching_rule_def)
apply (rule rev_induct)
apply simp_all
apply (case_tac "r = xa", simp_all)
done

lemma auxx8: "removeShadowRules1_alternative_rev [x] = [x]"
by (case_tac "x", simp_all)

lemma RS1End[rule_format]: "x \<noteq> DenyAll \<longrightarrow> removeShadowRules1 (xs @ [x]) = (removeShadowRules1 xs)@[x]"
by (induct_tac xs, simp_all)

lemma aux114: "x \<noteq> DenyAll \<Longrightarrow> removeShadowRules1_alternative_rev (x#xs) = x#(removeShadowRules1_alternative_rev xs)"
apply (induct_tac xs)
apply (auto simp: auxx8)
by (case_tac "x", simp_all)

lemma aux115[rule_format]: "x \<noteq> DenyAll \<Longrightarrow> removeShadowRules1_alternative (xs@[x]) = (removeShadowRules1_alternative xs)@[x]"
apply (simp add: removeShadowRules1_alternative_def aux114)
done

lemma RS1_DA[simp]: "removeShadowRules1 (xs @ [DenyAll]) = [DenyAll]"
by (induct_tac xs, simp_all)

lemma rSR1_eq: "removeShadowRules1_alternative = removeShadowRules1"
apply (rule ext)
apply (simp add: removeShadowRules1_alternative_def)
apply (rule_tac xs = x in rev_induct)
apply simp_all
apply (case_tac "xa = DenyAll", simp_all)
apply (metis RS1End aux114 rev.simps)
done

lemma mrMTNone[simp]: "matching_rule x [] = None"
by (simp add: matching_rule_def)

lemma DAAux[simp]: "x \<in> dom (C DenyAll)"
by (simp add: dom_def PolicyCombinators.PolicyCombinators C.simps)

lemma mrSet[rule_format]: "matching_rule x p = Some r \<longrightarrow> r \<in> set p"
apply (simp add: matching_rule_def)
apply (rule_tac xs=p in rev_induct)
apply simp_all
done

lemma mr_not_Conc: "singleCombinators p \<Longrightarrow> matching_rule x p \<noteq> Some (a\<oplus>b)"
apply (auto simp:  mrSet)
apply (drule mrSet)
apply (erule SCnotConc,simp)
done

lemma foo25[rule_format]: "wellformed_policy3 (p@[x]) \<longrightarrow> wellformed_policy3 p"
by (induct p, simp_all, case_tac a, simp_all)

lemma mr_in_dom[rule_format]: "matching_rule x p = Some a \<longrightarrow> x \<in> dom (C a)"
apply (rule_tac xs = p in rev_induct)
by (auto simp: matching_rule_def)

lemma domInterMT[rule_format]: "\<lbrakk>dom a \<inter> dom b = {}; x \<in> dom a\<rbrakk> \<Longrightarrow> x \<notin> dom b"
by auto

lemma wp3EndMT[rule_format]: "wellformed_policy3 (p@[xs]) \<longrightarrow> AllowPortFromTo a b po \<in> set p \<longrightarrow>
                             dom (C (AllowPortFromTo a b po)) \<inter> dom (C xs) = {}"
apply (induct p,simp_all)
apply (rule impI)+
apply (drule mp)
apply (erule wp3Conc)
by clarify auto

lemma foo29: "\<lbrakk>dom (C a) \<noteq> {}; dom (C a) \<inter> dom (C b) = {}\<rbrakk> \<Longrightarrow> a \<noteq> b"
by auto

lemma foo28:  "\<lbrakk>AllowPortFromTo a b po \<in> set p; dom (C (AllowPortFromTo a b po)) \<noteq> {}; (wellformed_policy3 (p@[x]))\<rbrakk> \<Longrightarrow>
                        x \<noteq> AllowPortFromTo a b po"
by (metis foo29 C.simps wp3EndMT)

lemma foo28a[rule_format]: "x \<in> dom (C a) \<Longrightarrow> dom (C a) \<noteq> {}"
by auto

lemma allow_deny_dom[simp]: "dom (C (AllowPortFromTo a b po)) \<subseteq> dom (C (DenyAllFromTo a b))"
by (simp_all add: twoNetsDistinct_def netsDistinct_def PLemmas) auto 

lemma DenyAllowDisj: "dom (C (AllowPortFromTo a b p)) \<noteq> {} \<Longrightarrow> 
                      dom (C (DenyAllFromTo a b)) \<inter> dom (C (AllowPortFromTo a b p))  \<noteq> {}"
by (metis Int_absorb1 allow_deny_dom)

lemma domComm: "dom a \<inter> dom b = dom b \<inter> dom a"
by auto

lemma foo31: "\<lbrakk>(\<forall> r. r \<in> set p \<and> x \<in> dom (C r) \<longrightarrow> (r = AllowPortFromTo a b po \<or> r = DenyAllFromTo a b \<or> r = DenyAll)); 
                set p = set s\<rbrakk> \<Longrightarrow> 
              (\<forall> r. r \<in> set s \<and> x \<in> dom (C r) \<longrightarrow> (r = AllowPortFromTo a b po \<or> r = DenyAllFromTo a b \<or> r = DenyAll))"
by auto

lemma r_not_DA_in_tl[rule_format]: "wellformed_policy1_strong p \<longrightarrow>  a \<in> set p \<longrightarrow>  a \<noteq> DenyAll \<longrightarrow> a \<in> set (tl p)"
by (induct p,simp_all)

lemma wp1_aux1aa[rule_format]: "wellformed_policy1_strong p \<longrightarrow> DenyAll \<in> set p"
by (induct p,simp_all)

lemma mauxa: "(\<exists> r. a b = Some r) = (a b \<noteq> None)"
by auto

lemma wp1_auxa: "wellformed_policy1_strong p \<Longrightarrow> (\<exists> r. matching_rule x p = Some r)"
apply (rule DAimpliesMR_E)
by (erule wp1_aux1aa)

lemma l2p_aux[rule_format]: "list \<noteq> [] \<longrightarrow> list2policy (a # list) = a \<oplus>(list2policy list)"
by (induct "list", simp_all)

lemma l2p_aux2[rule_format]: "list = [] \<Longrightarrow> list2policy (a # list) = a"
by simp

lemma deny_dom[simp]:  "twoNetsDistinct a b c d \<Longrightarrow> dom (C (DenyAllFromTo a b)) \<inter> dom (C (DenyAllFromTo c d)) = {}"
apply (simp add: C.simps)
by (erule aux6)

lemma domTrans: "\<lbrakk>dom a \<subseteq> dom b; dom(b) \<inter> dom (c) = {}\<rbrakk> \<Longrightarrow> dom(a) \<inter> dom(c) = {}" 
by auto

lemma DomInterAllowsMT: "\<lbrakk> twoNetsDistinct a b c d\<rbrakk> 
       \<Longrightarrow> dom (C (AllowPortFromTo a b p)) \<inter> dom (C (AllowPortFromTo c d po)) = {}"
apply (case_tac "p = po", simp_all)
apply (rule_tac b = "C (DenyAllFromTo a b)" in domTrans, simp_all)
apply (metis domComm aux26 tNDComm)
by (simp add: twoNetsDistinct_def netsDistinct_def PLemmas) auto

lemma DomInterAllowsMT_Ports: "\<lbrakk>p \<noteq> po\<rbrakk>
      \<Longrightarrow> dom (C (AllowPortFromTo a b p)) \<inter> dom (C (AllowPortFromTo c d po)) = {}"
by (simp add: twoNetsDistinct_def netsDistinct_def PLemmas) auto

lemma aux7aa: "\<lbrakk>AllowPortFromTo a b poo \<in> set p; allNetsDistinct ((AllowPortFromTo c d po) # p); a \<noteq> c \<or> b \<noteq> d\<rbrakk> \<Longrightarrow> twoNetsDistinct a b c d"
apply (simp add: allNetsDistinct_def twoNetsDistinct_def)
apply (case_tac "a \<noteq> c")
apply (rule disjI1)
apply (drule_tac x = "a" in spec, drule_tac x = "c" in spec)
apply (simp split: if_splits)
apply (simp_all add: ND0aux3,metis)
apply (rule disjI2)
apply (drule_tac x = "b" in spec, drule_tac x = "d" in spec)
apply (simp split: if_splits)
apply (metis ND0aux4 mem_def mem_iff)+
done

lemma wellformed_policy3_charn[rule_format]: "singleCombinators p \<longrightarrow> distinct p \<longrightarrow> allNetsDistinct p \<longrightarrow> wellformed_policy1 p \<longrightarrow> wellformed_policy2 p  \<longrightarrow> wellformed_policy3 p"
apply (induct_tac p)   
apply simp_all
apply clarify
apply simp_all
apply (auto intro: singleCombinatorsConc ANDConc waux2 wp2Conc)
apply (case_tac a) 
apply simp_all
apply clarify
apply (case_tac r)
apply simp_all
apply (metis Int_commute) 
apply (metis DomInterAllowsMT aux7aa DomInterAllowsMT_Ports)
apply (metis aux0_0 mem_def)
done

lemma ANDConcEnd: "\<lbrakk> allNetsDistinct (xs @ [xa]); singleCombinators xs\<rbrakk> \<Longrightarrow> allNetsDistinct xs"
by (rule aNDSubset) auto

lemma WP1ConcEnd[rule_format]: "wellformed_policy1 (xs @ [xa]) \<longrightarrow> wellformed_policy1 xs"
by (induct xs, simp_all)

lemma NDComm: "netsDistinct a b = netsDistinct b a"
by (auto simp: netsDistinct_def in_subnet_def)

lemma DistinctNetsDenyAllow: 
 "\<lbrakk>DenyAllFromTo b c \<in> set p; AllowPortFromTo a d po \<in> set p; allNetsDistinct p; dom (C (DenyAllFromTo b c)) \<inter> dom (C (AllowPortFromTo a d po)) \<noteq> {}\<rbrakk>
       \<Longrightarrow> b = a \<and> c = d" 
apply (simp add: allNetsDistinct_def)
apply (frule_tac x = "b" in spec) 
apply (drule_tac x = "d" in spec)
apply (drule_tac x = "a" in spec)
apply (drule_tac x = "c" in spec)
apply (metis Int_commute ND0aux1 ND0aux3 NDComm aux26 twoNetsDistinct_def ND0aux2 ND0aux4)
done

lemma DistinctNetsAllowAllow: 
 "\<lbrakk>AllowPortFromTo b c poo \<in> set p; AllowPortFromTo a d po \<in> set p; allNetsDistinct p; dom (C (AllowPortFromTo b c poo)) \<inter> dom (C (AllowPortFromTo a d po)) \<noteq> {}\<rbrakk>
       \<Longrightarrow> b = a \<and> c = d \<and> poo = po" 
apply (simp add: allNetsDistinct_def)
apply (frule_tac x = "b" in spec) 
apply (drule_tac x = "d" in spec)
apply (drule_tac x = "a" in spec)
apply (drule_tac x = "c" in spec)
apply (metis DomInterAllowsMT DomInterAllowsMT_Ports ND0aux3 ND0aux4 NDComm twoNetsDistinct_def)
done

lemma WP2RS2[simp]: 
 "\<lbrakk>singleCombinators p;
 distinct p;
 allNetsDistinct p\<rbrakk> \<Longrightarrow> wellformed_policy2 (removeShadowRules2 p)"
 proof (induct p)
   case Nil thus ?case by simp
 next   
   case (Cons x xs) 
      have wp_xs: "wellformed_policy2 (removeShadowRules2 xs)" using prems by (metis ANDConc distinct.simps singleCombinatorsConc) 
      show ?case
      proof (cases x) 
         case DenyAll thus ?thesis using wp_xs by simp
      next  
         case (DenyAllFromTo a b) thus ?thesis
           using prems wp_xs by (simp,metis Cons DenyAllFromTo aux aux7 tNDComm mem_def deny_dom)
      next
         case (AllowPortFromTo a b p) thus ?thesis
             using prems wp_xs by (simp, metis aux26 AllowPortFromTo Cons(4) aux aux7a mem_def tNDComm)
      next
         case (Conc a b) thus ?thesis
             using prems by (metis Conc Cons(2) singleCombinators.simps(2))
      qed
 qed

lemma wellformed1_sorted[simp]: 
  assumes wp1: "wellformed_policy1 p" 
  shows        "wellformed_policy1 (sort p l)"
 proof (cases p)
   case Nil thus ?thesis by simp
 next
   case (Cons x xs) thus ?thesis
    proof (cases "x = DenyAll") 
      case True thus ?thesis using prems by simp 
      next
      case False thus ?thesis using prems  
        by (metis Cons set_sort False  waux2 wellformed_eq wellformed_policy1_strong.simps(2)) 
    qed
 qed

lemma SC1[simp]: "singleCombinators p \<Longrightarrow> singleCombinators (removeShadowRules1 p)"
by (erule SCSubset) (rule SR1Subset)

lemma SC2[simp]: "singleCombinators p \<Longrightarrow> singleCombinators (removeShadowRules2 p)"
by (erule SCSubset) (rule RS2Set)


lemma SC3[simp]: "singleCombinators p \<Longrightarrow> singleCombinators (sort p l)"
by (erule SCSubset) simp

lemma aND_RS1[simp]: "\<lbrakk>singleCombinators p; allNetsDistinct p\<rbrakk> \<Longrightarrow> allNetsDistinct (removeShadowRules1 p)"
apply (rule aNDSubset)
apply (erule SC_RS1, simp_all)
apply (rule SR1Subset)
done

lemma aND_RS2[simp]: "\<lbrakk>singleCombinators p; allNetsDistinct p\<rbrakk> \<Longrightarrow> allNetsDistinct (removeShadowRules2 p)"
apply (rule aNDSubset)
apply (erule SC2, simp_all)
apply (rule RS2Set)
done
 
lemma aND_sort[simp]: "\<lbrakk>singleCombinators p; allNetsDistinct p\<rbrakk> \<Longrightarrow>  allNetsDistinct (sort p l)"
apply (rule aNDSubset)
by (erule SC3, simp_all)

lemma inRS2[rule_format,simp]: "x \<notin> set p \<longrightarrow> x \<notin> set (removeShadowRules2 p)"
apply (insert RS2Set [of p])
by blast

lemma distinct_RS2[rule_format,simp]: "distinct p \<longrightarrow> distinct (removeShadowRules2 p)"
apply (induct p)
apply simp_all
apply clarify
apply (case_tac "a")
by auto

lemma setPaireq: " {x, y} = {a, b} \<Longrightarrow> x = a \<and> y = b \<or> x = b \<and> y = a"
by (metis Un_empty_left Un_insert_left doubleton_eq_iff)

lemma position_positive[rule_format]: "a \<in> set l \<longrightarrow> position a l > 0"
by (induct l, simp_all)

lemma pos_noteq[rule_format]: 
  "a \<in> set l  \<longrightarrow> b  \<in> set l  \<longrightarrow>    c \<in> set l \<longrightarrow> a \<noteq> b \<longrightarrow> 
   (position a l) <= (position b l) \<longrightarrow>   
    (position b l) <= (position c l) \<longrightarrow> 
       a \<noteq> c"
apply (induct l)
apply simp_all
apply (rule conjI)
apply (rule impI)+
apply (simp add: position_positive)+
apply (metis gr_implies_not0 position_positive)
done

lemma setPair_noteq: "{a,b} \<noteq> {c,d} \<Longrightarrow> \<not> ((a = c) \<and> (b = d))" 
by auto

lemma setPair_noteq_allow: "{a,b} \<noteq> {c,d} \<Longrightarrow> \<not> ((a = c) \<and> (b = d) \<and> P)" 
by auto

lemma order_trans:  
 "\<lbrakk>in_list x l; in_list y l; in_list z l; singleCombinators [x]; singleCombinators [y]; singleCombinators [z]; smaller x y l; smaller y z l\<rbrakk> \<Longrightarrow> 
  smaller x z l"
apply (case_tac x)
apply simp_all
apply (case_tac z)
apply simp_all
apply (case_tac y)
apply simp_all
apply (case_tac y)
apply simp_all
apply (rule conjI|rule impI)+
apply (rule setPaireq,simp)
apply (rule conjI|rule impI)+
apply (simp_all split: if_splits)
apply metis
apply metis
apply (simp add: setPair_noteq)
apply (rule impI, simp_all)
apply (erule setPaireq)
apply (rule impI)
apply (case_tac y, simp_all)
apply (simp_all split: if_splits)
apply metis
apply (simp_all add: setPair_noteq setPair_noteq_allow)
apply (case_tac z)
apply simp_all
apply (case_tac y)
apply simp_all
apply (case_tac y)
apply simp_all
apply (rule impI|rule conjI)+
apply (simp_all split: if_splits)
apply (simp add: setPair_noteq)
apply (erule pos_noteq)
apply simp_all
apply (rule impI)
apply (simp add: setPair_noteq)
apply (rule conjI)
apply (simp add: setPair_noteq_allow)
apply (erule pos_noteq, simp_all)
apply (rule impI)
apply (simp add: setPair_noteq_allow)
apply (rule impI)
apply (rule disjI2)
apply (case_tac y, simp_all)
apply (simp_all split: if_splits)
apply metis
apply (simp_all add: setPair_noteq_allow)
done

lemma sortedConcStart[rule_format]: 
  "sorted (a # aa # p) l \<longrightarrow> in_list a l \<longrightarrow> in_list aa l \<longrightarrow> all_in_list p l \<longrightarrow> singleCombinators [a] \<longrightarrow> singleCombinators [aa] \<longrightarrow> singleCombinators p \<longrightarrow> sorted (a#p) l"
apply (induct p)
apply simp_all
apply (rule impI)+
apply simp
apply (rule_tac y = "aa" in order_trans)
apply simp_all
apply (case_tac ab, simp_all)
done

lemma singleCombinatorsStart[simp]: "singleCombinators (x#xs) \<Longrightarrow> singleCombinators [x]"
by (case_tac x, simp_all)


lemma sorted_is_smaller[rule_format]: "sorted (a # p) l \<longrightarrow> in_list a l \<longrightarrow> in_list b l \<longrightarrow> all_in_list p l \<longrightarrow>  singleCombinators [a] \<longrightarrow> singleCombinators p \<longrightarrow> b \<in> set p \<longrightarrow> smaller a b l"
apply (induct p)
apply (auto intro: singleCombinatorsConc sortedConcStart) 
done

lemma sortedConcEnd[rule_format]: "sorted (a # p) l \<longrightarrow> in_list a l \<longrightarrow> all_in_list p l \<longrightarrow> singleCombinators [a] \<longrightarrow> singleCombinators p  \<longrightarrow> sorted p l"
apply (induct p)
apply (auto intro: singleCombinatorsConc sortedConcStart)
done

lemma AD_aux: "\<lbrakk>AllowPortFromTo a b po \<in> set p ;DenyAllFromTo c d \<in> set p; 
                allNetsDistinct  p ; singleCombinators p; 
        a \<noteq> c \<or> b \<noteq> d\<rbrakk>
    \<Longrightarrow> dom (C (AllowPortFromTo a b po)) \<inter> dom (C (DenyAllFromTo c d)) = {}"
apply (rule aux26)
apply (rule_tac x = "AllowPortFromTo a b po" and y = "DenyAllFromTo c d" in tND)
apply auto 
done

lemma in_set_in_list[rule_format]: "a \<in> set p \<longrightarrow> all_in_list p l \<longrightarrow> in_list a l"
by (induct p) auto

lemma sorted_WP2[rule_format]: "sorted p l \<longrightarrow> all_in_list p l \<longrightarrow> distinct p \<longrightarrow> allNetsDistinct p \<longrightarrow> singleCombinators p \<longrightarrow> wellformed_policy2 p"
proof (induct p)
 case Nil thus ?case by simp 
next
 case (Cons a p) thus ?case
  proof (cases a)
   case DenyAll thus ?thesis using prems by (auto intro: ANDConc singleCombinatorsConc sortedConcEnd)
  next
   case (DenyAllFromTo c d) thus ?thesis using prems
      apply simp
      apply (rule impI)+
      apply (rule conjI)
      apply (rule allI)+
      apply (rule impI)+
      apply (rule deny_dom)
      apply (auto intro: aux7 tNDComm ANDConc singleCombinatorsConc sortedConcEnd) 
      done
   next
   case (AllowPortFromTo c d e) thus ?thesis using prems
      apply simp
      apply (rule impI|rule conjI|rule allI)+
      apply (rule aux26)
      apply (rule_tac x = "AllowPortFromTo c d e" and y = "DenyAllFromTo aa b" in tND)
      apply (assumption,simp_all)
      apply (subgoal_tac "smaller (AllowPortFromTo c d e) (DenyAllFromTo aa b) l") 	
      apply (simp split: if_splits)
      apply metis
      apply (erule sorted_is_smaller)
      apply simp_all
      apply (metis List.set.simps(2) bothNet.simps(2) in_list.simps(2) in_set_in_list mem_def set_empty2)
      apply (auto intro: aux7 tNDComm ANDConc singleCombinatorsConc sortedConcEnd) 
      done       
   next
   case (Conc a b) thus ?thesis using prems by simp 
   qed
qed
   
lemma sorted_Consb[rule_format]: "all_in_list (x#xs) l \<longrightarrow> singleCombinators (x#xs) \<longrightarrow> (sorted xs l & (ALL y:set xs. smaller x y l)) \<longrightarrow>  (sorted (x#xs) l) "
apply(induct xs arbitrary: x) 
apply simp
apply (auto simp: order_trans)
done

lemma sorted_Cons: "\<lbrakk>all_in_list (x#xs) l; singleCombinators (x#xs)\<rbrakk> \<Longrightarrow> (sorted xs l & (ALL y:set xs. smaller x y l)) =  (sorted (x#xs) l)"
apply auto
apply (rule sorted_Consb, simp_all)
apply (metis singleCombinatorsConc singleCombinatorsStart sortedConcEnd)
apply (erule sorted_is_smaller)
apply (auto intro: singleCombinatorsConc singleCombinatorsStart in_set_in_list)
done

lemma smaller_antisym: "\<lbrakk>\<not> smaller a b l; in_list a l; in_list b l; singleCombinators[a]; singleCombinators [b]\<rbrakk> \<Longrightarrow>  smaller b a l"
apply (case_tac a)
apply simp_all
apply (case_tac b)
apply simp_all
apply (simp_all split: if_splits)
apply (rule setPaireq)
apply simp
apply (case_tac b)
apply simp_all
apply (simp_all split: if_splits)
done

lemma set_insort_insert: "set (insort x xs l) \<subseteq> insert x (set xs)"
by (induct xs) (auto simp: set_insert)

lemma all_in_listSubset[rule_format]: "all_in_list b l \<longrightarrow> singleCombinators a \<longrightarrow> set a \<subseteq> set b \<longrightarrow> all_in_list a l"
by (induct_tac a) (auto intro: in_set_in_list singleCombinatorsConc)

lemma singleCombinators_insort: "\<lbrakk>singleCombinators [x]; singleCombinators xs\<rbrakk> \<Longrightarrow> singleCombinators (insort x xs l)"
by (metis SCSubset SCConca FWCompilationProof.set_insort set.simps(2) subset_refl)

lemma all_in_list_insort: "\<lbrakk>all_in_list xs l; singleCombinators (x#xs); in_list x l\<rbrakk> \<Longrightarrow>  all_in_list (insort x xs l) l"
apply (rule_tac b = "x#xs" in all_in_listSubset)
apply simp_all
apply (metis singleCombinatorsConc singleCombinatorsStart singleCombinators_insort)
apply (rule set_insort_insert)
done

lemma sorted_ConsA:"\<lbrakk>all_in_list (x#xs) l; singleCombinators (x#xs)\<rbrakk> \<Longrightarrow> (sorted (x#xs) l)  = (sorted xs l & (ALL y:set xs. smaller x y l))"
by (metis sorted_Cons)

lemma is_in_insort: "y \<in> set xs \<Longrightarrow> y \<in> set (insort x xs l)"
by (metis ListMem_iff insert mem_def set_insort set.simps(2))

lemma sorted_insorta[rule_format]: " sorted (insort x xs l) l \<longrightarrow> all_in_list (x#xs) l \<longrightarrow> distinct (x#xs) \<longrightarrow> singleCombinators [x] \<longrightarrow> singleCombinators xs \<longrightarrow> sorted xs l"
apply (induct xs)
apply simp_all
apply (rule impI)+
apply simp
apply (auto intro: is_in_insort sorted_ConsA set_insort singleCombinators_insort singleCombinatorsConc sortedConcEnd all_in_list_insort)
apply (metis sort.simps(2) set_sort SCSubset all_in_list_insort set_subset_Cons singleCombinators.simps(3) singleCombinatorsConc singleCombinatorsStart singleCombinators_insort sortedConcEnd)
apply (rule sorted_Consb)
apply simp_all
apply (rule ballI)
apply (rule_tac p = "insort x xs l" in sorted_is_smaller)
apply (auto intro: in_set_in_list all_in_listSubset singleCombinators_insort singleCombinatorsConc set_insort_insert is_in_insort)
apply (rule_tac b = "x#xs" in all_in_listSubset)
apply simp_all
apply (erule singleCombinators_insort)
apply (erule singleCombinatorsConc)
apply (rule set_insort_insert)
done

lemma sorted_insortb[rule_format]: "sorted xs l \<longrightarrow> all_in_list (x#xs) l \<longrightarrow> distinct (x#xs) \<longrightarrow> singleCombinators [x] \<longrightarrow> singleCombinators xs \<longrightarrow> sorted (insort x xs l) l"
apply (induct xs)
apply simp_all
apply (rule impI)+
apply (subgoal_tac "sorted (FWCompilation.insort x xs l) l")
apply simp
defer 1
apply (metis FWCompilationProof.sorted_Cons all_in_list.simps(2) singleCombinatorsConc)
apply (rule sorted_Consb)
apply simp_all
apply auto
apply (rule_tac b = "x#xs" in all_in_listSubset)
apply simp_all
apply (rule singleCombinators_insort, simp_all)
apply (erule singleCombinatorsConc)
apply (rule set_insort_insert)
apply (metis SCConca singleCombinatorsConc singleCombinatorsStart singleCombinators_insort)
apply (case_tac "y = x")
apply simp_all
apply (rule smaller_antisym)
apply simp_all
apply (subgoal_tac "y \<in> set xs")
apply (auto intro: in_set_in_list all_in_list_insort aux0_1 singleCombinatorsConc aux79 sorted_is_smaller smaller_antisym)
done

lemma sorted_insort: "\<lbrakk>all_in_list (x#xs) l; distinct(x#xs); singleCombinators [x]; singleCombinators xs\<rbrakk> \<Longrightarrow>
                  sorted (insort x xs l) l = sorted xs l"
by (auto intro: sorted_insorta sorted_insortb)

lemma distinct_insort: "distinct (insort x xs l) = (x \<notin> set xs \<and> distinct xs)"
by(induct xs)(auto simp:set_insort)

lemma distinct_sort[simp]: "distinct (sort xs l) = distinct xs"
by(induct xs)(simp_all add:distinct_insort)


lemma sort_is_sorted[rule_format]: "all_in_list p l \<longrightarrow> distinct p \<longrightarrow> singleCombinators p \<longrightarrow> sorted (sort p l) l"
apply (induct p)
apply (auto intro: SC3 all_in_listSubset SC3 singleCombinatorsConc sorted_insort)
apply (subst sorted_insort)
apply (auto intro: singleCombinatorsConc all_in_listSubset SC3)
apply (erule all_in_listSubset, auto intro: SC3 singleCombinatorsConc sorted_insort)
done

lemma wellformed2_sorted[simp]: "\<lbrakk>all_in_list p l; distinct p; allNetsDistinct p; singleCombinators p\<rbrakk> \<Longrightarrow> wellformed_policy2 (sort p l)"
apply (rule sorted_WP2)
apply (erule sort_is_sorted, simp_all)
apply (erule all_in_listSubset, auto intro: SC3 singleCombinatorsConc sorted_insort)
done

lemma inSet_not_MT: "a \<in> set p \<Longrightarrow> p \<noteq> []"
by auto 

lemma C_DenyAll[simp]: "C (list2policy (xs @ [DenyAll])) x = Some (deny x)"
by (auto simp: PLemmas)

lemma RS1n_assoc: "x \<noteq> DenyAll
           \<Longrightarrow>  removeShadowRules1_alternative xs @ [x] = removeShadowRules1_alternative (xs @ [x])"
by (simp add: removeShadowRules1_alternative_def aux114)

lemma RS1n_nMT[rule_format,simp]: "p \<noteq> [] \<longrightarrow> removeShadowRules1_alternative p \<noteq> []"
apply (simp add: removeShadowRules1_alternative_def)
apply (rule_tac xs = p in rev_induct, simp_all) 
apply (case_tac "xs = []", simp_all)
apply (case_tac x, simp_all)
apply (rule_tac xs = "xs" in rev_induct, simp_all)
apply (case_tac x, simp_all)+
done

lemma RS1N_DA[simp]: "removeShadowRules1_alternative (a@[DenyAll]) = [DenyAll]"
by (simp add: removeShadowRules1_alternative_def)

lemma C_eq_RS1n: " C(list2policy (removeShadowRules1_alternative p)) = C(list2policy p)"
apply (case_tac "p = []")
apply simp_all
apply (metis rSR1_eq removeShadowRules1.simps(2))
apply (rule rev_induct)
apply (metis rSR1_eq removeShadowRules1.simps(2))
apply (case_tac "xs = []", simp_all)
apply (simp add: removeShadowRules1_alternative_def)
apply (case_tac x, simp_all)
apply (rule ext) 
apply (case_tac "x = DenyAll")
apply (simp_all add: C_DenyAll PLemmas)
apply (rule_tac t = "removeShadowRules1_alternative (xs @ [x])" and s = "(removeShadowRules1_alternative xs)@[x]" in subst)
apply (erule RS1n_assoc)
apply (case_tac "xa \<in> dom (C x)")
apply simp_all
done

lemma C_eq_RS1[simp]: "p \<noteq> [] \<Longrightarrow> C(list2policy (removeShadowRules1 p)) = C(list2policy p)"
by (metis rSR1_eq C_eq_RS1n)


lemma EX_MR_aux[rule_format]: "matching_rule x (DenyAll # p) \<noteq> Some DenyAll \<longrightarrow> (\<exists>y. matching_rule x p = Some y)"
apply (simp add: matching_rule_def)
apply (rule_tac xs = p in rev_induct, simp_all)  
done

lemma EX_MR : "\<lbrakk>matching_rule x p \<noteq> (Some DenyAll); p = DenyAll#ps\<rbrakk> \<Longrightarrow> 
                           (matching_rule x p = matching_rule x ps)"
apply auto
apply (subgoal_tac "matching_rule x (DenyAll#ps) \<noteq> None")
apply auto
apply (metis mrConcEnd the.simps)
apply (metis DAimpliesMR_E is_in_insort saux wellformed_policy1_strong.simps(2) wp1_auxa)
done

lemma mr_not_DA: "\<lbrakk>wellformed_policy1_strong s; matching_rule x p = Some (DenyAllFromTo a ab); set p = set s\<rbrakk> \<Longrightarrow>
               matching_rule x s \<noteq> Some DenyAll"
apply (subst wp1n_tl, simp_all)
apply (subgoal_tac "x \<in> dom (C (DenyAllFromTo a ab))")
apply (subgoal_tac "DenyAllFromTo a ab \<in> set (tl s)")
apply (metis wp1n_tl foo98 wellformed_policy1_strong.simps(2))
apply (erule r_not_DA_in_tl, simp_all)
apply (subgoal_tac "DenyAllFromTo a ab \<in> set p", simp)
apply (erule mrSet)
apply (erule mr_in_dom)
done

lemma domsMT_notND_DD: " \<lbrakk>dom (C (DenyAllFromTo a b)) \<inter> dom (C (DenyAllFromTo c d)) \<noteq> {}\<rbrakk> \<Longrightarrow> \<not> netsDistinct a c"
apply (erule contrapos_nn)
apply (simp add: C.simps)
apply (rule aux6)
apply (simp add: twoNetsDistinct_def)
done

lemma WP1n_DA_notinSet[rule_format]: "wellformed_policy1_strong p \<longrightarrow> DenyAll \<notin> set (tl p)"
by (induct p) (simp_all) 

lemma domsMT_notND_DD2: " \<lbrakk>dom (C (DenyAllFromTo a b)) \<inter> dom (C (DenyAllFromTo c d)) \<noteq> {}\<rbrakk> \<Longrightarrow> \<not> netsDistinct b d"
apply (erule contrapos_nn)
apply (simp add: C.simps)
apply (rule aux6)
apply (simp add: twoNetsDistinct_def)
done

lemma domsMT_notND_DD3: " \<lbrakk>x \<in> dom (C (DenyAllFromTo a b)); x \<in> dom (C (DenyAllFromTo c d))\<rbrakk> \<Longrightarrow> \<not> netsDistinct a c"
apply (rule domsMT_notND_DD)
apply auto
done

lemma domsMT_notND_DD4: " \<lbrakk>x \<in> dom (C (DenyAllFromTo a b)); x \<in> dom (C (DenyAllFromTo c d))\<rbrakk> \<Longrightarrow> \<not> netsDistinct b d"
apply (rule domsMT_notND_DD2)
apply auto
done

lemma NetsEq_if_sameP_DD: "\<lbrakk>allNetsDistinct p; u\<in> set p; v\<in> set p; u = (DenyAllFromTo a b); v = (DenyAllFromTo c d); x \<in> dom (C (u)); x \<in> dom (C (v))\<rbrakk>\<Longrightarrow>       a = c \<and> b = d"
apply (simp add: allNetsDistinct_def)
apply (metis ND0aux1 ND0aux2 domsMT_notND_DD3 domsMT_notND_DD4 mem_def)
done

lemma mt_sym: "dom a \<inter> dom b = {} \<Longrightarrow> dom b \<inter> dom a = {}"
by auto

lemma rule_charn1: 
 assumes aND: "allNetsDistinct p"
 and mr_is_allow: "matching_rule x p = Some (AllowPortFromTo a b po)"
 and SC: "singleCombinators p"
 and inp: "r \<in> set p" 
 and inDom: "x \<in> dom (C r)"
 shows "(r = AllowPortFromTo a b po \<or> r = DenyAllFromTo a b \<or> r = DenyAll)"
proof (cases r) 
  case DenyAll show ?thesis using prems by simp
next
  case (DenyAllFromTo x y) show ?thesis using prems 
     apply (simp,rule_tac  p = p and po =po in DistinctNetsDenyAllow, simp_all)
     apply (metis mrSet)
     by (metis Int_iff mr_in_dom inSet_not_MT mem_def set_empty2)
next
  case (AllowPortFromTo x y b) show ?thesis using prems
     apply simp
     apply (rule DistinctNetsAllowAllow, simp_all)
     apply (metis mrSet)
     by (metis Int_iff mr_in_dom inSet_not_MT mem_def set_empty2)
next
  case (Conc x y) thus ?thesis using prems by (metis aux0_0)
qed

lemma DAnotTL[rule_format]: "xs \<noteq> [] \<longrightarrow> wellformed_policy1 (xs @ [DenyAll]) \<longrightarrow> False"
by (induct xs, simp_all)

lemma nMTRS3[simp]: "noneMT (removeShadowRules3 p)"
by (induct p) simp_all

lemma nMTcharn: "noneMT p = (\<forall> r \<in> set p. dom (C r) \<noteq> {})"
by (induct p) simp_all

lemma nMTeqSet: "set p = set s \<Longrightarrow> noneMT p = noneMT s"
by (simp add: nMTcharn)

lemma nMTSort: "noneMT p \<Longrightarrow> noneMT (sort p l)"
by (metis set_sort nMTeqSet)

lemma wp3char[rule_format]: "noneMT xs \<and>  dom (C (AllowPortFromTo a b po)) \<noteq> {} \<and> wellformed_policy3 (xs @ [DenyAllFromTo a b]) \<longrightarrow> AllowPortFromTo a b po \<notin> set xs"
apply (induct xs)
apply simp_all
apply (metis wp3Conc Int_absorb1 Int_commute allow_deny_dom in_set_conv_decomp mem_def not_Cons_self removeShadowRules2.simps(1) set_empty2 wellformed_policy3.simps(2))
done

lemma wp3charn[rule_format]: 
assumes domAllow: "dom (C (AllowPortFromTo a b po)) \<noteq> {}" 
and wp3: "wellformed_policy3 (xs @ [DenyAllFromTo a b])"
shows allowNotInList: "AllowPortFromTo a b po \<notin> set xs"
apply (insert prems)
proof (induct xs)
 case Nil show ?case by simp
next
 case (Cons x xs) show ?case using prems
 by (simp,auto intro: wp3Conc) (auto simp: DenyAllowDisj domAllow)
qed

lemma notMTnMT: "\<lbrakk>a \<in> set p; noneMT p\<rbrakk> \<Longrightarrow> dom (C a) \<noteq> {}"
by (simp add: nMTcharn)

lemma noneMTconc[rule_format]: "noneMT (a@[b]) \<longrightarrow> noneMT a"
by (induct a, simp_all)

lemma rule_charn2: 
 assumes aND: "allNetsDistinct p"
 and wp1: "wellformed_policy1 p"
 and SC: "singleCombinators p"
 and wp3: "wellformed_policy3 p"
 and allow_in_list: "AllowPortFromTo c d po \<in> set p"
 and x_in_dom_allow: "x \<in> dom (C (AllowPortFromTo c d po))"
 shows  "matching_rule x p = Some (AllowPortFromTo c d po)"
 proof  (insert prems, induct p rule: rev_induct)    
   case Nil show ?case using prems by simp
 next 
   case (snoc y ys) show ?case using prems
     apply simp
     apply (case_tac "y = (AllowPortFromTo c d po)")
     apply (simp add: matching_rule_def)
     apply simp_all
     apply (subgoal_tac "ys \<noteq> []")
     apply (subgoal_tac "matching_rule x ys = Some (AllowPortFromTo c d po)")
     defer 1
     apply (metis ANDConcEnd SCConcEnd WP1ConcEnd foo25 snoc(2) snoc(3) snoc(4) snoc(5))
apply (metis inSet_not_MT)
     proof (cases y)
      case DenyAll thus ?thesis using prems      
            apply simp
            by (metis DAnotTL DenyAll inSet_not_MT mem_def policy2list.simps(2))
       next
      case (DenyAllFromTo a b) thus ?thesis using prems apply simp
     apply (simp_all add: matching_rule_def)
     apply (rule conjI)
     apply (metis domInterMT  wp3EndMT)
     apply (rule impI)
     by (metis ANDConcEnd DenyAllFromTo SCConcEnd WP1ConcEnd foo25)
       next
     case (AllowPortFromTo a1 a2 b) thus ?thesis using prems apply simp
     apply (simp_all add: matching_rule_def)
     apply (rule conjI)
     apply (metis domInterMT  wp3EndMT)
     by (metis ANDConcEnd AllowPortFromTo SCConcEnd WP1ConcEnd foo25 x_in_dom_allow)
     next
     case (Conc a b) thus ?thesis using prems apply simp
        by (metis Conc aux0_0 in_set_conv_decomp) 
     qed
qed

lemma rule_charn3: 
       "\<lbrakk>wellformed_policy1 p; allNetsDistinct p; singleCombinators p; wellformed_policy3 p;
        matching_rule x p = Some (DenyAllFromTo c d); AllowPortFromTo a b po \<in> set p\<rbrakk> \<Longrightarrow>
           x \<notin> dom (C (AllowPortFromTo a b po))"
by (clarify, auto simp: rule_charn2 dom_def) 

lemma rule_charn4: 
assumes wp1: "wellformed_policy1 p" 
and aND: "allNetsDistinct p" 
and SC: "singleCombinators p"
and wp3: "wellformed_policy3 p"  
and DA: "DenyAll \<notin> set p" 
and mr: "matching_rule x p = Some (DenyAllFromTo a b)"
and rinp: "r \<in> set p"
and xindom: "x \<in> dom (C r)"
shows "r = DenyAllFromTo a b"
proof (cases r)
  case DenyAll thus ?thesis using prems by simp 
next
  case (DenyAllFromTo c d) thus ?thesis using prems apply simp
    apply (erule_tac x = x and p = p and v = "(DenyAllFromTo a b)" and u = "(DenyAllFromTo c d)"  in NetsEq_if_sameP_DD)
    apply simp_all
    apply (erule mrSet)
    by (erule mr_in_dom) 
next
  case (AllowPortFromTo c d e) thus ?thesis using prems apply simp
    apply (subgoal_tac "x \<notin> dom (C  (AllowPortFromTo c d e))")
    apply simp 
    apply (rule_tac p = p in rule_charn3)
    by (auto intro: SCnotConc) 
next
  case (Conc a b) thus ?thesis using prems apply simp
      by (metis Conc aux0_0 in_set_conv_decomp) 
qed


lemma AND_tl[rule_format]: "allNetsDistinct ( p) \<longrightarrow> allNetsDistinct (tl p)"
apply (induct p, simp_all)
by (auto intro: ANDConc)


lemma distinct_tl[rule_format]: "distinct p \<longrightarrow> distinct (tl p)"
by (induct p, simp_all)


lemma SC_tl[rule_format]: "singleCombinators ( p) \<longrightarrow> singleCombinators (tl p)"
apply (induct p, simp_all)
by (auto intro: singleCombinatorsConc) 

lemma Conc_not_MT: "p = x#xs \<Longrightarrow> p \<noteq> []"
by auto

lemma wp1_tl[rule_format]: "p \<noteq> [] \<and> wellformed_policy1 p \<longrightarrow> wellformed_policy1 (tl p)"
apply (induct p)
apply simp_all
apply (auto intro: waux2)
done

lemma nMTtail[rule_format]: "noneMT p \<longrightarrow> noneMT (tl p)"
by (induct p, simp_all)

lemma foo31a: "\<lbrakk>(\<forall> r. r \<in> set p \<and> x \<in> dom (C r) \<longrightarrow> (r = AllowPortFromTo a b po \<or> r = DenyAllFromTo a b \<or> r = DenyAll)); 
                set p = set s; r \<in> set s ; x \<in> dom (C r)\<rbrakk> \<Longrightarrow> (r = AllowPortFromTo a b po \<or> r = DenyAllFromTo a b \<or> r = DenyAll)"
by auto

lemma wp1_eq[rule_format]: "wellformed_policy1_strong p \<Longrightarrow> wellformed_policy1 p"
apply (case_tac "DenyAll \<in> set p")
apply (subst wellformed_eq)
apply simp_all
apply (erule waux2)
done

lemma aux4[rule_format]: 
  "matching_rule x (a#p) = Some a \<longrightarrow> a \<notin> set (p) \<longrightarrow> matching_rule x p = None"
apply (rule rev_induct)
apply simp_all
apply (rule impI)+
apply simp
apply (simp add: matching_rule_def)
apply (simp split: if_splits)
done

lemma mrDA_tl: 
 assumes mr_DA: "matching_rule x p = Some DenyAll"
 and wp1n: "wellformed_policy1_strong p"
 shows "matching_rule x (tl p) = None"
 apply (rule aux4 [where a = DenyAll])
 apply (metis wp1n_tl mr_DA wp1n)
 by (metis WP1n_DA_notinSet wp1n)

lemma rule_charnDAFT: 
"\<lbrakk>wellformed_policy1_strong p; allNetsDistinct p; singleCombinators p; wellformed_policy3 p;  
  matching_rule x p = Some (DenyAllFromTo a b); r \<in> set (tl p); x \<in> dom (C r)\<rbrakk> \<Longrightarrow> r = DenyAllFromTo a b"
apply (subgoal_tac "p = DenyAll#(tl p)")
apply (rule_tac p = "tl p" in rule_charn4)
apply simp_all
apply (metis wellformed_policy1_strong.simps(1) wp1_eq wp1_tl)
apply (erule AND_tl)
apply (erule SC_tl)
apply (erule wp3tl)
apply (erule WP1n_DA_notinSet)
apply (metis Combinators.simps(1) DAAux EX_MR matching_rule_def matching_rule_rev.simps(1) mem_def mrSet option.inject rev_rev_ident set_rev tl.simps(2) wellformed_policy1_charn wp1_eq)
apply (metis  wp1n_tl) 
done

lemma mrDenyAll_is_unique: "\<lbrakk>wellformed_policy1_strong p; matching_rule x p = Some DenyAll; r \<in> set (tl p)\<rbrakk> \<Longrightarrow> x \<notin> dom (C r)"
apply (rule_tac a = "[]" and b = "DenyAll" and c = "tl p"  in foo3a, simp_all)  
apply (metis wp1n_tl)
by (metis WP1n_DA_notinSet)

theorem  C_eq_Sets_mr: 
 assumes sets_eq: "set p = set s"
 and SC: "singleCombinators p"
 and wp1_p: "wellformed_policy1_strong p"
 and wp1_s: "wellformed_policy1_strong s"
 and wp3_p: "wellformed_policy3 p"       
 and wp3_s: "wellformed_policy3 s"  
 and aND: "allNetsDistinct p"        
   
 shows "matching_rule x p = matching_rule x s"
proof (cases "matching_rule x p")

 case None 
     have DA: "DenyAll \<in> set p" using wp1_p by (auto simp: wp1_aux1aa)
     have notDA: "DenyAll \<notin> set p" using None by (auto simp: DAimplieMR) 
     thus ?thesis using DA by (contradiction)
 next
 
 case (Some y) thus ?thesis
   proof (cases y) 
      have tl_p: "p = DenyAll#(tl p)" by (metis wp1_p wp1n_tl) 
      have tl_s: "s = DenyAll#(tl s)" by (metis wp1_s wp1n_tl)
      have tl_eq: "set (tl p) = set (tl s)"
       by (metis tl.simps(2) WP1n_DA_notinSet foo2 mem_def sets_eq wellformed_policy1_charn wp1_aux1aa wp1_eq wp1_p wp1_s)
    {
   case DenyAll
     have mr_p_is_DenyAll: "matching_rule x p = Some DenyAll" by (simp add: DenyAll Some) 
     hence x_notin_tl_p: "\<forall> r. r \<in> set (tl p) \<longrightarrow>  x \<notin> dom (C r)" using wp1_p by (auto simp: mrDenyAll_is_unique)
     hence x_notin_tl_s: "\<forall> r. r \<in> set (tl s) \<longrightarrow>  x \<notin> dom (C r)" using tl_eq by auto
     hence mr_s_is_DenyAll: "matching_rule x s = Some DenyAll" using tl_s by (auto simp: mr_first)
     thus ?thesis using mr_p_is_DenyAll by simp
}
{   case (DenyAllFromTo a b) 
     have mr_p_is_DAFT: "matching_rule x p = Some (DenyAllFromTo a b)" by (simp add: DenyAllFromTo Some)
     have DA_notin_tl: "DenyAll \<notin> set (tl p)" by (metis WP1n_DA_notinSet wp1_p)    
     have mr_tl_p: "matching_rule x p = matching_rule x (tl p)" by (metis Combinators.simps(1) DenyAllFromTo Some mrConcEnd tl_p)
     have dom_tl_p: "\<And> r. r \<in> set (tl p) \<and> x \<in> dom (C r) \<Longrightarrow> r = (DenyAllFromTo a b)" using wp1_p aND SC wp3_p mr_p_is_DAFT
       by (auto simp: rule_charnDAFT)
     hence dom_tl_s: "\<And> r. r \<in> set (tl s) \<and> x \<in> dom (C r) \<Longrightarrow> r = (DenyAllFromTo a b)" using tl_eq  by auto
     have DAFT_in_tl_s: "DenyAllFromTo a b \<in> set (tl s)" using mr_tl_p by (metis DenyAllFromTo mrSet mr_p_is_DAFT tl_eq) 
     have x_in_dom_DAFT: "x \<in> dom (C (DenyAllFromTo a b))" by (metis mr_p_is_DAFT DenyAllFromTo mr_in_dom)
     hence mr_tl_s_is_DAFT: "matching_rule x (tl s) = Some (DenyAllFromTo a b)" using DAFT_in_tl_s dom_tl_s  by (auto simp: mr_charn)
     hence mr_s_is_DAFT: "matching_rule x s = Some (DenyAllFromTo a b)" using tl_s 
          by (metis  DA_notin_tl DenyAllFromTo EX_MR mrDA_tl mr_p_is_DAFT not_Some_eq tl_eq wellformed_policy1_strong.simps(2))
     thus ?thesis using mr_p_is_DAFT by simp
}
{  case (AllowPortFromTo a b c)
     have wp1s: "wellformed_policy1 s" by (metis wp1_eq wp1_s)
     have mr_p_is_A: "matching_rule x p = Some (AllowPortFromTo a b c)" by (simp add: AllowPortFromTo Some)
     hence A_in_s: "AllowPortFromTo a b c \<in> set s" using sets_eq by (auto intro: mrSet) 
     have x_in_dom_A: "x \<in> dom (C (AllowPortFromTo a b c))" by (metis mr_p_is_A AllowPortFromTo mr_in_dom)
     have SCs: "singleCombinators s" using SC sets_eq by (auto intro: SCSubset)
     hence ANDs: "allNetsDistinct s" using aND sets_eq SC by (auto intro: aNDSetsEq)
     hence mr_s_is_A: "matching_rule x s = Some (AllowPortFromTo a b c)" using A_in_s wp1s mr_p_is_A aND SCs wp3_s x_in_dom_A
          by (simp add: rule_charn2)
     thus ?thesis using mr_p_is_A by simp
}
    case (Conc a b) thus ?thesis by (metis Some mr_not_Conc SC) 
 qed
qed

lemma C_eq_Sets: 
  "\<lbrakk>singleCombinators p; wellformed_policy1_strong p; wellformed_policy1_strong s; wellformed_policy3 p; wellformed_policy3 s;  
   allNetsDistinct p; set p = set s\<rbrakk> \<Longrightarrow> 
  C (list2policy p) x  = C (list2policy s) x"
  apply (rule C_eq_if_mr_eq)
  apply (rule C_eq_Sets_mr [symmetric])
  apply simp_all
  apply (metis wellformed_policy1_strong.simps(1) wp1_auxa)+
done

lemma wellformed1_alternative_sorted: "wellformed_policy1_strong p \<Longrightarrow> wellformed_policy1_strong (sort p l)"
by (case_tac "p", simp_all)

lemma C_eq_sorted: "\<lbrakk>distinct p; all_in_list p l; singleCombinators p; wellformed_policy1_strong p; wellformed_policy3 p; allNetsDistinct p\<rbrakk> \<Longrightarrow> 
C (list2policy (sort p l))= C (list2policy p)"
 apply (rule ext)
 apply (rule C_eq_Sets)
 apply (auto simp: nMTSort wellformed1_alternative_sorted wellformed_policy3_charn wellformed1_sorted wp1_eq)
done

lemma wp1n_RS2[rule_format]: "wellformed_policy1_strong p \<longrightarrow> wellformed_policy1_strong (removeShadowRules2 p)"
by (induct p, simp_all)

lemma RS2_NMT[rule_format]: "p \<noteq> [] \<longrightarrow> removeShadowRules2 p \<noteq> []"
apply (induct p, simp_all)
apply (case_tac "p \<noteq> []", simp_all)
apply (case_tac "a", simp_all)+
done

lemma mrconc[rule_format]: "matching_rule x p = Some a \<longrightarrow> matching_rule x (b#p) = Some a" 
apply (rule rev_induct)  back 
apply (simp)
apply (rule impI)
apply (case_tac "x \<in> dom (C xa)")
apply (simp_all add: matching_rule_def)
done

lemma mreq_end: "\<lbrakk>matching_rule x b = Some r; matching_rule x c = Some r\<rbrakk> \<Longrightarrow> 
 matching_rule x (a#b) = matching_rule x (a#c)"
by (simp add: mrconc)

lemma mrconcNone[rule_format]: "matching_rule x p = None \<longrightarrow> matching_rule x (b#p) = matching_rule x [b]"
apply (rule_tac xs = p in rev_induct) 
apply simp_all
apply (rule impI)
apply (case_tac "x \<in> dom (C xa)")
apply (simp_all add: matching_rule_def)
done

lemma mreq_endNone: "\<lbrakk>matching_rule x b = None; matching_rule x c = None\<rbrakk> \<Longrightarrow> 
     matching_rule x (a#b) = matching_rule x (a#c)"
by (metis mrconcNone)

lemma mreq_end2: "matching_rule x b = matching_rule x c \<Longrightarrow> 
     matching_rule x (a#b) = matching_rule x (a#c)"
apply (case_tac "matching_rule x b = None")
apply (auto intro: mreq_end mreq_endNone)
done

lemma mreq_end3: "matching_rule x p \<noteq> None \<Longrightarrow> matching_rule x (b # p) = matching_rule x (p)"
by (auto simp: mrconc)

lemma mrNoneMT[rule_format]: "r \<in> set p \<longrightarrow> matching_rule x p = None \<longrightarrow> x \<notin> dom (C r)"
apply (rule rev_induct, simp_all)
apply (rule conjI| rule impI)+
apply simp_all
apply (case_tac "xa \<in> set xs")
apply (simp_all add: matching_rule_def split: if_splits)
done

lemma C_eq_RS2_mr: "matching_rule x (removeShadowRules2 p)= matching_rule x p"
proof (induct p)
 case Nil thus ?case by simp next
 case (Cons y ys) thus ?case
 proof (cases "ys = []")
  case True thus ?thesis by (cases y, simp_all) next
  case False thus ?thesis
   proof (cases y)
    case DenyAll thus ?thesis by (simp, metis Cons DenyAll mreq_end2) next
    case (DenyAllFromTo a b) thus ?thesis by (simp, metis Cons DenyAllFromTo mreq_end2) next
    case (AllowPortFromTo a b p) thus ?thesis
      proof (cases "DenyAllFromTo a b \<in> set ys")
        case True thus ?thesis using prems
          apply (cases "matching_rule x ys = None", simp_all)
          apply (subgoal_tac "x \<notin> dom (C (AllowPortFromTo a b p))")
          apply (subst mrconcNone, simp_all)
          apply (simp add: matching_rule_def )
          apply (rule contra_subsetD [OF allow_deny_dom])
          apply (erule mrNoneMT,simp)
          apply (metis AllowPortFromTo mrconc)
          done
        next
        case False thus ?thesis using prems by (simp, metis AllowPortFromTo Cons mreq_end2) qed
    next
    case (Conc a b) thus ?thesis by (metis Cons mreq_end2 removeShadowRules2.simps(4))
   qed
  qed
qed

lemma wp1_alternative_not_mt[simp]: "wellformed_policy1_strong p \<Longrightarrow> p \<noteq> []"
by auto

lemma C_eq_None[rule_format]: "p \<noteq> []  --> matching_rule x p = None \<longrightarrow>  C (list2policy p) x = None" 
apply (simp add: matching_rule_def)
apply (rule rev_induct, simp_all)
apply (rule impI)+
apply simp
apply (case_tac "xs \<noteq> []")
apply (simp_all add: dom_def)
done

lemma C_eq_None2: "\<lbrakk>a \<noteq> []; b \<noteq> []; matching_rule x a = None; matching_rule x b = None\<rbrakk> \<Longrightarrow>  
  (C (list2policy a)) x = (C (list2policy b)) x"
by (auto simp: C_eq_None)

lemma C_eq_RS2: "wellformed_policy1_strong p \<Longrightarrow> 
 C (list2policy (removeShadowRules2 p))= C (list2policy p)"
apply (rule ext)
apply (rule C_eq_if_mr_eq)
apply (rule C_eq_RS2_mr [symmetric], simp_all)
apply (metis wp1_alternative_not_mt wp1n_RS2)
done

lemma AIL1[rule_format,simp]: "all_in_list p l \<longrightarrow> all_in_list (removeShadowRules1 p) l"
by (induct_tac p, simp_all)

lemma noneMTsubset[rule_format]: "noneMT a \<longrightarrow> set b \<subseteq> set a \<longrightarrow> noneMT b"
by (induct b,auto simp: notMTnMT)

lemma noneMTRS2: "noneMT p \<Longrightarrow> noneMT (removeShadowRules2 p)"
by (auto simp: noneMTsubset RS2Set)

lemma CconcNone: "\<lbrakk>dom (C a) = {}; p \<noteq> []\<rbrakk> \<Longrightarrow> C (list2policy (a # p)) x = C (list2policy p) x"
apply (case_tac "p = []", simp_all)
apply (case_tac "x\<in> dom (C (list2policy(p)))")
apply (metis Cdom2 list2policyconc mem_def)
apply (metis C.simps(4) Cauxb domIff inSet_not_MT list2policyconc set_empty2)
done

lemma notMTpolicyimpnotMT[simp]: "notMTpolicy p \<Longrightarrow> p \<noteq> []"
by auto

lemma SR3nMT[rule_format]: "\<not> notMTpolicy p \<longrightarrow> removeShadowRules3 p = []"
by (induct p, simp_all)

lemma wp1ID: "wellformed_policy1_strong (insertDeny (removeShadowRules1 p))"
by (induct p, simp_all, case_tac a, simp_all)

lemma noneMTrd[rule_format]: "noneMT p \<longrightarrow> noneMT (remdups p)"
by (induct p, simp_all)

lemma DARS3[rule_format]: "DenyAll \<notin> set p \<longrightarrow> DenyAll \<notin> set (removeShadowRules3 p)"
by (induct p, simp_all)

lemma DAnMT: "dom (C DenyAll) \<noteq> {}"
by (simp add: dom_def C.simps PolicyCombinators.PolicyCombinators)

lemma wp1n_RS3[rule_format,simp]: "wellformed_policy1_strong p \<longrightarrow> wellformed_policy1_strong (removeShadowRules3 p)"
apply (induct p, simp_all)
apply (rule conjI| rule impI|simp)+
apply (metis DAAux inSet_not_MT set_empty2)
apply (rule conjI| rule impI|simp)+
apply (metis DARS3) 
done

lemma dRD[simp]: "distinct (remdups p)"
by simp

lemma AILrd[rule_format,simp]: "all_in_list p l \<longrightarrow> all_in_list (remdups p) l"
by (induct p, simp_all)

lemma AILRS3[rule_format,simp]: "all_in_list p l \<longrightarrow> all_in_list (removeShadowRules3 p) l"
by (induct p, simp_all)

lemma AILiD[rule_format,simp]: "all_in_list p l \<longrightarrow> all_in_list (insertDeny p) l"
apply (induct p, simp_all)
apply (rule impI, simp)
apply (case_tac "a", simp_all)
done

lemma SCrd[rule_format,simp]: "singleCombinators p \<longrightarrow> singleCombinators(remdups p)"
apply (induct p, simp_all)
apply (case_tac "a", simp_all)
done

lemma SCRiD[rule_format,simp]: "singleCombinators p \<longrightarrow> singleCombinators(insertDeny p)"
apply (induct p, simp_all)
apply (case_tac "a", simp_all)
done

lemma SCRS3[rule_format,simp]: "singleCombinators p \<longrightarrow> singleCombinators(removeShadowRules3 p)"
apply (induct p, simp_all)
apply (case_tac "a", simp_all)
done

lemma WP1rd[rule_format,simp]: "wellformed_policy1_strong p \<longrightarrow> wellformed_policy1_strong (remdups p)"
apply (induct p, simp_all)
done

lemma ANDrd[rule_format,simp]: "singleCombinators p \<longrightarrow> allNetsDistinct p \<longrightarrow> allNetsDistinct (remdups p)"
apply (rule impI)+
apply (rule_tac b = p in aNDSubset)
apply simp_all
done

lemma RS3subset: "set (removeShadowRules3 p)  \<subseteq> set p "
by (induct p, auto)

lemma ANDRS3[simp]: "\<lbrakk>singleCombinators p; allNetsDistinct p\<rbrakk> \<Longrightarrow> allNetsDistinct (removeShadowRules3 p)"
apply (rule_tac b = p in aNDSubset)
apply simp_all
apply (rule RS3subset)
done

lemma ANDiD[rule_format,simp]: "allNetsDistinct p \<longrightarrow> allNetsDistinct (insertDeny p)"
apply (induct p, simp_all)
apply (simp add: allNetsDistinct_def)
apply (auto intro: ANDConc)
apply (case_tac "a")
apply (simp_all add: allNetsDistinct_def)
done

lemma nlpaux: "x \<notin> dom (C b) \<Longrightarrow> C (a \<oplus> b) x = C a x"
by (simp add: C.simps Cauxb)

lemma notindom[rule_format]: "a \<in> set p \<longrightarrow>  x \<notin> dom (C (list2policy p)) \<longrightarrow> x \<notin> dom (C a)"
apply (induct p)
apply simp_all
apply (rule conjI| rule impI)+
apply (metis CConcStartA)
apply (rule impI)+
apply simp
apply (metis CConcStartA Cdom2 domIff insert_absorb list.simps(1) list2policyconc set.simps(2) set_empty set_empty2)
done

lemma C_eq_rd[rule_format]: "p \<noteq> [] \<Longrightarrow> C (list2policy (remdups p)) = C (list2policy p)"
apply (rule ext)
proof (induct p) 
  case Nil thus ?case by simp next
  case (Cons y ys) thus ?case
    proof (cases "ys = []")
      case True thus ?thesis by simp next
      case False thus ?thesis using prems apply simp
       apply (rule conjI, rule impI)
       apply (cases "x \<in> dom (C (list2policy ys))")
       apply (metis Cdom2 False list2policyconc mem_def)
       apply (metis False domIff list2policyconc mem_def nlpaux notindom)
       apply (rule impI)
       apply (cases "x \<in> dom (C (list2policy ys))")
       apply (subgoal_tac "x \<in> dom (C (list2policy (remdups ys)))")
       apply (metis Cdom2 False list2policyconc mem_def remdups_eq_nil_iff)
       apply (metis domIff)
       apply (subgoal_tac "x \<notin> dom (C (list2policy (remdups ys)))")
       apply (metis False list2policyconc nlpaux remdups_eq_nil_iff)
       apply (metis domIff)
       done
  qed
qed
   
lemma RS3nMT[rule_format]: "notMTpolicy p \<longrightarrow> notMTpolicy (removeShadowRules3 p)"
by (induct p,simp_all)


lemma nMT_domMT: "\<lbrakk>\<not> notMTpolicy p; p \<noteq> []\<rbrakk> \<Longrightarrow> r \<notin> dom (C (list2policy p))"
 proof (induct p)
   case Nil thus ?case by simp next
   case (Cons x xs) thus ?case apply simp
      apply (simp split: if_splits)
      apply (cases "xs = []")
      apply simp_all
      apply (metis CconcNone domIff set_empty2)
   done
qed
            
lemma C_eq_RS3_aux[rule_format]: "notMTpolicy p \<Longrightarrow> C (list2policy p) x = C (list2policy (removeShadowRules3 p)) x"
proof (induct p)
 case Nil thus ?case by simp next
 case (Cons y ys) thus ?case
   proof (cases "notMTpolicy ys")
     case True thus ?thesis using prems apply simp
       apply (rule conjI, rule impI, simp)
       apply (metis CconcNone True notMTpolicyimpnotMT set_empty2)
       apply (rule impI, simp)
       apply (cases "x \<in> dom (C (list2policy ys))")
       apply (subgoal_tac "x \<in> dom (C (list2policy (removeShadowRules3 ys)))")
       apply (metis Cdom2 RS3nMT True list2policyconc mem_def notMTpolicyimpnotMT)
       apply (simp add: domIff)
       apply (subgoal_tac "x \<notin>  dom (C (list2policy (removeShadowRules3 ys)))")
       apply (metis RS3nMT True list2policyconc nlpaux notMTpolicyimpnotMT)
       apply (metis domIff)
     done
     next 
     case False thus ?thesis using prems
        proof (cases "ys = []")
          case True thus ?thesis using prems by (simp) (rule impI, simp) next
          case False thus ?thesis using prems apply (simp)
            apply (rule conjI| rule impI| simp)+
            apply (subgoal_tac "removeShadowRules3 ys = []")
            apply simp_all
            apply (subgoal_tac "x \<notin> dom (C (list2policy ys))")
            apply (metis False list2policyconc nlpaux)
            apply (erule nMT_domMT, simp_all)
            by (metis SR3nMT)
        qed
     qed
qed       

lemma mr_iD[rule_format]: "wellformed_policy1_strong p  \<longrightarrow> matching_rule x p = matching_rule x (insertDeny p)"
by (induct p, simp_all)

lemma WP1iD[rule_format,simp]: "wellformed_policy1_strong p \<longrightarrow> wellformed_policy1_strong (insertDeny p)"
by (induct p, simp_all)

lemma C_eq_id: "wellformed_policy1_strong p \<Longrightarrow> C(list2policy (insertDeny p)) = C (list2policy p)"
apply (rule ext)
apply (rule C_eq_if_mr_eq)
apply simp_all
apply (erule mr_iD)
done

lemma C_eq_RS3: "notMTpolicy p \<Longrightarrow> C(list2policy (removeShadowRules3 p)) = C (list2policy p)"
apply (rule ext)
by (erule C_eq_RS3_aux[symmetric])

lemma NMPcharn[rule_format]: "a \<in> set p\<longrightarrow> dom (C a) \<noteq> {} \<longrightarrow> notMTpolicy p"
by (induct p, simp_all)

lemma NMPrd[rule_format]: "notMTpolicy p \<longrightarrow> notMTpolicy (remdups p)"
apply (induct p, simp_all)
by (auto simp: NMPcharn)

lemma NMPRS3[rule_format]: "notMTpolicy p \<longrightarrow> notMTpolicy (removeShadowRules3 p)"
by (induct p, simp_all)

lemma DAiniD: "DenyAll \<in> set (insertDeny p)"
by (induct p, simp_all, case_tac a, simp_all)

lemma NMPDA[rule_format]: "DenyAll \<in> set p \<longrightarrow> notMTpolicy p"
by (induct p, simp_all add: DAnMT)

lemma NMPiD[rule_format]: "notMTpolicy (insertDeny p)"
apply (insert DAiniD [of p])
apply (erule NMPDA)
done

lemma p2lNmt: "policy2list p \<noteq> []"
by (rule policy2list.induct, simp_all)

lemma list2policy2list[rule_format]: "C (list2policy(policy2list p)) = (C p)"
apply (rule ext)
apply (induct_tac p, simp_all)
apply (case_tac "x \<in> dom (C (Combinators2))") 
apply (metis Cdom2 CeqEnd domIff p2lNmt)
apply (metis CeqStart domIff p2lNmt nlpaux)
done

lemma AIL2[rule_format,simp]: "all_in_list p l \<longrightarrow> all_in_list (removeShadowRules2 p) l"
by (induct_tac p, simp_all, case_tac a, simp_all)

lemmas C_eq_Lemmas = noneMTRS2 noneMTrd dRD SC2 SCrd SCRS3 SCRiD SC1 aux0 wp1n_RS2 WP1rd WP2RS2 wp1n_RS3 wp1ID NMPiD wp1alternative_RS1 p2lNmt list2policy2list wellformed_policy3_charn waux2 wp1_eq

lemmas C_eq_subst_Lemmas = C_eq_sorted C_eq_RS2 C_eq_rd C_eq_RS3 C_eq_id 

lemma C_eq_All_untilSorted: 
"\<lbrakk>DenyAll \<in> set (policy2list p);  all_in_list (policy2list p) l; allNetsDistinct (policy2list p)\<rbrakk> \<Longrightarrow> 
  C(list2policy (sort (removeShadowRules2 (remdups (removeShadowRules3 (insertDeny (removeShadowRules1  (policy2list p)))))) l)) = C p"
apply (subst C_eq_sorted)
apply (simp_all add: C_eq_Lemmas)
apply (subst C_eq_RS2)
apply (simp_all add: C_eq_Lemmas) 
apply (subst C_eq_rd)
apply (simp_all add: C_eq_Lemmas)
apply (subst C_eq_RS3)
apply (simp_all add: C_eq_Lemmas)
apply (subst C_eq_id)
apply (simp_all add: C_eq_Lemmas)
done


(* or, even shorter *)

lemma C_eq_All_untilSorted_withSimps: 
"\<lbrakk>DenyAll \<in> set (policy2list p);  all_in_list (policy2list p) l; allNetsDistinct (policy2list p)\<rbrakk> \<Longrightarrow> 
  C(list2policy (sort (removeShadowRules2 (remdups (removeShadowRules3 (insertDeny (removeShadowRules1  (policy2list p)))))) l)) = C p"
by (simp_all add: C_eq_Lemmas C_eq_subst_Lemmas)

lemma InDomConc[rule_format]: "p \<noteq> [] \<longrightarrow> x \<in> dom (C (list2policy (p))) \<longrightarrow> x \<in>  dom (C (list2policy (a#p)))"
apply (induct p)
apply simp_all
apply (case_tac "p = []")
apply (simp_all add: dom_def C.simps)
done

lemma not_in_member[rule_format]: "member a b \<longrightarrow> x \<notin> dom (C b) \<longrightarrow> x \<notin> dom (C a)"
apply (induct b)
apply (simp_all add: dom_def C.simps)
done

lemma subnetAux: "D \<inter> A \<noteq> {} \<Longrightarrow> A \<subseteq> B \<Longrightarrow>  D \<inter> B \<noteq> {}"
apply auto
done

lemma soadisj: "\<lbrakk>x \<in> subnetsOfAdr a; y \<in> subnetsOfAdr a\<rbrakk> \<Longrightarrow> \<not> netsDistinct x y"
by (simp add: subnetsOfAdr_def netsDistinct_def, auto simp: PLemmas)

lemma not_member: "\<not> member a (x\<oplus>y) \<Longrightarrow> \<not> member a x"
apply auto
done

lemma src_in_sdnets[rule_format]: "\<not> member DenyAll x \<longrightarrow> p \<in> dom (C x) \<longrightarrow> subnetsOfAdr (src p) \<inter> (fst_set (sdnets x)) \<noteq> {}"
apply (induct rule: Combinators.induct)
apply simp
apply (simp add: fst_set_def subnetsOfAdr_def PLemmas)
apply (simp add: fst_set_def subnetsOfAdr_def PLemmas)
apply (rule impI)+
apply (simp add: fst_set_def)
apply (case_tac "p \<in> dom (C Combinators2)")
apply simp_all
apply (rule subnetAux)
apply assumption
apply (auto simp: PLemmas)
done

lemma dest_in_sdnets[rule_format]: "\<not> member DenyAll x \<longrightarrow> p \<in> dom (C x) \<longrightarrow> subnetsOfAdr (dest p) \<inter> (snd_set (sdnets x)) \<noteq> {}"
apply (induct rule: Combinators.induct)
apply simp
apply (simp add: snd_set_def subnetsOfAdr_def PLemmas)
apply (simp add: snd_set_def subnetsOfAdr_def PLemmas)
apply (rule impI)+
apply (simp add: snd_set_def)
apply (case_tac "p \<in> dom (C Combinators2)")
apply simp_all
apply (rule subnetAux)
apply assumption
apply (auto simp: PLemmas)
done

lemma soadisj2: "(\<forall> a x y. x \<in> subnetsOfAdr a \<and> y \<in> subnetsOfAdr a \<longrightarrow> \<not> netsDistinct x y)"
by (simp add: subnetsOfAdr_def netsDistinct_def, auto simp: PLemmas)

lemma ndFalse1: "\<lbrakk>(\<forall>a b c d. (a,b)\<in>A  \<and> (c,d)\<in>B \<longrightarrow> netsDistinct a c);
        \<exists>(a, b)\<in>A. a \<in> subnetsOfAdr D; 
        \<exists>(a, b)\<in>B. a \<in> subnetsOfAdr D\<rbrakk>
    \<Longrightarrow> False"
apply (auto simp: soadisj)
apply (insert soadisj2)
apply (rotate_tac -1, drule_tac x = D in spec)
apply (rotate_tac -1, drule_tac x = a in spec)
apply (rotate_tac -1, drule_tac x = aa in spec)
by auto

lemma ndFalse2: "\<lbrakk>(\<forall>a b c d. (a,b)\<in>A  \<and> (c,d)\<in>B \<longrightarrow> netsDistinct b d);
        \<exists>(a, b)\<in>A. b \<in> subnetsOfAdr D; 
        \<exists>(a, b)\<in>B. b \<in> subnetsOfAdr D\<rbrakk>
    \<Longrightarrow> False"
apply (auto simp: soadisj)
apply (insert soadisj2)
apply (rotate_tac -1, drule_tac x = D in spec)
apply (rotate_tac -1, drule_tac x = b in spec)
apply (rotate_tac -1, drule_tac x = ba in spec)
apply simp
apply auto
done

lemma tndFalse: "\<lbrakk>(\<forall>a b c d. (a,b)\<in>A  \<and> (c,d)\<in>B \<longrightarrow> twoNetsDistinct a b c d);
        \<exists>(a, b)\<in>A. a \<in> subnetsOfAdr (D::('a::adr)) \<and> b \<in> subnetsOfAdr (F::'a); 
        \<exists>(a, b)\<in>B. a \<in> subnetsOfAdr D\<and> b\<in> subnetsOfAdr F\<rbrakk>
    \<Longrightarrow> False"
apply (simp add: twoNetsDistinct_def)
apply (auto simp: ndFalse1 ndFalse2)
apply (metis soadisj)
done

lemma sdnets_in_subnets[rule_format]: "p\<in> dom (C x) \<longrightarrow> \<not> member DenyAll x \<longrightarrow> (\<exists> (a,b)\<in>sdnets x. a \<in> subnetsOfAdr (src p) \<and> b \<in> subnetsOfAdr (dest p))"
apply (rule Combinators.induct)
apply simp_all
apply (simp add: PLemmas subnetsOfAdr_def)
apply (simp add: PLemmas subnetsOfAdr_def)
apply (rule impI)+
apply simp
apply (case_tac "p \<in> dom (C (Combinators2))")
apply simp_all
apply (auto simp: PLemmas subnetsOfAdr_def)
done

lemma disjSD_no_p_in_both[rule_format]:   
 "\<lbrakk>disjSD_2 x y; \<not> member DenyAll x; \<not> member DenyAll y;  
   p \<in> dom (C x); p \<in> dom (C y)\<rbrakk> \<Longrightarrow> False"
apply (rule_tac A = "sdnets x" and B = "sdnets y" and D = "src p" and F = "dest p" in tndFalse)
by (auto simp: dest_in_sdnets src_in_sdnets sdnets_in_subnets disjSD_2_def)

lemma list2policy_eq: "zs \<noteq> [] \<Longrightarrow> C (list2policy (x \<oplus> y # z)) p = C (x \<oplus> list2policy (y # z)) p"
apply (metis C.simps(4) CConcStartaux C_eq_None C_eq_RS3 C_eq_if_mr_eq C_eq_rd Cdom2 ConcAssoc domIff in_set_conv_decomp l2p_aux2 list.simps(1) list2policy.simps(2) list2policyconc map_add_None mem_def mrMTNone mrconcNone mreq_end3 mreq_endNone nlpaux not_Cons_self remdups.simps(2) removeShadowRules3.simps(2) self_append_conv2)
done

lemma sepnMT[rule_format]: "p \<noteq> [] \<longrightarrow> (separate p) \<noteq> []"
apply (rule separate.induct) back back back
by simp_all

lemma sepDA[rule_format]: "DenyAll \<notin> set p \<longrightarrow> DenyAll \<notin> set (separate p)"
apply (rule separate.induct) back
apply simp_all
done

lemma dom_sep[rule_format]: "x \<in> dom (C (list2policy p)) \<longrightarrow> x \<in> dom (C (list2policy(separate p)))"
apply (rule separate.induct) back
apply simp_all
apply (rule conjI)
apply (rule impI)+
apply simp
apply (thin_tac "False \<Longrightarrow> ?S")
apply (drule mp)
apply (case_tac "x \<in> dom (C (DenyAllFromTo v va))")
apply (metis CConcStartA domIff eq_Nil_appendI in_set_conv_decomp l2p_aux2 list2policyconc mem_def not_Cons_self notindom)
apply (subgoal_tac "x \<in> dom (C (list2policy (y #z)))")
apply (metis CConcStartA Cdom2 InDomConc domIff l2p_aux2 list2policyconc nlpaux)
apply (subgoal_tac "x \<in> dom (C (list2policy ((DenyAllFromTo v va)#y#z)))")
apply (simp add: dom_def C.simps)
apply simp
apply simp
apply (rule impI)+
apply simp
apply (thin_tac "False \<Longrightarrow> ?S")
apply (case_tac "x \<in> dom (C (DenyAllFromTo v va))")
apply simp_all
apply (subgoal_tac "x \<in> dom (C (list2policy (y #z)))")
apply (metis InDomConc sepnMT list.simps(2))
apply (subgoal_tac "x \<in> dom (C (list2policy ((DenyAllFromTo v va)#y#z)))")
apply (simp add: dom_def C.simps)
apply simp
apply (rule impI | rule conjI)+
apply simp
apply (case_tac "x \<in> dom (C (AllowPortFromTo v va vb))")
apply (metis CConcStartA domIff eq_Nil_appendI in_set_conv_decomp l2p_aux2 list2policyconc mem_def not_Cons_self notindom)
apply (subgoal_tac "x \<in> dom (C (list2policy (y #z)))")
apply simp
apply (metis CConcStartA Cdom2 InDomConc domIff l2p_aux2 list2policyconc nlpaux)
apply (simp add: dom_def C.simps)
apply (rule impI)+
apply simp
apply (case_tac "x \<in> dom (C (AllowPortFromTo v va vb))")
apply (metis CConcStartA)
apply (metis CConcStartA InDomConc domIff list.simps(1) list2policy.simps(2) nlpaux sepnMT)
apply (rule conjI | rule impI)+
apply simp
apply (thin_tac "False \<Longrightarrow> ?S")
apply (drule mp)
apply (case_tac "x \<in> dom (C ((v \<oplus> va)))")
apply (metis C.simps(4) CConcStartA ConcAssoc domIff eq_Nil_appendI in_set_conv_decomp list2policy2list list2policyconc mem_def notindom p2lNmt)
defer 1
apply simp_all
apply (rule impI)+
apply simp
apply (thin_tac "False \<Longrightarrow> ?S")
apply (case_tac "x \<in> dom (C ((v \<oplus> va)))")
apply (metis CConcStartA)
apply (drule mp)
apply (simp add: C.simps dom_def)
apply (metis InDomConc list.simps(1) mem_def sepnMT)
apply (subgoal_tac "x \<in> dom (C (list2policy (y#z)))")
apply (case_tac "x \<in> dom (C y)")
apply simp_all
apply (metis CConcStartA Cdom2 ConcAssoc domIff mem_def)
apply (metis InDomConc domIff l2p_aux2 list2policyconc nlpaux)
apply (case_tac "x \<in> dom (C y)")
apply simp_all
apply (metis InDomConc domIff l2p_aux2 list2policyconc nlpaux)
done

lemma domdConcStart[rule_format]: "   x \<in> dom (C (list2policy (a#b))) \<longrightarrow>
        x \<notin> dom (C (list2policy b))
       \<longrightarrow> x \<in> dom (C (a))"
apply (induct b, simp_all)
apply (auto simp: PLemmas)
done

lemma sep_dom2_aux: "\<lbrakk>x \<in> dom (C (list2policy (a \<oplus> y # z)))\<rbrakk>
       \<Longrightarrow> x \<in> dom (C (a \<oplus> list2policy (y # z)))"
by (metis CConcStartA InDomConc domIff domdConcStart l2p_aux2 list.simps(1) list2policy.simps(2) nlpaux)

lemma sep_dom2_aux2: 
"\<lbrakk>(x \<in> dom (C (list2policy (separate (y # z)))) \<longrightarrow> x \<in> dom (C (list2policy (y # z))));
   x \<in> dom (C (list2policy (a # separate (y # z))))\<rbrakk>
\<Longrightarrow> x \<in> dom (C (list2policy (a \<oplus> y # z)))"
by (metis CConcStartA Cdom2 InDomConc domIff  l2p_aux2 list2policyconc mem_def nlpaux)

lemma sep_dom2[rule_format]: 
 "x \<in> dom (C (list2policy (separate p))) \<longrightarrow> x \<in> dom (C (list2policy( p)))"
apply (rule separate.induct)
by (simp_all add: sep_dom2_aux sep_dom2_aux2)

lemma sepDom: "dom (C (list2policy p)) = dom (C (list2policy (separate p)))"
apply (rule equalityI)
by (rule subsetI, (erule dom_sep|erule sep_dom2))+

lemma C_eq_s_ext[rule_format]: "p \<noteq> [] \<longrightarrow> C (list2policy (separate p)) a  = C (list2policy p) a "
 proof (induct rule: separate.induct) 
  case goal1 thus ?case 
    apply simp
     apply (cases "x = []")
     apply (metis l2p_aux2 separate.simps(5))
     apply simp
     apply (cases "a \<in> dom (C (list2policy x))")
     apply (subgoal_tac "a \<in> dom (C (list2policy (separate x)))")
     apply (metis Cdom2 list2policyconc mem_def sepDom sepnMT)
     apply (metis sepDom)
     apply (subgoal_tac "a \<notin> dom (C (list2policy (separate x)))")
     apply (subst list2policyconc) 
     apply (simp add: sepnMT)
     apply (subst list2policyconc) 
     apply (simp add: sepnMT)
     apply (metis nlpaux sepDom)
     apply (metis sepDom)
     done
  next
  case goal2 thus ?case 
     apply simp 
     apply (cases "z = []")
     apply simp_all
     apply (rule conjI|rule impI|simp)+
     apply (subst list2policyconc)
      apply (metis not_Cons_self sepnMT)
     apply (metis C.simps(4) CConcStartaux Cdom2 domIff)
     apply (rule conjI|rule impI|simp)+
     apply (erule list2policy_eq)
     apply (rule impI, simp)
     apply (subst list2policyconc) 
     apply (metis list.simps(1) sepnMT) 
     apply (metis C.simps(4) CConcStartaux Cdom2 domIff list2policy.simps(2) sepDom)
     done
  next
  case goal3 thus ?case
  apply simp 
     apply (cases "z = []")
     apply simp_all
     apply (rule conjI|rule impI|simp)+
     apply (subst list2policyconc)
      apply (metis not_Cons_self sepnMT)
     apply (metis C.simps(4) CConcStartaux Cdom2 domIff)
     apply (rule conjI|rule impI|simp)+
     apply (erule list2policy_eq)
     apply (rule impI, simp)
     apply (subst list2policyconc) 
     apply (metis list.simps(1) sepnMT) 
     apply (metis C.simps(4) CConcStartaux Cdom2 domIff list2policy.simps(2) sepDom)
     done
  next
  case goal4 thus ?case
  apply simp 
     apply (cases "z = []")
     apply simp_all
     apply (rule conjI|rule impI|simp)+
     apply (subst list2policyconc)
      apply (metis not_Cons_self sepnMT)
     apply (metis C.simps(4) CConcStartaux Cdom2 domIff)
     apply (rule conjI|rule impI|simp)+
     apply (erule list2policy_eq)
     apply (rule impI, simp)
     apply (subst list2policyconc) 
     apply (metis list.simps(1) sepnMT) 
     apply (metis C.simps(4) CConcStartaux Cdom2 domIff list2policy.simps(2) sepDom)
     done
   next
  case goal5 thus ?case by simp next
  case goal6 thus ?case by simp next
  case goal7 thus ?case by simp next
  case goal8 thus ?case by simp next
 qed

lemma C_eq_s: "p \<noteq> [] \<Longrightarrow> C (list2policy (separate p)) = C (list2policy p)  "
apply (rule ext)
apply (rule C_eq_s_ext)
apply simp
done

lemma setnMT: "set a = set b \<Longrightarrow> a \<noteq> [] \<Longrightarrow> b \<noteq> []"
by auto

lemma sortnMT: "p \<noteq> [] \<Longrightarrow> sort p l \<noteq> []"
by (metis set_sort setnMT)

lemmas C_eq_Lemmas_sep = C_eq_Lemmas sortnMT RS2_NMT notMTpolicyimpnotMT NMPrd NMPRS3 NMPiD


lemma C_eq_until_separated: "\<lbrakk>DenyAll \<in> set (policy2list p); all_in_list (policy2list p) l; allNetsDistinct (policy2list p)\<rbrakk>
    \<Longrightarrow> C (list2policy
           (separate (sort (removeShadowRules2 (remdups (removeShadowRules3 (insertDeny (removeShadowRules1 (policy2list p)))))) l))) =
       C p"
apply (subst  C_eq_s)
apply (simp_all add: C_eq_Lemmas_sep)
apply (rule C_eq_All_untilSorted)
apply simp_all
done

lemma idNMT[rule_format]: "p \<noteq> [] \<longrightarrow> insertDenies p \<noteq> []"
apply (induct p, simp_all)
apply (case_tac a, simp_all)
done

lemma domID[rule_format]: "p \<noteq> [] \<and> x \<in> dom(C(list2policy p)) \<longrightarrow> x \<in> dom (C(list2policy(insertDenies p)))"
proof(induct p)
  case Nil then show ?case by simp
next 
  case (Cons a p) then show ?case
    proof(cases "p=[]")
       case goal1 then show ?case
          apply(simp) apply(rule impI)
          apply (cases a, simp_all)
          apply (simp add: C.simps dom_def)+
          apply (metis domIff mem_def Cdom2 ConcAssoc)
          done
    next
       case goal2 then show ?case
         proof(cases "x \<in> dom(C(list2policy p))")
            case goal1 then show ?case
               apply simp apply (rule impI)
               apply (cases a, simp_all)
               apply (metis InDomConc goal1(2) idNMT)
               apply (rule InDomConc, simp_all add: idNMT)+
               done
         next 
            case goal2 then show ?case  
               apply simp apply (rule impI)
               proof(cases "x \<in> dom (C (list2policy (insertDenies p)))")
                  case goal1 then show ?case
                     proof(induct a)
                        case DenyAll then show ?case by simp
                     next               
                        case (DenyAllFromTo src dest) then show ?case
                           apply simp by( rule InDomConc, simp add: idNMT)
                     next  
                       case (AllowPortFromTo src dest port) then show ?case
                           apply simp by(rule InDomConc, simp  add: idNMT)
                     next 
                       case (Conc _ _) then show ?case
                           apply simp by(rule InDomConc, simp add: idNMT)
                     qed
               next
                  case goal2 then show ?case
                     proof (induct a)
                       case DenyAll then show ?case by simp
                     next
                       case (DenyAllFromTo src dest) then show ?case
                           by(simp,metis domIff CConcStartA list2policyconc nlpaux Cdom2)
                     next
                       case (AllowPortFromTo src dest port) then show ?case
                           by(simp,metis domIff CConcStartA list2policyconc nlpaux Cdom2)
                     next
                       case (Conc _ _) then show ?case
                           by(simp,metis domIff CConcStartA list2policyconc nlpaux Cdom2)
                     qed
               qed
         qed
    qed
 qed
   

lemma DA_is_deny: " x \<in> dom (C (DenyAllFromTo a b \<oplus> DenyAllFromTo b a \<oplus> DenyAllFromTo a b))
    \<Longrightarrow> C (DenyAllFromTo a b \<oplus> DenyAllFromTo b a \<oplus> DenyAllFromTo a b) x = Some (deny x)"
apply (case_tac "x \<in> dom (C (DenyAllFromTo a b))")
apply (simp_all add: PLemmas)
apply (simp_all split: if_splits)
done

lemma iDdomAux[rule_format]: "p \<noteq> [] \<longrightarrow> x \<notin> dom (C (list2policy p)) \<longrightarrow> x \<in> dom (C (list2policy (insertDenies p))) \<longrightarrow> 
                                  C (list2policy (insertDenies p)) x = Some (deny x)"
 proof (induct p)
  case Nil thus ?case by simp
  next
  case (Cons y ys) thus ?case
    proof (cases y) 
      case DenyAll then show ?thesis by simp next
      case (DenyAllFromTo a b) then show ?thesis using prems
        apply simp
        apply (rule impI)+
        proof (cases "ys = []")
          case goal1 then show ?case by (simp add: DA_is_deny) next
          case goal2 then show ?case
            apply simp
            apply (drule mp)
            apply (metis DenyAllFromTo InDomConc goal2(3) goal2(5))
            apply (cases "x \<in> dom (C (list2policy (insertDenies ys)))")
            apply simp_all
            apply (metis Cdom2 DenyAllFromTo goal2(5) idNMT list2policyconc)
            apply (subgoal_tac "C (list2policy (DenyAllFromTo a b \<oplus> DenyAllFromTo b a \<oplus> DenyAllFromTo a b#insertDenies ys)) x = 
                                C ((DenyAllFromTo a b \<oplus> DenyAllFromTo b a \<oplus> DenyAllFromTo a b)) x ")
            apply simp
            apply (rule DA_is_deny)
            apply (metis DenyAllFromTo domdConcStart goal2(4))
            apply (metis DenyAllFromTo l2p_aux2 list2policyconc nlpaux)
            done 
        qed
      next
      case (AllowPortFromTo a b c) then show ?thesis using prems
        proof (cases "ys = []")
          case goal1 then show ?case 
            apply simp
            apply (rule impI)+
            apply (subgoal_tac "x \<in> dom (C (DenyAllFromTo a b \<oplus> DenyAllFromTo b a))")
            apply (simp_all add: PLemmas)
            apply (simp split: if_splits) apply auto 
            done next
          case goal2 then show ?case
            apply simp
            apply (rule impI)+
            apply (drule mp)
            apply (metis AllowPortFromTo InDomConc goal2(4))
            apply (cases "x \<in> dom (C (list2policy (insertDenies ys)))")
            apply simp_all
            apply (metis AllowPortFromTo Cdom2 goal2(4) idNMT list2policyconc)
            apply (subgoal_tac "C (list2policy (DenyAllFromTo a b \<oplus> DenyAllFromTo b a \<oplus> AllowPortFromTo a b c#insertDenies ys)) x = 
                                C ((DenyAllFromTo a b \<oplus> DenyAllFromTo b a)) x ")
            apply simp
            defer 1 
            apply (metis AllowPortFromTo CConcStartA ConcAssoc goal2(4) idNMT list2policyconc nlpaux)
            apply (simp add: PLemmas, simp split: if_splits) apply auto 
          done
         qed
      next
      case (Conc a b) then show ?thesis
 proof (cases "ys = []")
          case goal1 then show ?case 
            apply simp
            apply (rule impI)+
            apply (subgoal_tac "x \<in> dom (C (DenyAllFromTo (first_srcNet a) (first_destNet a) \<oplus> DenyAllFromTo (first_destNet a) (first_srcNet a)))")
            apply (simp_all add: PLemmas)
            apply (simp split: if_splits) apply auto 
            done next
          case goal2 then show ?case
            apply simp
            apply (rule impI)+
            apply (cases "x \<in> dom (C (list2policy (insertDenies ys)))")
            apply (metis Cdom2 Conc Cons InDomConc goal2(2) idNMT list2policyconc)
            apply (subgoal_tac "C (list2policy (DenyAllFromTo (first_srcNet a) (first_destNet a) \<oplus> DenyAllFromTo (first_destNet a) (first_srcNet a) \<oplus> a\<oplus> b#insertDenies ys)) x = 
 C ((DenyAllFromTo (first_srcNet a) (first_destNet a)  \<oplus> DenyAllFromTo (first_destNet a)  (first_srcNet a) \<oplus> a\<oplus> b)) x ")
            apply simp
            defer 1 
            apply (metis Conc l2p_aux2 list2policyconc nlpaux)
            apply (subgoal_tac "C ((DenyAllFromTo (first_srcNet a) (first_destNet a) \<oplus> DenyAllFromTo (first_destNet a) (first_srcNet a) \<oplus> a\<oplus> b)) x = 
 C ((DenyAllFromTo (first_srcNet a) (first_destNet a)  \<oplus> DenyAllFromTo (first_destNet a)  (first_srcNet a))) x ")
            apply simp
            defer 1 
            apply (metis CConcStartA Conc ConcAssoc nlpaux)
            apply (simp add: PLemmas, simp split: if_splits) apply auto 
          done
        qed
    qed
qed         

lemma iD_isD[rule_format]: "p \<noteq> [] \<longrightarrow> x \<notin> dom (C (list2policy p))
          \<longrightarrow> C (DenyAll \<oplus> list2policy (insertDenies p)) x = C DenyAll x"
apply (case_tac "x \<in> dom (C (list2policy (insertDenies p)))")
apply (rule impI)+
apply (metis C.simps(1) deny_all_def iDdomAux mem_def Cdom2)
apply (rule impI)+
apply (subst nlpaux)
apply simp_all
done

lemma OTNoTN[rule_format]: " OnlyTwoNets p \<longrightarrow> x \<noteq> DenyAll \<longrightarrow> x \<in> set p \<longrightarrow> onlyTwoNets x"
apply (induct p, simp_all)
apply (rule impI)+
apply (rule conjI)
apply (rule impI)
apply simp
apply (case_tac a, simp_all)
apply (rule impI)
apply (drule mp, simp_all)
apply (case_tac a, simp_all)
done

lemma first_isIn[rule_format]: "\<not> member DenyAll x \<longrightarrow> (first_srcNet x,first_destNet x) \<in> sdnets x"
by (induct x,case_tac x, simp_all)

lemma sdnets2: "\<lbrakk>\<exists>a b. sdnets x = {(a, b), (b, a)}; \<not> member DenyAll x\<rbrakk>  \<Longrightarrow> sdnets x = {(first_srcNet x, first_destNet x), (first_destNet x, first_srcNet x)}"
apply (subgoal_tac "(first_srcNet x, first_destNet x) \<in> sdnets x")
apply (drule exE)
prefer 2
apply assumption
apply (drule exE)
prefer 2
apply assumption
apply simp
apply (case_tac "first_srcNet x = a \<and> first_destNet x = b")
apply simp_all
apply (metis insert_commute)
apply (erule first_isIn)
done

lemma alternativelistconc1[rule_format]: "a \<in> set (net_list_aux [x]) \<longrightarrow> a \<in> set (net_list_aux [x,y])"
by (induct x,simp_all)

lemma alternativelistconc2[rule_format]: "a \<in> set (net_list_aux [x]) \<longrightarrow> a \<in> set (net_list_aux [y,x])"
by (induct y, simp_all)


lemma noDA[rule_format]:"noDenyAll xs \<longrightarrow> s \<in> set xs \<longrightarrow> \<not> member DenyAll s"
by (induct xs, simp_all)

lemma isInAlternativeList: "(aa \<in> set (net_list_aux [a]) \<or> aa \<in> set (net_list_aux p))
       \<Longrightarrow> aa \<in> set (net_list_aux (a # p))"
apply (case_tac a,simp_all)
done

lemma netlistaux: "x \<in> set (net_list_aux (a # p))\<Longrightarrow>x \<in> set (net_list_aux ([a])) \<or> x \<in> set (net_list_aux (p))"
apply (case_tac " x \<in> set (net_list_aux [a])")
apply simp_all
apply (case_tac a, simp_all)
done

lemma firstInNet[rule_format]: "\<not> member DenyAll a \<longrightarrow> first_destNet a \<in> set (net_list_aux (a # p))"
apply (rule Combinators.induct) 
apply simp_all
apply (metis netlistaux)
done

lemma firstInNeta[rule_format]: "\<not> member DenyAll a \<longrightarrow> first_srcNet a \<in> set (net_list_aux (a # p))"
apply (rule Combinators.induct) 
apply simp_all
apply (metis netlistaux)
done  

lemma disjComm: "disjSD_2 a b \<Longrightarrow> disjSD_2 b a"
apply (simp add: disjSD_2_def)
apply (rule allI)+
apply (rule impI)
apply (rule conjI)
apply (drule_tac x = c in spec)
apply (drule_tac x = d in spec)
apply (drule_tac x = aa in spec)
apply (drule_tac x = ba in spec)
apply (metis tNDComm)
apply (drule_tac x = c in spec)
apply (drule_tac x = d in spec)
apply (drule_tac x = aa in spec)
apply (drule_tac x = ba in spec)
apply simp
apply (simp add: twoNetsDistinct_def)
apply (metis nDComm)+
done

lemma disjSD2aux: "\<lbrakk>disjSD_2 a b; \<not> member DenyAll a; \<not> member DenyAll b\<rbrakk>  \<Longrightarrow> 
 disjSD_2 (DenyAllFromTo (first_srcNet a) (first_destNet a) \<oplus> DenyAllFromTo (first_destNet a) (first_srcNet a) \<oplus> a)  b"
apply (drule disjComm)
apply (rule disjComm)
apply (simp add: disjSD_2_def)
apply (rule allI)+
apply (rule impI)+
apply safe
apply (drule_tac x = "aa" in spec, drule_tac x = ba in spec, drule_tac x = "first_srcNet a" in spec, drule_tac x = "first_destNet a" in spec, auto intro: first_isIn)+
done

lemma inDomConc:"\<lbrakk> x\<notin>dom (C a); x\<notin>dom (C (list2policy p))\<rbrakk> \<Longrightarrow>  x \<notin> dom (C (list2policy(a#p)))"
by (metis domdConcStart)

lemma domsdisj[rule_format]: "p \<noteq> [] \<longrightarrow> (\<forall> x s. s \<in> set p \<and> x \<in> dom (C A) \<longrightarrow> x \<notin> dom (C s)) \<longrightarrow> y \<in> dom (C A) \<longrightarrow> y \<notin> dom (C (list2policy p))"
apply (induct p)
apply simp
apply (case_tac "p = []")
apply simp
apply (rule_tac x = y in spec)
apply (simp add: split_tupled_all)
apply (rule impI)+
apply (rule inDomConc)
apply (drule_tac x = y in spec, drule_tac x = a in spec)
apply auto
done

lemma isSepaux:  "\<lbrakk>p \<noteq> []; noDenyAll (a#p); separated (a # p); 
                   x \<in> dom (C (DenyAllFromTo (first_srcNet a) (first_destNet a) \<oplus> DenyAllFromTo (first_destNet a) (first_srcNet a) \<oplus> a))\<rbrakk> \<Longrightarrow>
             x \<notin> dom (C (list2policy p))"
apply (rule_tac A = "(DenyAllFromTo (first_srcNet a) (first_destNet a) \<oplus> DenyAllFromTo (first_destNet a) (first_srcNet a) \<oplus> a)" in domsdisj)
apply simp_all
apply (rule notI)
apply (rule_tac p = xa and x = " (DenyAllFromTo (first_srcNet a) (first_destNet a) \<oplus> DenyAllFromTo (first_destNet a) (first_srcNet a) \<oplus> a)" and y = s in disjSD_no_p_in_both)
apply simp_all
apply (simp add: disjSD_2_def)
apply (rule allI)+
apply (metis first_isIn tNDComm twoNetsDistinct_def)
apply (metis noDA)
done

lemma noDA1eq[rule_format]: "noDenyAll p \<longrightarrow> noDenyAll1 p"
apply (induct p)
apply simp
apply (case_tac a, simp_all)
done

lemma noDA1C[rule_format]: "noDenyAll1 (a#p) \<longrightarrow> noDenyAll1 p"
apply (case_tac a, simp_all)
apply (rule impI, rule noDA1eq, simp)+
done

lemma disjSD_2IDa: "\<lbrakk>disjSD_2 x y; \<not> member DenyAll x; \<not> member DenyAll y; a = (first_srcNet x); b = (first_destNet x)\<rbrakk>  \<Longrightarrow>
disjSD_2 ((DenyAllFromTo a b) \<oplus> (DenyAllFromTo b a) \<oplus> x) y"
apply simp
apply (rule disjSD2aux)
apply simp_all
done

lemma noDAID[rule_format]: "noDenyAll p \<longrightarrow> noDenyAll (insertDenies p)"
apply (induct p)
apply simp_all
apply (case_tac a, simp_all)
done

lemma isInIDo[rule_format]:  "noDenyAll p  \<longrightarrow> s \<in> set (insertDenies p) \<longrightarrow> 
 (\<exists>! a. s = (DenyAllFromTo (first_srcNet a) (first_destNet a)) \<oplus> (DenyAllFromTo (first_destNet a) (first_srcNet a)) \<oplus> a \<and> a \<in> set p)"
apply (induct p)
apply simp_all
apply (case_tac "a = DenyAll")
apply simp
apply (case_tac a, simp_all)
apply auto
done

lemma id_aux1[rule_format]: "DenyAllFromTo (first_srcNet s) (first_destNet s) \<oplus> DenyAllFromTo (first_destNet s) (first_srcNet s) \<oplus> s\<in> set (insertDenies p)
    \<longrightarrow> s \<in> set p"
apply (induct p)
apply simp_all
apply (case_tac a, simp_all)
done

lemma id_aux2: "\<lbrakk>noDenyAll p; (\<forall>s. s \<in> set p \<longrightarrow> disjSD_2 a s); \<not> member DenyAll a; 
        ((DenyAllFromTo (first_srcNet s) (first_destNet s)) \<oplus> (DenyAllFromTo (first_destNet s) (first_srcNet s)) \<oplus> s) \<in> set (insertDenies p)\<rbrakk>  \<Longrightarrow>
       disjSD_2 a ((DenyAllFromTo (first_srcNet s) (first_destNet s)) \<oplus> (DenyAllFromTo (first_destNet s) (first_srcNet s)) \<oplus> s)"
apply (rule disjComm)
apply (rule disjSD_2IDa)
apply simp_all
apply (metis disjComm id_aux1)
apply (metis id_aux1 noDA)
done

lemma id_aux4[rule_format]: "\<lbrakk>noDenyAll p; (\<forall>s. s \<in> set p \<longrightarrow> disjSD_2 a s);s \<in> set (insertDenies p); \<not> member DenyAll a\<rbrakk> \<Longrightarrow>  disjSD_2 a s"
apply (subgoal_tac "   \<exists>a. s =
          DenyAllFromTo (first_srcNet a) (first_destNet a) \<oplus>
          DenyAllFromTo (first_destNet a) (first_srcNet a) \<oplus> a \<and>
          a \<in> set p")
apply (drule_tac Q = "disjSD_2 a s" in exE)
apply simp_all
apply (rule id_aux2, simp_all)
apply (rule ex1_implies_ex)
apply (rule isInIDo)
apply simp_all
done

lemma sepNetsID[rule_format]: "noDenyAll1 p \<longrightarrow> separated p \<longrightarrow> separated (insertDenies p)"
apply (induct p)
apply simp_all
apply (rule impI)
apply (drule mp)
apply (erule noDA1C)
apply (rule impI)
apply (case_tac "a = DenyAll")
apply simp_all
apply (simp add: disjSD_2_def)
apply (case_tac a,simp_all)
apply auto
apply (rule disjSD_2IDa, simp_all, rule id_aux4, simp_all, metis noDA noDAID)+
done

lemma noneMTsep[rule_format]: "noneMT p \<longrightarrow> noneMT (separate p)"
apply (rule separate.induct) back
apply simp_all
apply (rule impI, simp)
apply (rule impI)
apply simp
apply (drule mp)
apply (simp add: C.simps)
apply simp
apply (rule impI)+
apply simp
apply (drule mp)
apply (simp add: C.simps)
apply simp
apply (rule impI)+
apply (simp)
apply (drule mp)
apply (simp add: C.simps)
apply (simp)
done

lemma aNDDA[rule_format]: "allNetsDistinct p \<longrightarrow> allNetsDistinct(DenyAll#p)"
apply (case_tac p)
apply simp
apply (rule impI)
apply (simp add: allNetsDistinct_def)
apply (rule impI)
apply (auto)
apply (simp add: allNetsDistinct_def)
done

lemma OTNConc[rule_format]: "OnlyTwoNets (y # z) \<longrightarrow>  OnlyTwoNets z"
apply (case_tac y, simp_all)
done

lemma first_bothNetsd: "\<not> member DenyAll x \<Longrightarrow> first_bothNet x = {first_srcNet x, first_destNet x}"
apply (induct x)
apply simp_all
done

lemma bNaux: "\<lbrakk>\<not> member DenyAll x; \<not> member DenyAll y;first_bothNet x = first_bothNet y\<rbrakk> \<Longrightarrow> {first_srcNet x, first_destNet x} = {first_srcNet y, first_destNet y}"
apply (simp add: first_bothNetsd)
done

lemma setPair: "{a,b} = {a,d} \<Longrightarrow> b = d"
apply (metis Un_empty_right Un_insert_right insert_absorb2 setPaireq)
done

lemma setPair1: "{a,b} = {d,a} \<Longrightarrow> b = d"
apply (metis Un_empty_right Un_insert_right insert_absorb2 setPaireq)
done

lemma setPair4: "{a,b} = {c,d} \<Longrightarrow> a \<noteq> c \<Longrightarrow> a = d"
by auto

lemma otnaux1: " {x, y, x, y} = {x,y}"
by auto


lemma OTNIDaux4: "{x,y,x} = {y,x}"
by auto

lemma setPair5: "{a,b} = {c,d} \<Longrightarrow> a \<noteq> c \<Longrightarrow> a = d"
by auto

lemma otnaux: "   
  \<lbrakk>first_bothNet x = first_bothNet y; \<not> member DenyAll x; \<not> member DenyAll y; 
   onlyTwoNets y; onlyTwoNets x\<rbrakk> \<Longrightarrow> 
   onlyTwoNets (x \<oplus> y)"
apply (simp add: onlyTwoNets_def)
apply (subgoal_tac " {first_srcNet x, first_destNet x} = {first_srcNet y, first_destNet y}")
apply (case_tac "(\<exists>a b. sdnets y = {(a, b)})")
apply simp_all
apply (case_tac "(\<exists>a b. sdnets x = {(a, b)})")
apply simp_all
apply (subgoal_tac "sdnets x = {(first_srcNet x, first_destNet x)}")
apply (subgoal_tac "sdnets y = {(first_srcNet y, first_destNet y)}")
apply simp
apply (case_tac "first_srcNet x = first_srcNet y")
apply simp_all
apply (rule disjI1)
apply (rule setPair)
apply simp
apply (subgoal_tac "first_srcNet x = first_destNet y")
apply simp
apply (subgoal_tac "first_destNet x = first_srcNet y")
apply simp
apply (rule_tac x ="first_srcNet y" in exI, rule_tac x = "first_destNet y" in exI,simp)
apply (rule setPair1)
apply simp
apply (rule setPair4)
apply simp_all
apply (metis first_isIn singletonE)
apply (metis first_isIn singletonE)
apply (subgoal_tac "sdnets x = {(first_srcNet x, first_destNet x),(first_destNet x, first_srcNet x)}")
apply (subgoal_tac "sdnets y = {(first_srcNet y, first_destNet y)}")
apply simp
apply (case_tac "first_srcNet x = first_srcNet y")
apply simp_all
apply (subgoal_tac "first_destNet x = first_destNet y")
apply simp
apply (rule setPair)
apply simp
apply (subgoal_tac "first_srcNet x = first_destNet y")
apply simp
apply (subgoal_tac "first_destNet x = first_srcNet y")
apply simp
apply (rule_tac x ="first_srcNet y" in exI, rule_tac x = "first_destNet y" in exI)
apply (metis DomainI Domain_empty Domain_insert OTNIDaux4 RangeI Range_empty Range_insert insertE insert_absorb insert_commute insert_iff mem_def singletonE)
apply (rule setPair1)
apply simp
apply (rule setPair5)
apply assumption
apply simp
apply (metis first_isIn singletonE)
apply (rule sdnets2)
apply simp_all
apply (case_tac "(\<exists>a b. sdnets x = {(a, b)})")
apply simp_all
apply (subgoal_tac "sdnets x = {(first_srcNet x, first_destNet x)}")
apply (subgoal_tac "sdnets y = {(first_srcNet y, first_destNet y),(first_destNet y, first_srcNet y)}")
apply simp
apply (case_tac "first_srcNet x = first_srcNet y")
apply simp_all
apply (subgoal_tac "first_destNet x = first_destNet y")
apply simp
apply (rule_tac x ="first_srcNet y" in exI, rule_tac x = "first_destNet y" in exI)
apply (metis DomainI Domain_empty Domain_insert OTNIDaux4 RangeI Range_empty Range_insert insertE insert_absorb insert_commute insert_iff mem_def singletonE)
apply (rule setPair)
apply simp
apply (subgoal_tac "first_srcNet x = first_destNet y")
apply simp
apply (subgoal_tac "first_destNet x = first_srcNet y")
apply simp
apply (rule setPair1)
apply simp
apply (rule setPair4)
apply assumption
apply simp
apply (rule sdnets2)
apply simp
apply simp
apply (metis singletonE first_isIn)
apply (subgoal_tac "sdnets x = {(first_srcNet x, first_destNet x),(first_destNet x, first_srcNet x)}")
apply (subgoal_tac "sdnets y = {(first_srcNet y, first_destNet y),(first_destNet y, first_srcNet y)}")
apply simp
apply (case_tac "first_srcNet x = first_srcNet y")
apply simp_all
apply (subgoal_tac "first_destNet x = first_destNet y")
apply simp
apply (rule_tac x ="first_srcNet y" in exI, rule_tac x = "first_destNet y" in exI)
apply (rule otnaux1)
apply (rule setPair)
apply simp
apply (subgoal_tac "first_srcNet x = first_destNet y")
apply simp
apply (subgoal_tac "first_destNet x = first_srcNet y")
apply simp
apply (rule_tac x ="first_srcNet y" in exI, rule_tac x = "first_destNet y" in exI)
apply (metis DomainI Domain_empty Domain_insert OTNIDaux4 RangeI Range_empty Range_insert first_isIn insertE insert_absorb insert_commute insert_iff mem_def singletonE)
apply (rule setPair1)
apply simp
apply (rule setPair4)
apply assumption
apply simp
apply (rule sdnets2,simp_all)+
apply (rule bNaux, simp_all)
done

lemma OTNSepaux: "\<lbrakk>onlyTwoNets (a \<oplus> y) \<and> OnlyTwoNets z \<longrightarrow>
     OnlyTwoNets (separate (a \<oplus> y # z));
     \<not> FWCompilation.member DenyAll a; 
      \<not> FWCompilation.member DenyAll y; noDenyAll z;
     onlyTwoNets a; OnlyTwoNets (y # z);first_bothNet (a) = first_bothNet y\<rbrakk>
    \<Longrightarrow> OnlyTwoNets (separate (a \<oplus> y # z))"
apply (drule mp)
apply simp_all
apply (rule conjI)
apply (rule otnaux)
apply simp_all
apply (rule_tac p = "(y # z)" in OTNoTN)
apply simp_all
apply (metis FWCompilation.member.simps(2))
apply (simp add: onlyTwoNets_def)
apply (rule_tac y = y in OTNConc,simp)
done

lemma OTNSEp[rule_format]: "noDenyAll1 p \<longrightarrow> OnlyTwoNets p  \<longrightarrow> OnlyTwoNets (separate p)"
apply (rule separate.induct) back 
by (simp_all add: OTNSepaux noDA1eq)

lemma nda[rule_format]: "singleCombinators (a#p) \<longrightarrow> noDenyAll p \<longrightarrow> noDenyAll1 (a # p)"
apply (induct p)
apply simp_all
apply (case_tac a, simp_all)
apply (case_tac a, simp_all)
done

lemma nDAcharn[rule_format]: "noDenyAll p = (\<forall> r \<in> set p. \<not> member DenyAll r)"
apply (induct p)
apply simp_all
done

lemma nDAeqSet: "set p = set s \<Longrightarrow> noDenyAll p = noDenyAll s"
apply (simp add: nDAcharn)
done

lemma nDASCaux[rule_format]: "DenyAll \<notin> set p \<longrightarrow> singleCombinators p \<longrightarrow> r \<in> set p \<longrightarrow>  \<not> member DenyAll r"
apply (case_tac r) 
apply simp_all
apply (rule impI)
apply (rule impI)
apply (rule impI)
apply (rule FalseE)
apply (rule SCnotConc)
apply simp
apply simp
done

lemma nDASC[rule_format]: "wellformed_policy1 p \<longrightarrow> singleCombinators p \<longrightarrow> noDenyAll1 p"
apply (induct p)
apply (rule impI)
apply simp_all
apply (rule impI)+
apply (drule mp)
apply (erule waux2)
apply (drule mp)
apply (erule singleCombinatorsConc)
apply (rule nda)
apply simp
apply (simp add: nDAcharn)
apply (rule ballI)
apply (rule nDASCaux)apply simp_all
apply (erule singleCombinatorsConc)
done

lemma noDAAll[rule_format]: "noDenyAll p = (\<not> memberP DenyAll p)"
apply (induct p)
apply simp_all
done

lemma memberPsep[symmetric]: "memberP x p = memberP x (separate p)"
apply (rule separate.induct) back 
apply simp_all
done

lemma noDAsep[rule_format]: "noDenyAll p \<Longrightarrow> noDenyAll (separate p)"
apply (simp add:noDAAll)
apply (subst memberPsep) 
apply simp
done

lemma noDA1sep[rule_format]: "noDenyAll1 p \<longrightarrow> noDenyAll1 (separate p)"
apply (rule separate.induct) back
apply simp_all
apply (rule impI)
apply (rule noDAsep)
apply simp
apply (rule impI)+
apply (rule noDAsep)
apply (case_tac y, simp_all)
apply (rule impI)+
apply (rule noDAsep)
apply (case_tac y, simp_all)
apply (rule impI)+
apply (rule noDAsep)
apply (case_tac y, simp_all)
done

lemma isInAlternativeLista: "(aa \<in> set (net_list_aux [a]))\<Longrightarrow> aa \<in> set (net_list_aux (a # p))"
apply (case_tac a,simp_all)
apply safe
done

lemma isInAlternativeListb: "(aa \<in> set (net_list_aux p))\<Longrightarrow> aa \<in> set (net_list_aux (a # p))"
apply (case_tac a,simp_all)
done

lemma ANDSepaux: "allNetsDistinct (x # y # z) \<Longrightarrow> allNetsDistinct (x \<oplus> y # z)"
apply (simp add: allNetsDistinct_def)
apply (rule allI)+
apply (rule impI)
apply (drule_tac x = a in spec, drule_tac x = b in spec)
apply simp
apply (drule mp)
apply (rule conjI, simp_all)
apply (metis isInAlternativeList)+
done

lemma netlistalternativeSeparateaux: "net_list_aux [y] @ net_list_aux z = net_list_aux (y # z)"
apply (case_tac y, simp_all)
done

lemma netlistalternativeSeparate: "net_list_aux p = net_list_aux (separate p)"
apply (rule separate.induct) back
apply simp_all
apply (simp_all add: netlistalternativeSeparateaux)
done

lemma  ANDSepaux2: "\<lbrakk>allNetsDistinct (x # y # z); allNetsDistinct (separate (y # z))\<rbrakk>
       \<Longrightarrow> allNetsDistinct (x # separate (y # z))"
apply (simp add: allNetsDistinct_def)
apply (rule allI)+
apply (rule impI)
apply (drule_tac x = a in spec)
apply (rotate_tac -1)
apply (drule_tac x = b in spec)
apply (simp)
apply (drule mp)
apply (rule conjI)
apply (case_tac "a \<in> set (net_list_aux [x])")
apply simp_all
apply (rule isInAlternativeLista)
apply simp
apply (rule isInAlternativeListb)
apply (subgoal_tac "a \<in> set (net_list_aux (separate (y#z)))")
apply (metis netlistalternativeSeparate)
apply (metis netlistaux netlistalternativeSeparate)
apply (case_tac "b \<in> set (net_list_aux [x])")
apply (rule isInAlternativeLista)
apply simp
apply (rule isInAlternativeListb)
apply (subgoal_tac "b \<in> set (net_list_aux (separate (y#z)))")
apply (metis netlistalternativeSeparate)
apply (metis netlistaux netlistalternativeSeparate)
done



lemma ANDSep[rule_format]: "allNetsDistinct p \<longrightarrow> allNetsDistinct(separate p)"
apply (rule separate.induct) back
apply simp_all
apply (metis ANDConc aNDDA separate.simps(1))
apply (metis ANDConc ANDSepaux ANDSepaux2)
apply (metis ANDConc ANDSepaux ANDSepaux2)
apply (metis ANDConc ANDSepaux ANDSepaux2)
done

lemma dom_id: "\<lbrakk>noDenyAll (a#p); separated (a#p); p \<noteq> []; x \<notin> dom (C (list2policy p)); x \<in> dom (C (a))\<rbrakk> 
       \<Longrightarrow> x \<notin> dom (C (list2policy (insertDenies p)))"
apply (rule_tac a = a in isSepaux)
apply simp_all
apply (rule idNMT)
apply simp
apply (rule noDAID)
apply simp
apply (rule conjI)
apply (rule allI)
apply (rule impI)
apply (rule id_aux4)
apply simp_all
apply (rule sepNetsID)
apply simp_all
apply (metis noDA1eq)
apply (simp add: C.simps)
done

lemma C_eq_iD_aux2[rule_format]: 
 "noDenyAll1 p \<longrightarrow>
separated p\<longrightarrow>
p \<noteq> []\<longrightarrow>
x \<in> dom (C (list2policy p))\<longrightarrow>
C(list2policy (insertDenies p)) x = C(list2policy p) x"
proof (induct p)
case Nil thus ?case by simp 
next
case (Cons y ys)  thus ?case using prems
 proof (cases y)
 case DenyAll thus ?thesis using prems apply simp
   apply (case_tac "ys = []")
   apply simp_all
   apply (case_tac "x \<in> dom (C (list2policy ys))") 
   apply simp_all
apply (metis Cdom2 Combinators.simps(1) DenyAll FWCompilation.member.simps(3) bar3 domID idNMT in_set_conv_decomp insert_absorb insert_code list2policyconc mem_def nMT_domMT noDA1C noDA1eq noDenyAll.simps(1) notMTpolicyimpnotMT notindom)
apply (metis DenyAll iD_isD idNMT list2policyconc nlpaux)
   done
 next
 case (DenyAllFromTo a b) thus ?thesis using prems apply simp
   apply (rule impI|rule allI|rule conjI|simp)+
   apply (case_tac "ys = []")
   apply simp_all
   apply (metis Cdom2 ConcAssoc DenyAllFromTo)
   apply (case_tac "x \<in> dom (C (list2policy ys))")
   apply simp_all
   apply (drule mp)
   apply (metis noDA1eq)
   apply (case_tac "x \<in> dom (C (list2policy (insertDenies ys)))")
   apply (metis Cdom2 DenyAllFromTo idNMT list2policyconc)
   apply (metis domID)
   apply (case_tac "x \<in> dom (C (list2policy (insertDenies ys)))")
   apply (subgoal_tac "C (list2policy (DenyAllFromTo a b \<oplus> DenyAllFromTo b a \<oplus> DenyAllFromTo a b # insertDenies ys)) x = Some (deny x)")
   apply simp_all
   apply (subgoal_tac "C (list2policy (DenyAllFromTo a b # ys)) x = C ((DenyAllFromTo a b)) x")
   apply (simp add: PLemmas, simp split: if_splits)
   apply (metis list2policyconc nlpaux)
   apply (metis Combinators.simps(1) DenyAllFromTo FWCompilation.member.simps(3) dom_id domdConcStart mem_def noDenyAll.simps(1) separated.simps(1))
   apply (metis Cdom2 ConcAssoc DenyAllFromTo domdConcStart l2p_aux2 list2policyconc nlpaux)
   done
 next
 case (AllowPortFromTo a b c) thus ?thesis using prems apply simp
   apply (rule impI|rule allI|rule conjI|simp)+
   apply (case_tac "ys = []")
   apply simp_all
   apply (metis Cdom2 ConcAssoc AllowPortFromTo)
   apply (case_tac "x \<in> dom (C (list2policy ys))")
   apply simp_all
   apply (drule mp)
   apply (metis noDA1eq)
   apply (case_tac "x \<in> dom (C (list2policy (insertDenies ys)))")
   apply (metis Cdom2 AllowPortFromTo idNMT list2policyconc)
   apply (metis domID)
   apply (subgoal_tac "x \<in> dom (C (AllowPortFromTo a b c))")
   apply (case_tac "x \<notin> dom (C (list2policy (insertDenies ys)))")
   apply simp_all
   apply (metis AllowPortFromTo Cdom2 ConcAssoc l2p_aux2 list2policyconc nlpaux)
   apply (metis AllowPortFromTo Combinators.simps(3) FWCompilation.member.simps(4) dom_id mem_def noDenyAll.simps(1) separated.simps(1))
   apply (metis AllowPortFromTo domdConcStart)
   done
 next
 case (Conc a b) thus ?thesis using prems apply simp
   apply (rule impI|rule allI|rule conjI|simp)+
   apply (case_tac "ys = []")
   apply simp_all
   apply (metis Cdom2 ConcAssoc Conc)
   apply (case_tac "x \<in> dom (C (list2policy ys))")
   apply simp_all
   apply (drule mp)
   apply (metis noDA1eq)
   apply (case_tac "x \<in> dom (C (a \<oplus> b))")
   apply (case_tac "x \<notin> dom (C (list2policy (insertDenies ys)))")
   apply simp_all
   apply (subst list2policyconc)
   apply (rule idNMT, simp)
   apply (metis domID)
   apply (metis Cdom2 Conc idNMT list2policyconc)
   apply (metis CConcEnd2 CConcStartA Cdom2 Conc aux0_4 domID domIff idNMT in_set_conv_decomp l2p_aux2 list2policyconc mem_def nMT_domMT notMTpolicyimpnotMT not_Cons_self notindom)
   apply (case_tac "x \<in> dom (C (a \<oplus> b))")
   apply (case_tac "x \<notin> dom (C (list2policy (insertDenies ys)))")
   apply simp_all
   apply (subst list2policyconc)
   apply (rule idNMT, simp)
   apply (metis Cdom2 Conc ConcAssoc list2policyconc nlpaux)
   apply (metis Conc FWCompilation.member.simps(1) dom_id mem_def noDenyAll.simps(1) separated.simps(1))
   apply (metis Conc domdConcStart)
   done
 qed
qed

lemma C_eq_iD: "\<lbrakk>separated p; noDenyAll1 p; wellformed_policy1_strong p\<rbrakk>  \<Longrightarrow> 
 C (list2policy (insertDenies p)) = C (list2policy p)"
apply (rule ext)
apply (rule C_eq_iD_aux2)
apply simp_all 
apply (subgoal_tac "DenyAll \<in> set p")
apply (metis C_eq_RS1 DAAux append_is_Nil_conv domIff l2p_aux list.simps(1) mem_def nlpaux removeShadowRules1.simps(1) split_list_first)
apply (erule wp1_aux1aa)
done

lemma wp1_alternativesep[rule_format]: "wellformed_policy1_strong p \<longrightarrow> wellformed_policy1_strong (separate p)"
apply (rule impI)
apply (subst wp1n_tl) back
apply simp
apply simp
apply (rule sepDA)
apply (erule WP1n_DA_notinSet)
done

lemma noDAsort[rule_format]: "noDenyAll1 p \<longrightarrow> noDenyAll1 (sort p l)"
apply (case_tac "p")
apply simp
apply simp
apply (case_tac "a = DenyAll")
apply simp_all 
apply (rule impI)
apply (subst nDAeqSet)
defer 1
apply simp
defer 1
apply (rule set_sort)
apply (rule impI)
apply (case_tac "insort a (sort list l) l")
apply simp_all
apply (rule noDA1eq)
apply (subgoal_tac "noDenyAll (a#list)")
defer 1
apply (case_tac a, simp,simp)
apply simp
apply simp
apply (subst nDAeqSet)
defer 1
apply assumption
apply (metis sort.simps(2) set_sort)
done

lemma OTNSC[rule_format]: "singleCombinators p \<longrightarrow> OnlyTwoNets p"
apply (induct p)
apply simp_all
apply (rule impI)
apply (drule mp)
apply (erule singleCombinatorsConc)
apply (case_tac a, simp_all)
apply (simp add: onlyTwoNets_def)+
done

lemma fMTaux: "\<not> member DenyAll x \<Longrightarrow> first_bothNet x \<noteq> {}"
apply (metis bot_set_eq first_bothNetsd insert_not_empty)
done

lemma fl2[rule_format]: "firstList (separate p) = firstList p"
apply (rule separate.induct)
apply simp_all
done

lemma fl3[rule_format]: "NetsCollected p \<longrightarrow> (first_bothNet x \<noteq> firstList p \<longrightarrow>(\<forall>a\<in>set p. first_bothNet x \<noteq> first_bothNet a))\<longrightarrow>  NetsCollected (x#p)"
apply (induct p)
apply simp_all
done

lemma sortedConc[rule_format]: " sorted (a # p) l \<longrightarrow>  sorted p l"
apply (induct p)
apply simp_all
done

lemma smalleraux2: 
 "{a,b} \<in> set l \<Longrightarrow> {c,d} \<in> set l \<Longrightarrow> {a,b} \<noteq> {c,d} \<Longrightarrow> 
   smaller (DenyAllFromTo a b) (DenyAllFromTo c d) l \<Longrightarrow> 
  \<not> smaller (DenyAllFromTo c d) (DenyAllFromTo a b) l"
apply simp
apply (rule conjI)
apply (rule impI)
apply simp
apply (metis)
apply (metis eq_imp_le mem_def pos_noteq)
done

lemma smalleraux2a: 
 "{a,b} \<in> set l \<Longrightarrow> {c,d} \<in> set l \<Longrightarrow> {a,b} \<noteq> {c,d} \<Longrightarrow> 
   smaller (DenyAllFromTo a b) (AllowPortFromTo c d p) l \<Longrightarrow> 
  \<not> smaller (AllowPortFromTo c d p) (DenyAllFromTo a b) l"
apply simp
apply (metis eq_imp_le mem_def pos_noteq)
done


lemma smalleraux2b: 
 "{a,b} \<in> set l \<Longrightarrow> {c,d} \<in> set l \<Longrightarrow> {a,b} \<noteq> {c,d} \<Longrightarrow> y = DenyAllFromTo a b \<Longrightarrow>
   smaller (AllowPortFromTo  c d p) y l \<Longrightarrow> 
  \<not> smaller y (AllowPortFromTo  c d p) l"
apply simp
apply (metis eq_imp_le mem_def pos_noteq)
done

lemma smalleraux2c: 
 "{a,b} \<in> set l \<Longrightarrow> {c,d} \<in> set l \<Longrightarrow> {a,b} \<noteq> {c,d} \<Longrightarrow> y = AllowPortFromTo a b q \<Longrightarrow> 
   smaller (AllowPortFromTo  c d p) y l \<Longrightarrow> 
  \<not> smaller y (AllowPortFromTo  c d p) l"
apply simp
apply (metis eq_imp_le mem_def pos_noteq)
done

lemma smalleraux3: 
  assumes "x \<in> set l"
  assumes " y \<in> set l" 
  assumes "x \<noteq> y"
  assumes "x = bothNet a" 
  assumes "y = bothNet b"
  assumes "smaller a b l" 
  assumes "singleCombinators [a]" 
  assumes "singleCombinators [b]"  
  shows "\<not> smaller b a l"
proof (cases a)
 case DenyAll thus ?thesis using prems by (case_tac b,simp_all)  
 next
 case (DenyAllFromTo c d) thus ?thesis 
  proof (cases b)
   case DenyAll thus ?thesis using prems by simp
   next
   case (DenyAllFromTo e f) thus ?thesis using prems apply simp
    by (metis Combinators.simps(13) DenyAllFromTo assms(1) assms(2) assms(3) eq_imp_le le_anti_sym pos_noteq)
   next
   case (AllowPortFromTo e f g) thus ?thesis using prems apply simp
    by (metis assms(1) assms(2) assms(3) eq_imp_le pos_noteq)
   next
   case (Conc e f) thus ?thesis using prems by simp
  qed
 next
 case (AllowPortFromTo c d p) thus ?thesis
  proof (cases b)
   case DenyAll thus ?thesis using prems by simp
   next
   case (DenyAllFromTo e f) thus ?thesis using prems apply simp
    by (metis assms(1) assms(2) assms(3) eq_imp_le pos_noteq)
   next
   case (AllowPortFromTo e f g) thus ?thesis using prems apply simp
    by (metis assms(1) assms(2) assms(3) pos_noteq)
   next
   case (Conc e f) thus ?thesis using prems by simp
  qed
 next
 case (Conc c d) thus ?thesis using prems by simp
qed


lemma smalleraux3a: 
   "a \<noteq> DenyAll \<Longrightarrow> b \<noteq> DenyAll \<Longrightarrow> in_list b l \<Longrightarrow> in_list a l \<Longrightarrow>  bothNet a \<noteq> bothNet b \<Longrightarrow> 
   smaller a b l \<Longrightarrow> singleCombinators [a] \<Longrightarrow> singleCombinators [b] \<Longrightarrow>  
  \<not> smaller b a l"
apply (rule smalleraux3)
apply simp_all
apply (case_tac a, simp_all)
apply (case_tac b, simp_all)
done

lemma posaux[rule_format]: "position a l < position b l \<longrightarrow> a \<noteq> b"
apply (induct l)
apply simp_all
done

lemma posaux6[rule_format]: "a \<in> set l \<longrightarrow> b \<in> set l \<longrightarrow> a \<noteq> b \<longrightarrow>
   position a l \<noteq>  position b l"
apply (induct l)
apply simp_all
apply (rule conjI)
apply (rule impI)+
apply (rule conjI, rule impI,simp)
apply (erule position_positive)
apply (metis position_positive)
apply (metis position_positive)
done

lemma notSmallerTransaux[rule_format]: 
 "\<lbrakk>x \<noteq> DenyAll; r \<noteq> DenyAll; singleCombinators [x]; singleCombinators [y]; singleCombinators [r]; 
  \<not> smaller y x l; smaller x y l; smaller x r l; smaller y r l; 
  in_list x l; in_list y l; in_list r l\<rbrakk> \<Longrightarrow> 
     \<not> smaller r x l"
by (metis FWCompilationProof.order_trans)


lemma notSmallerTrans[rule_format]: 
 "x \<noteq> DenyAll \<longrightarrow> r \<noteq> DenyAll \<longrightarrow> singleCombinators (x#y#z) \<longrightarrow> 
  \<not> smaller y x l \<longrightarrow> sorted (x#y#z) l \<longrightarrow> r \<in> set z \<longrightarrow> 
  all_in_list (x#y#z) l \<longrightarrow> \<not> smaller r x l"
apply (rule impI)+
apply (rule notSmallerTransaux)
apply simp_all
apply (metis singleCombinatorsConc singleCombinatorsStart)
apply (metis SCSubset equalityE mem_def remdups.simps(2) set_remdups singleCombinatorsConc singleCombinatorsStart)
apply metis
apply (metis FWCompilation.sorted.simps(3) in_set_in_list singleCombinatorsConc singleCombinatorsStart sortedConcStart sorted_is_smaller)
apply (metis FWCompilationProof.sorted_Cons all_in_list.simps(2) singleCombinatorsConc)
apply metis
apply (metis in_set_in_list)
done

lemma  NCSaux1[rule_format]:
 "noDenyAll p \<longrightarrow> {x, y} \<in> set l \<longrightarrow>  all_in_list p l \<longrightarrow> singleCombinators p \<longrightarrow> 
  sorted (DenyAllFromTo x y # p) l \<longrightarrow> {x, y} \<noteq> firstList p \<longrightarrow> DenyAllFromTo u v \<in> set p \<longrightarrow> 
  {x, y} \<noteq> {u, v}"
proof (cases p)
 case Nil thus ?thesis by simp next
 case (Cons a p) thus ?thesis using prems apply simp
  apply (rule impI)+
  apply (rule conjI)
  apply (metis bothNet.simps(2) first_bothNet.simps(3))
  apply (rule impI) 
  apply (subgoal_tac "smaller (DenyAllFromTo x y) (DenyAllFromTo u v) l")
apply (subgoal_tac "\<not> smaller (DenyAllFromTo u v) (DenyAllFromTo x y) l")
apply (rule notI)
apply (case_tac "smaller (DenyAllFromTo u v) (DenyAllFromTo x y) l")
apply (simp del: smaller.simps)
apply simp
apply (case_tac "x = u")
apply simp
apply (case_tac "y = v")
apply simp
apply (subgoal_tac "u = v")
apply simp
apply simp
apply simp
apply (rule_tac y = a and z = p in notSmallerTrans)
apply (simp_all del: smaller.simps)
apply (rule smalleraux3a)
apply (simp_all del: smaller.simps)
apply (case_tac a, simp_all del: smaller.simps)
apply (case_tac a, simp_all del: smaller.simps)
apply (rule_tac y = a in order_trans)
apply simp_all
apply (subgoal_tac "in_list (DenyAllFromTo u v) l")
apply simp
apply (rule_tac p = p in in_set_in_list)
apply simp
apply (case_tac a, simp_all del: smaller.simps)
apply (metis all_in_list.simps(2) sorted_Cons mem_def)
done
qed

lemma posaux3[rule_format]: "a \<in> set l \<longrightarrow> b \<in> set l \<longrightarrow> a \<noteq> b \<longrightarrow> position a l \<noteq> position b l"
apply (induct l)
apply simp_all
apply (rule conjI)
apply (rule impI)+
apply (rule conjI)
apply (rule impI)
apply simp_all
apply (metis position_positive)+
done

lemma posaux4[rule_format]: "singleCombinators [a] \<longrightarrow> a\<noteq> DenyAll \<longrightarrow> b \<noteq> DenyAll \<longrightarrow> in_list a l \<longrightarrow>in_list b l \<longrightarrow>smaller a b  l\<longrightarrow> x = (bothNet a) \<longrightarrow> y = (bothNet b) \<longrightarrow> position x l <= position y l"
proof (cases a) 
 case DenyAll then show ?thesis by simp
 next
 case (DenyAllFromTo c d) thus ?thesis
  proof (cases b)
   case DenyAll thus ?thesis by simp next
   case (DenyAllFromTo e f) thus ?thesis using prems 
    apply simp
    by (metis bot_set_eq eq_imp_le) 
   next
   case (AllowPortFromTo e f p) thus ?thesis using prems by simp next
   case (Conc e f) thus ?thesis using prems by simp
 qed
 next
 case (AllowPortFromTo c d p) thus ?thesis
  proof (cases b)
   case DenyAll thus ?thesis by simp next
   case (DenyAllFromTo e f) thus ?thesis using prems by simp next
   case (AllowPortFromTo e f p) thus ?thesis using prems by simp next
   case (Conc e f) thus ?thesis using prems by simp
 qed
 next
 case (Conc c d) thus ?thesis by simp
qed

lemma  NCSaux2[rule_format]:"noDenyAll p \<longrightarrow> {a, b} \<in> set l \<longrightarrow>   all_in_list p l \<longrightarrow>singleCombinators p \<longrightarrow> sorted (DenyAllFromTo a b # p) l \<longrightarrow> {a, b} \<noteq> firstList p \<longrightarrow> AllowPortFromTo u v w \<in> set p \<longrightarrow> 
        {a, b} \<noteq> {u, v}"
apply (case_tac p)
apply simp_all
apply (rule impI)+
apply (rule conjI)
apply (rule impI)
apply (rotate_tac -1, drule sym)
apply simp
apply (rule impI)
apply (subgoal_tac "smaller (DenyAllFromTo a b) (AllowPortFromTo u v w) l")
apply (subgoal_tac "\<not> smaller (AllowPortFromTo u v w) (DenyAllFromTo a b) l")
defer 1
apply (rule_tac y = aa and z = list in notSmallerTrans)
apply (simp_all del: smaller.simps)
apply (rule smalleraux3a)
apply (simp_all del: smaller.simps)
apply (case_tac aa, simp_all del: smaller.simps)
apply (case_tac aa, simp_all del: smaller.simps)
apply (rule_tac y = aa in order_trans)
apply (simp_all del: smaller.simps)
apply (subgoal_tac "in_list (AllowPortFromTo u v w) l")
apply simp
apply (rule_tac p = list in in_set_in_list)
apply simp
apply simp
apply (metis all_in_list.simps(2) sorted_Cons mem_def)
apply (rule_tac l = l in posaux) 
apply (rule_tac y = "position (first_bothNet aa) l" in basic_trans_rules(22))
apply simp
apply (simp split: if_splits)
apply (case_tac aa, simp_all)
apply (case_tac "a = \<alpha>1 \<and> b = \<alpha>2")
apply simp_all
apply (case_tac "a = \<alpha>1")
apply simp_all
apply (rule basic_trans_rules(18))
apply simp
apply (rule posaux3)
apply simp
apply simp
apply simp
apply (rule basic_trans_rules(18))
apply simp
apply (rule posaux3)
apply simp
apply simp
apply simp
apply (rule basic_trans_rules(18))
apply simp
apply (rule posaux3)
apply simp
apply simp
apply simp
apply (rule basic_trans_rules(18))
apply (rule_tac a = "DenyAllFromTo a b" and b = aa in posaux4)
apply simp_all
apply (case_tac aa,simp_all)
apply (case_tac aa, simp_all)
apply (rule posaux3)
apply simp_all
apply (case_tac aa, simp_all)
apply (simp split: if_splits)
apply (rule_tac a = aa and b = "AllowPortFromTo u v w" in posaux4)
apply simp_all
apply (case_tac aa,simp_all)
apply (rule_tac p = list in sorted_is_smaller)
apply simp_all
apply (case_tac aa, simp_all)
apply (case_tac aa, simp_all)
apply (rule_tac a = aa and b = "AllowPortFromTo u v w" in posaux4)
apply simp_all
apply (case_tac aa,simp_all)
apply (subgoal_tac "in_list (AllowPortFromTo u v w) l")
apply simp
apply (rule_tac p = list in in_set_in_list)
apply simp
defer 1
apply simp_all
apply (metis all_in_list.simps(2) sorted_Cons mem_def)
apply (case_tac aa, simp_all)
done

lemma  NCSaux3[rule_format]:
 "noDenyAll p \<longrightarrow> {a, b} \<in> set l \<longrightarrow>  all_in_list p l \<longrightarrow>singleCombinators p \<longrightarrow> 
  sorted (AllowPortFromTo a b w # p) l \<longrightarrow> {a, b} \<noteq> firstList p \<longrightarrow> DenyAllFromTo u v \<in> set p \<longrightarrow> 
{a, b} \<noteq> {u, v}"
apply (case_tac p)
apply simp_all
apply (rule impI)+
apply (rule conjI)
apply (rule impI)
apply (rotate_tac -1, drule sym)
apply simp
apply (rule impI)
apply (subgoal_tac "smaller (AllowPortFromTo a b w) (DenyAllFromTo u v) l")
apply (subgoal_tac "\<not> smaller (DenyAllFromTo u v) (AllowPortFromTo a b w) l")
apply (simp split: if_splits)
apply (rule_tac y = aa and z = list in notSmallerTrans)
apply (simp_all del: smaller.simps)
apply (rule smalleraux3a)
apply (simp_all del: smaller.simps)
apply (case_tac aa, simp_all del: smaller.simps)
apply (case_tac aa, simp_all del: smaller.simps)
apply (rule_tac y = aa in order_trans)
apply (simp_all del: smaller.simps)
apply (subgoal_tac "in_list (DenyAllFromTo u v) l")
apply simp
apply (rule_tac p = list in in_set_in_list)
apply simp
apply simp
apply (rule_tac p = list in sorted_is_smaller)
apply (simp_all del: smaller.simps)
apply (subgoal_tac "in_list (DenyAllFromTo u v) l")
apply simp
apply (rule_tac p = list in in_set_in_list)
apply simp
apply simp
apply (erule singleCombinatorsConc)
done


lemma  NCSaux4[rule_format]:
 "noDenyAll p \<longrightarrow> {a, b} \<in> set l \<longrightarrow>  all_in_list p l \<longrightarrow> singleCombinators p \<longrightarrow> 
  sorted (AllowPortFromTo a b c # p) l \<longrightarrow> {a, b} \<noteq> firstList p \<longrightarrow> AllowPortFromTo u v w \<in> set p \<longrightarrow> 
 {a, b} \<noteq> {u, v}" 
apply (case_tac p)
apply simp_all
apply (rule impI)+
apply (rule conjI)
apply (rule impI)
apply (rotate_tac -1, drule sym)
apply simp
apply (rule impI)
apply (subgoal_tac "smaller (AllowPortFromTo a b c) (AllowPortFromTo u v w) l")
apply (subgoal_tac "\<not> smaller (AllowPortFromTo u v w) (AllowPortFromTo a b c) l")
apply (simp split: if_splits)
apply (rule_tac y = aa and z = list in notSmallerTrans)
apply (simp_all del: smaller.simps)
apply (rule smalleraux3a)
apply (simp_all del: smaller.simps)
apply (case_tac aa, simp_all del: smaller.simps)
apply (case_tac aa, simp_all del: smaller.simps)
apply (case_tac aa, simp_all del: smaller.simps)
apply (rule_tac y = aa in order_trans)
apply (simp_all del: smaller.simps)
apply (subgoal_tac "in_list (AllowPortFromTo u v w) l")
apply simp
apply (rule_tac p = list in in_set_in_list)
apply simp
apply (case_tac aa, simp_all del: smaller.simps)
apply (rule_tac p = list in sorted_is_smaller)
apply (simp_all del: smaller.simps)
apply (subgoal_tac "in_list (AllowPortFromTo u v w) l")
apply simp
apply (rule_tac p = list in in_set_in_list)
apply simp
apply simp
apply (rule_tac y = aa in order_trans)
apply (simp_all del: smaller.simps)
apply (subgoal_tac "in_list (AllowPortFromTo u v w) l")
apply simp
apply (rule_tac p = list in in_set_in_list)
apply simp
apply simp
apply (rule_tac p = list in sorted_is_smaller)
apply (simp_all del: smaller.simps)
apply (subgoal_tac "in_list (AllowPortFromTo u v w) l")
apply simp
apply (rule_tac p = list in in_set_in_list)
apply simp_all
done


lemma NetsCollectedSorted[rule_format]: 
 "noDenyAll1 p \<longrightarrow> all_in_list p l \<longrightarrow> singleCombinators p \<longrightarrow> sorted p l \<longrightarrow> NetsCollected p" 
apply (induct p)
apply simp
apply (rule impI)+
apply (drule mp)
apply (erule noDA1C)
apply (drule mp)
apply simp
apply (drule mp)
apply (erule singleCombinatorsConc)
apply (drule mp)
apply (erule sortedConc)

apply (rule fl3)
apply simp
apply simp
apply (case_tac a)
apply simp_all
apply (metis fMTaux noDA set_empty2)
apply (case_tac aa)
apply simp_all
apply (rule NCSaux1, simp_all)
apply (rule NCSaux2, simp_all)
apply (metis aux0_0)
apply (case_tac "aa")
apply simp_all
apply (rule NCSaux3,simp_all)
apply (rule NCSaux4,simp_all)
apply (metis aux0_0)
done





lemma NetsCollectedSort: "distinct p \<Longrightarrow>noDenyAll1 p \<Longrightarrow> all_in_list p l \<Longrightarrow> singleCombinators p \<Longrightarrow> NetsCollected (sort p l)"
apply (rule_tac l = l in NetsCollectedSorted)
apply (rule noDAsort)
apply simp_all
apply (rule_tac b=p in all_in_listSubset)
apply simp_all
apply (rule sort_is_sorted)
apply simp_all
done


lemma fBNsep[rule_format]: "(\<forall>a\<in>set z. {b,c} \<noteq> first_bothNet a) \<longrightarrow> (\<forall>a\<in>set (separate z). {b,c} \<noteq> first_bothNet a)"
apply (rule separate.induct) back
apply simp
apply (rule impI, simp)+
done


lemma fBNsep1[rule_format]: " (\<forall>a\<in>set z. first_bothNet x \<noteq> first_bothNet a) \<longrightarrow>  (\<forall>a\<in>set (separate z). first_bothNet x \<noteq> first_bothNet a)"
apply (rule separate.induct) back
apply simp
apply (rule impI, simp)+
done


lemma NetsCollectedSepauxa: " \<lbrakk>{b,c} \<noteq> firstList z; noDenyAll1 z; 
                             (\<forall>a\<in>set z. {b,c} \<noteq> first_bothNet a); NetsCollected (z);
                              NetsCollected (separate (z)); {b,c} \<noteq> firstList (separate (z)); 
                              a \<in> set (separate (z))\<rbrakk>
       \<Longrightarrow> {b,c} \<noteq> first_bothNet a"
apply (rule fBNsep)
apply simp_all
done


lemma NetsCollectedSepaux: " \<lbrakk>first_bothNet (x::('a,'b)Combinators) \<noteq> first_bothNet y; \<not> member DenyAll y \<and> noDenyAll z; 
                             (\<forall>a\<in>set z. first_bothNet x \<noteq> first_bothNet a) \<and> NetsCollected (y # z);
                              NetsCollected (separate (y # z)); first_bothNet x \<noteq> firstList (separate (y # z)); 
                              a \<in> set (separate (y # z))\<rbrakk>
       \<Longrightarrow> first_bothNet (x::('a,'b)Combinators) \<noteq> first_bothNet (a::('a,'b)Combinators)"  
apply (rule fBNsep1)
apply simp_all
apply auto
done

lemma NetsCollectedSep[rule_format]: "noDenyAll1 p \<longrightarrow> NetsCollected p \<longrightarrow> NetsCollected (separate p)"
apply (rule separate.induct) back
apply simp_all
apply (metis fMTaux noDA noDA1eq noDAsep set_empty2)
apply (rule conjI|rule impI)+
apply simp
apply (metis fBNsep set_ConsD)
apply (metis noDA1eq noDenyAll.simps(1) set_empty2)
apply (rule conjI|rule impI)+
apply (metis fBNsep mem_def set_ConsD)
apply (metis noDA1eq noDenyAll.simps(1) set_empty2)
apply (rule conjI|rule impI)+
apply simp
apply (metis NetsCollected.simps(1) NetsCollectedSepaux firstList.simps(1) fl2 fl3 noDA1eq noDenyAll.simps(1))
apply (metis noDA1eq noDenyAll.simps(1))
done

lemma OTNaux: "onlyTwoNets a \<Longrightarrow> \<not> member DenyAll a \<Longrightarrow>
               (x,y) \<in> sdnets a \<Longrightarrow> 
        (x = first_srcNet a \<and> y = first_destNet a) \<or> (x = first_destNet a \<and> y = first_srcNet a)"
apply (case_tac "(x = first_srcNet a \<and> y = first_destNet a)")
apply simp_all
apply (simp add: onlyTwoNets_def)
apply (case_tac "(\<exists>aa b. sdnets a = {(aa, b)})")
apply simp_all
apply (subgoal_tac "sdnets a = {(first_srcNet a,first_destNet a)}")
apply simp_all
apply (metis singletonE first_isIn)
apply (subgoal_tac "sdnets a = {(first_srcNet a,first_destNet a),(first_destNet a,first_srcNet a)}")
apply simp_all
apply (rule sdnets2)
apply simp_all
done

lemma sdnets_charn: "onlyTwoNets a \<Longrightarrow> \<not> member DenyAll a \<Longrightarrow>
sdnets a = {(first_srcNet a,first_destNet a)} \<or> sdnets a = {(first_srcNet a, first_destNet a),(first_destNet a, first_srcNet a)}"
apply (case_tac "sdnets a = {(first_srcNet a, first_destNet a)}")  
apply simp_all
apply (simp add: onlyTwoNets_def)
apply (case_tac "(\<exists>aa b. sdnets a = {(aa, b)})")
apply simp_all
apply (metis singletonE first_isIn)
apply (subgoal_tac "sdnets a = {(first_srcNet a,first_destNet a),(first_destNet a,first_srcNet a)}")
apply simp_all
apply (rule sdnets2)
apply simp_all
done

lemma first_bothNet_charn[rule_format]: " \<not> member DenyAll a \<longrightarrow> first_bothNet a = {first_srcNet a, first_destNet a}"
apply (induct a)
apply simp_all
done

lemma sdnets_noteq:"\<lbrakk>onlyTwoNets a; onlyTwoNets aa; first_bothNet a \<noteq> first_bothNet aa; 
                     \<not> member DenyAll a; \<not> member DenyAll aa\<rbrakk>
              \<Longrightarrow> sdnets a \<noteq> sdnets aa"
apply (insert sdnets_charn [of a])
apply (insert sdnets_charn [of aa])
apply (insert first_bothNet_charn [of a])
apply (insert first_bothNet_charn [of aa])
apply simp
apply (metis OTNaux first_bothNetsd first_isIn insert_absorb2 insert_commute)
done


lemma fbn_noteq: " \<lbrakk>onlyTwoNets a; onlyTwoNets aa; first_bothNet a \<noteq> first_bothNet aa; 
                     \<not> member DenyAll a; \<not> member DenyAll aa; allNetsDistinct [a, aa]\<rbrakk>
              \<Longrightarrow> first_srcNet a \<noteq> first_srcNet aa \<or> first_srcNet a \<noteq> first_destNet aa \<or> first_destNet a \<noteq> first_srcNet aa \<or> first_destNet a \<noteq> first_destNet aa" 
apply (insert sdnets_charn [of a])
apply (insert sdnets_charn [of aa])
apply simp
apply (insert sdnets_noteq [of a aa])
apply simp
apply (rule impI)+
apply simp
apply (case_tac " sdnets a = {(first_destNet aa, first_srcNet aa)}")
apply simp_all
apply (case_tac "sdnets aa = {(first_srcNet aa, first_destNet aa)}")
apply simp_all 
done

lemma NCisSD2aux: " \<lbrakk>onlyTwoNets a; onlyTwoNets aa; first_bothNet a \<noteq> first_bothNet aa; 
                     \<not> member DenyAll a; \<not> member DenyAll aa; allNetsDistinct [a, aa]\<rbrakk>
              \<Longrightarrow> disjSD_2 a aa" 
apply (simp add: disjSD_2_def)
apply (rule allI)+
apply (rule impI)
apply (insert sdnets_charn [of a])
apply (insert sdnets_charn [of aa])
apply simp
apply (insert sdnets_noteq [of a aa])
apply (insert fbn_noteq [of a aa])
apply simp
apply (simp add: allNetsDistinct_def twoNetsDistinct_def)
apply (rule conjI)
apply (cases "sdnets a = {(first_srcNet a, first_destNet a)}")
apply (cases "sdnets aa = {(first_srcNet aa, first_destNet aa)}")
apply simp_all
apply (metis firstInNeta firstInNet alternativelistconc2)
apply (case_tac "(c = first_srcNet aa \<and> d = first_destNet aa)")
apply simp_all
apply (case_tac "(first_srcNet a) \<noteq> (first_srcNet aa)")
apply (metis firstInNeta firstInNet alternativelistconc2)
apply simp
apply (subgoal_tac "first_destNet a \<noteq> first_destNet aa")
apply (metis firstInNeta firstInNet alternativelistconc2)
apply (metis first_bothNetsd set_empty2)
apply (case_tac "(first_destNet aa) \<noteq> (first_srcNet a)")
apply (metis firstInNeta firstInNet alternativelistconc2)
apply simp
apply (case_tac "first_destNet aa \<noteq> first_destNet a")
apply simp
apply (subgoal_tac "first_srcNet aa \<noteq> first_destNet a")
apply (metis firstInNeta firstInNet alternativelistconc2)
apply (metis first_bothNetsd insert_commute set_empty2)
apply (metis firstInNeta firstInNet alternativelistconc2)
apply (case_tac "(c = first_srcNet aa \<and> d = first_destNet aa)")
apply simp_all
apply (case_tac "(ab = first_srcNet a \<and> b = first_destNet a)")
apply simp_all
apply (case_tac "(first_srcNet a) \<noteq> (first_srcNet aa)")
apply (metis firstInNeta firstInNet alternativelistconc2)
apply simp
apply (subgoal_tac "first_destNet a \<noteq> first_destNet aa")
apply (metis firstInNeta firstInNet alternativelistconc2)
apply (metis first_bothNetsd set_empty2)
apply (case_tac "(first_destNet aa) \<noteq> (first_srcNet a)")
apply (metis firstInNeta firstInNet alternativelistconc2)
apply simp
apply (case_tac "first_destNet aa \<noteq> first_destNet a")
apply simp
apply (subgoal_tac "first_srcNet aa \<noteq> first_destNet a")
apply (metis firstInNeta firstInNet alternativelistconc2)
apply (metis first_bothNetsd insert_commute set_empty2)
apply (metis firstInNeta firstInNet alternativelistconc2)
apply (case_tac "(ab = first_srcNet a \<and> b = first_destNet a)")
apply simp_all
apply (case_tac "c = first_srcNet aa")
apply simp_all
apply (metis OTNaux)
apply (subgoal_tac "c = first_destNet aa")
apply simp
apply (subgoal_tac "d = first_srcNet aa")
apply simp
apply (case_tac "(first_srcNet a) \<noteq> (first_destNet aa)")
apply (metis firstInNeta firstInNet alternativelistconc2 alternativelistconc1)
apply simp
apply (subgoal_tac "first_destNet a \<noteq> first_srcNet aa")
apply (metis firstInNeta firstInNet alternativelistconc2)
apply (metis first_bothNetsd set_empty2 insert_commute)
apply (metis OTNaux)
apply (metis OTNaux)
apply (case_tac "c = first_srcNet aa")
apply simp_all
apply (metis OTNaux)
apply (subgoal_tac "c = first_destNet aa")
apply simp
apply (subgoal_tac "d = first_srcNet aa")
apply simp
apply (case_tac "(first_destNet a) \<noteq> (first_destNet aa)")
apply (metis firstInNeta firstInNet alternativelistconc2 alternativelistconc1)
apply simp
apply (subgoal_tac "first_srcNet a \<noteq> first_srcNet aa")
apply (metis firstInNeta firstInNet alternativelistconc2)
apply (metis first_bothNetsd set_empty2 insert_commute)
apply (metis OTNaux)
apply (metis OTNaux)
apply (cases "sdnets a = {(first_srcNet a, first_destNet a)}")
apply (cases "sdnets aa = {(first_srcNet aa, first_destNet aa)}")
apply simp_all
apply (case_tac "(c = first_srcNet aa \<and> d = first_destNet aa)")
apply simp_all
apply (case_tac "(first_srcNet a) \<noteq> (first_destNet aa)")
apply simp_all
apply (metis firstInNeta firstInNet alternativelistconc2 alternativelistconc1)
apply (subgoal_tac "first_destNet a \<noteq> first_srcNet aa")
apply (metis firstInNeta firstInNet alternativelistconc2 alternativelistconc1)
apply (metis first_bothNetsd set_empty2 insert_commute)
apply (case_tac "(c = first_srcNet aa \<and> d = first_destNet aa)")
apply simp_all
apply (case_tac "(ab = first_srcNet a \<and> b = first_destNet a)")
apply simp_all
apply (case_tac "(first_destNet a) \<noteq> (first_srcNet aa)")
apply (metis firstInNeta firstInNet alternativelistconc2)
apply simp
apply (subgoal_tac "first_srcNet a \<noteq> first_destNet aa")
apply (metis firstInNeta firstInNet alternativelistconc2)
apply (metis first_bothNetsd set_empty2 insert_commute)
apply (case_tac "(first_srcNet aa) \<noteq> (first_srcNet a)")
apply (metis firstInNeta firstInNet alternativelistconc2)
apply simp
apply (case_tac "first_destNet aa \<noteq> first_destNet a")
apply (metis firstInNeta firstInNet alternativelistconc2 alternativelistconc1)
apply simp
apply (metis first_bothNetsd set_empty2)
apply (cases "sdnets aa = {(first_srcNet aa, first_destNet aa)}")
apply simp_all
apply (case_tac "(c = first_srcNet aa \<and> d = first_destNet aa)")
apply simp_all
apply (case_tac "(ab = first_srcNet a \<and> b = first_destNet a)")
apply simp_all
apply (case_tac "(first_destNet a) \<noteq> (first_srcNet aa)")
apply (metis firstInNeta firstInNet alternativelistconc2)
apply simp
apply (subgoal_tac "first_srcNet a \<noteq> first_destNet aa")
apply (metis firstInNeta firstInNet alternativelistconc2)
apply (metis first_bothNetsd set_empty2 insert_commute)
apply (case_tac "(first_srcNet aa) \<noteq> (first_srcNet a)")
apply (metis firstInNeta firstInNet alternativelistconc2)
apply simp
apply (case_tac "first_destNet aa \<noteq> first_destNet a")
apply (metis firstInNeta firstInNet alternativelistconc2 alternativelistconc1)
apply simp
apply (metis first_bothNetsd set_empty2)
apply (case_tac "(c = first_srcNet aa \<and> d = first_destNet aa)")
apply simp_all
apply (case_tac "(ab = first_srcNet a \<and> b = first_destNet a)")
apply simp_all
apply (case_tac "(first_destNet a) \<noteq> (first_srcNet aa)")
apply (metis firstInNeta firstInNet alternativelistconc2)
apply simp
apply (subgoal_tac "first_srcNet a \<noteq> first_destNet aa")
apply (metis firstInNeta firstInNet alternativelistconc2)
apply (metis first_bothNetsd set_empty2 insert_commute)
apply (case_tac "(first_srcNet aa) \<noteq> (first_srcNet a)")
apply (metis firstInNeta firstInNet alternativelistconc2)
apply simp
apply (case_tac "first_destNet aa \<noteq> first_destNet a")
apply (metis firstInNeta firstInNet alternativelistconc2 alternativelistconc1)
apply simp
apply (case_tac "(ab = first_srcNet a \<and> b = first_destNet a)")
apply simp_all
apply (case_tac "(first_destNet a) \<noteq> (first_srcNet aa)")
apply (metis firstInNeta firstInNet alternativelistconc2)
apply simp
apply (subgoal_tac "first_srcNet a \<noteq> first_srcNet aa")
apply (metis firstInNeta firstInNet alternativelistconc2)
apply (metis first_bothNetsd set_empty2 insert_commute)
apply (case_tac "(first_srcNet aa) \<noteq> (first_destNet a)")
apply (metis firstInNeta firstInNet alternativelistconc2)
apply simp
apply (case_tac "first_destNet aa \<noteq> first_srcNet a")
apply (metis firstInNeta firstInNet alternativelistconc2 alternativelistconc1)
apply simp
apply (metis insert_commute set_empty2)
done

lemma ANDaux3[rule_format]: "y \<in> set xs \<longrightarrow> a \<in> set (net_list_aux [y]) \<longrightarrow> a \<in> set (net_list_aux xs)"
apply (induct xs)
apply simp_all
apply (rule conjI)
apply (rule impI)+
apply simp
apply (metis isInAlternativeList)
apply (rule impI)+
apply simp
apply (erule isInAlternativeListb)
done

lemma ANDaux2: "allNetsDistinct (x # xs) \<Longrightarrow> y \<in> set xs  
              \<Longrightarrow> allNetsDistinct [x,y]"
apply (simp add: allNetsDistinct_def)
apply (rule allI)
apply (rule allI)
apply (rule impI)+
apply (drule_tac x = a in spec)
apply (drule_tac x = b in spec)
apply simp
apply (drule mp)
apply simp_all
apply (rule conjI)
apply (case_tac "a \<in> set (net_list_aux [x])")
apply (erule isInAlternativeLista)
apply (rule isInAlternativeListb)
apply (rule ANDaux3)
apply simp_all
apply (metis netlistaux)
apply (case_tac "b \<in> set (net_list_aux [x])")
apply (erule isInAlternativeLista)
apply (rule isInAlternativeListb)
apply (rule ANDaux3)
apply simp_all
apply (metis netlistaux)
done
 
lemma NCisSD2[rule_format]: "
\<lbrakk>\<not> member DenyAll a; OnlyTwoNets (a#p); NetsCollected2 (a # p); NetsCollected (a#p);noDenyAll ( p); allNetsDistinct (a # p); s \<in> set p\<rbrakk> \<Longrightarrow>
 disjSD_2 a s"
apply (case_tac p)
apply simp_all
apply (rule NCisSD2aux)
apply simp_all 
apply (rule OTNoTN)
apply simp 
apply (case_tac a, simp_all)
apply (rule OTNoTN)
apply simp
apply (metis FWCompilation.member.simps(2) noDA)
apply simp
apply metis
apply (metis noDA)
apply (rule ANDaux2)
apply simp_all
apply simp
done

lemma separatedNC[rule_format]: "OnlyTwoNets p \<longrightarrow> NetsCollected2 p \<longrightarrow> NetsCollected p \<longrightarrow> noDenyAll1 p \<longrightarrow>  allNetsDistinct p  \<longrightarrow> separated p"
apply (induct p)
apply simp_all
apply (case_tac "a = DenyAll")
apply simp_all
defer 1
apply (rule impI)+
apply (drule mp)
apply (erule OTNConc)
apply (drule mp)
apply (case_tac p, simp_all)
apply (drule mp)
apply (erule noDA1C)
apply (rule conjI)
apply (rule allI)
apply (rule impI) 
apply (rule NCisSD2)
apply simp_all
apply (case_tac a, simp_all)
apply (case_tac a, simp_all)
apply (drule mp)
apply (erule ANDConc)
apply simp
apply (rule impI)+
apply (simp)
apply (drule mp)
apply (erule noDA1eq)
apply (drule mp)
apply (erule ANDConc)
apply simp
apply (simp add: disjSD_2_def)
done

lemma NC2Sep[rule_format]: "noDenyAll1 p \<longrightarrow> NetsCollected2 (separate p)"
apply (rule separate.induct) back
apply simp_all
apply (rule impI, drule mp)
apply (erule noDA1eq)
apply (case_tac "separate x = []")
apply simp_all
apply (case_tac x, simp_all)
apply (metis fMTaux firstList.simps(1) fl2 set_empty2)
apply (rule impI)+
apply simp
apply (drule mp)
apply (rule noDA1eq)
apply (case_tac y, simp_all)
apply (metis firstList.simps(1) fl2)
apply (rule impI)+
apply simp
apply (drule mp)
apply (rule noDA1eq)
apply (case_tac y, simp_all)
apply (metis firstList.simps(1) fl2)
apply (rule impI)+
apply simp
apply (drule mp)
apply (rule noDA1eq)
apply (case_tac y, simp_all)
apply (metis firstList.simps(1) fl2)
done

lemma separatedSep[rule_format]: "OnlyTwoNets p \<longrightarrow> NetsCollected2 p \<longrightarrow> NetsCollected p \<longrightarrow> noDenyAll1 p \<longrightarrow> allNetsDistinct p  \<longrightarrow> 
   separated (separate p)"
apply (rule impI)+
apply (rule separatedNC)
apply (rule OTNSEp)
apply simp_all
apply (erule NC2Sep)
apply (erule NetsCollectedSep)
apply simp
apply (erule noDA1sep)
apply (erule ANDSep)
done

lemmas CLemmas = noneMTsep nMTSort noneMTRS2 noneMTrd nMTRS3 separatedSep noDAsort nDASC wp1_eq WP1rd wp1ID SC2 SCrd SCRS3 SCRiD SC1 aux0 aND_sort SC2 SCrd aND_RS2 ANDRS3 wellformed1_sorted wp1ID ANDiD  ANDrd SC1 aND_RS1 SC3 ANDSep OTNSEp OTNSC noDA1sep wp1_alternativesep wellformed1_alternative_sorted  distinct_RS2 

lemmas C_eqLemmas_id = C_eq_Lemmas_sep CLemmas OTNSEp NC2Sep NetsCollectedSep NetsCollectedSort separatedNC

lemma C_eq_Until_InsertDenies: "\<lbrakk>DenyAll \<in> set (policy2list p); all_in_list (policy2list p) l; allNetsDistinct (policy2list p)\<rbrakk> \<Longrightarrow> 

C (list2policy ((insertDenies (separate (sort (removeShadowRules2 (remdups (removeShadowRules3 (insertDeny (removeShadowRules1 (policy2list p)))))) l))))) = C p"
apply (subst C_eq_iD)
apply (simp_all add: C_eqLemmas_id)
apply (rule C_eq_until_separated)
apply simp_all
done

lemma rADnMT[rule_format]: "p \<noteq> []  \<longrightarrow> removeAllDuplicates p \<noteq> []"
apply (induct p)
apply simp_all
done

lemma C_eq_RD_aux[rule_format]: "C (p) x = C (removeDuplicates p) x" 
apply (induct p)
apply simp_all
apply (rule conjI, rule impI)
apply (metis Cdom2 domIff nlpaux not_in_member)
apply (metis C.simps(4) CConcStartaux Cdom2 domIff)
done

lemma C_eq_RAD_aux[rule_format]: "p \<noteq> []  \<longrightarrow> C (list2policy p) x = C (list2policy (removeAllDuplicates p)) x"
apply (induct p)
apply simp_all
apply (case_tac "p = []")
apply simp_all
apply (metis C_eq_RD_aux)
apply (subst list2policyconc)
apply simp
apply (case_tac "x \<in> dom (C (list2policy p))")
apply (subst list2policyconc)
apply (rule rADnMT)
apply simp
apply (subst Cdom2)
apply simp
apply (drule sym)
apply (subst Cdom2)
apply (simp add: dom_def)
apply simp
apply (drule sym)
apply (subst nlpaux)
apply simp
apply (subst list2policyconc)
apply (rule rADnMT)
apply simp
apply (subst nlpaux)
apply (simp add: dom_def)
apply (rule C_eq_RD_aux)
done

lemma C_eq_RAD: "p \<noteq> []  \<Longrightarrow> C (list2policy p) = C (list2policy (removeAllDuplicates p)) "
apply (rule ext)
apply (erule C_eq_RAD_aux)
done


lemma C_eq_compile: 
"\<lbrakk>DenyAll \<in> set (policy2list p); all_in_list (policy2list p) l; allNetsDistinct (policy2list p)\<rbrakk> 
\<Longrightarrow> 
C (list2policy (removeAllDuplicates (insertDenies (separate (sort (removeShadowRules2 (remdups (removeShadowRules3 (insertDeny (removeShadowRules1 (policy2list p)))))) l))))) = C p"
apply (subst C_eq_RAD[symmetric])
apply (rule idNMT)
apply (simp add: C_eqLemmas_id)
apply (rule C_eq_Until_InsertDenies) 
apply simp_all
done



end
